@@ -25,7 +25,6 @@
 #include <qsplitter.h>
 #include <qlayout.h>
 #include <qstatusbar.h>
-#include <qtabbar.h>
 #include <qprogressbar.h>
 #include <qwidgetstack.h>
 #include <qtimer.h>
@@ -38,30 +37,15 @@
 
 #ifdef WIN32
 #include <windows.h>
+#else
+#ifdef USE_KDE
+#include "kdeisversion.h"
+#include <kwin.h>
+#endif
 #endif
 
 const unsigned ACCEL_MESSAGE = 0x1000;
 
-class UserTabBar : public QTabBar
-{
-public:
-    UserTabBar(QWidget *parent);
-    void raiseTab(unsigned id);
-    UserWnd *wnd(unsigned id);
-    UserWnd *currentWnd();
-    list<UserWnd*> windows();
-    void removeTab(unsigned id);
-    void changeTab(unsigned id);
-    void setBold(unsigned id, bool bState);
-    void setCurrent(unsigned i);
-    unsigned current();
-    bool isBold(UserWnd *wnd);
-protected:
-    virtual void layoutTabs();
-    virtual void mousePressEvent(QMouseEvent *e);
-    virtual void paintLabel(QPainter *p, const QRect &rc, QTab *t, bool bFocus) const;
-};
-
 class UserTab : public QTab
 {
 public:
@@ -126,10 +110,14 @@ Container::Container(unsigned id, const char *cfg)
     m_bBarChanged = false;
     m_bReceived = false;
     m_bNoSwitch = false;
+    m_bNoRead   = false;
+#ifdef WIN32
+    m_bInitWnd	= false;
+#endif
 
     SET_WNDPROC("container")
     setWFlags(WDestructiveClose);
-    QFrame *frm = new QFrame(this);
+    QFrame *frm = new QFrame(this, "container");
     setCentralWidget(frm);
 
     connect(CorePlugin::m_plugin, SIGNAL(modeChanged()), this, SLOT(modeChanged()));
@@ -155,13 +143,13 @@ Container::Container(unsigned id, const char *cfg)
         setId(id);
         memcpy(data.barState, CorePlugin::m_plugin->data.containerBar, sizeof(data.barState));
         memcpy(data.geometry, CorePlugin::m_plugin->data.containerGeo, sizeof(data.geometry));
-        if ((data.geometry[WIDTH] == -1) || (data.geometry[HEIGHT] == -1)){
+        if ((data.geometry[WIDTH].value == (unsigned long)-1) || (data.geometry[HEIGHT].value == (unsigned long)-1)){
             QWidget *desktop = QApplication::desktop();
-            data.geometry[WIDTH] = desktop->width() / 3;
-            data.geometry[HEIGHT] = desktop->height() / 3;
+            data.geometry[WIDTH].value = desktop->width() / 3;
+            data.geometry[HEIGHT].value = desktop->height() / 3;
         }
         bPos = false;
-        if ((data.geometry[TOP] != -1) || (data.geometry[LEFT] != -1)){
+        if ((data.geometry[TOP].value != (unsigned long)-1) || (data.geometry[LEFT].value != (unsigned long)-1)){
             bPos = true;
             QWidgetList  *list = QApplication::topLevelWidgets();
             for (int i = 0; i < 2; i++){
@@ -174,23 +162,23 @@ Container::Container(unsigned id, const char *cfg)
                         continue;
                     }
                     if (w->inherits("Container")){
-                        int dw = w->pos().x() - data.geometry[LEFT];
-                        int dh = w->pos().y() - data.geometry[TOP];
+                        int dw = w->pos().x() - data.geometry[LEFT].value;
+                        int dh = w->pos().y() - data.geometry[TOP].value;
                         if (dw < 0) dw = -dw;
                         if (dh < 0) dh = -dh;
                         if ((dw < 3) && (dh < 3)){
-                            int nl = data.geometry[LEFT];
-                            int nt = data.geometry[TOP];
+                            int nl = data.geometry[LEFT].value;
+                            int nt = data.geometry[TOP].value;
                             nl += 21;
                             nt += 20;
                             QWidget *desktop = QApplication::desktop();
-                            if (nl + data.geometry[WIDTH] > desktop->width())
+                            if (nl + (int)data.geometry[WIDTH].value > desktop->width())
                                 nl = 0;
-                            if (nt + data.geometry[WIDTH] > desktop->width())
+                            if (nt + (int)data.geometry[WIDTH].value > desktop->width())
                                 nt = 0;
-                            if ((nl != data.geometry[LEFT]) && (nt != data.geometry[TOP])){
-                                data.geometry[LEFT] = nl;
-                                data.geometry[TOP]  = nt;
+                            if ((nl != (int)data.geometry[LEFT].value) && (nt != (int)data.geometry[TOP].value)){
+                                data.geometry[LEFT].value = nl;
+                                data.geometry[TOP].value  = nt;
                                 bOK = false;
                             }
                         }
@@ -256,9 +244,9 @@ void Container::setupAccel()
     }
 }
 
-void Container::setNoSwitch()
+void Container::setNoSwitch(bool bState)
 {
-    m_bNoSwitch = true;
+    m_bNoSwitch = bState;
 }
 
 list<UserWnd*> Container::windows()
@@ -386,7 +374,7 @@ void Container::init()
         Contact *contact = getContacts()->contact(id);
         if (contact == NULL)
             continue;
-        addUserWnd(new UserWnd(id, getWndConfig(id), false), true);
+        addUserWnd(new UserWnd(id, getWndConfig(id), false, true), true);
     }
     if (m_tabBar->count() == 0)
         QTimer::singleShot(0, this, SLOT(close()));
@@ -403,6 +391,7 @@ void Container::contactSelected(int)
     if (userWnd == NULL)
         return;
     m_wnds->raiseWidget(userWnd);
+    userWnd->setFocus();
     m_bar->setParam((void*)userWnd->id());
     QString name = userWnd->getName();
     Command cmd;
@@ -416,6 +405,7 @@ void Container::contactSelected(int)
     cmd->flags = BTN_PICT;
     Event e(EventCommandChange, cmd);
     m_bar->processEvent(&e);
+    setMessageType(userWnd->type());
     setIcon(Pict(cmd->icon));
     setCaption(userWnd->getLongName());
     m_bar->checkState();
@@ -542,10 +532,47 @@ typedef struct FLASHWINFO
     unsigned long dwTimeout;
 } FLASHWINFO;
 
+#ifndef FLASHW_TRAY
+#define FLASHW_TRAY         0x00000002
+#define FLASHW_TIMERNOFG    0x0000000C
+#endif
+
 static BOOL (WINAPI *FlashWindowEx)(FLASHWINFO*) = NULL;
 static bool initFlash = false;
 #endif
 
+#if 0
+i18n("male", "%1 is typing")
+i18n("female", "%1 is typing")
+#endif
+
+void Container::flash()
+{
+#ifdef WIN32
+    if (!initFlash){
+        HINSTANCE hLib = GetModuleHandleA("user32");
+        if (hLib != NULL)
+            (DWORD&)FlashWindowEx = (DWORD)GetProcAddress(hLib,"FlashWindowEx");
+        initFlash = true;
+    }
+    if (FlashWindowEx){
+        FLASHWINFO fInfo;
+        fInfo.cbSize  = sizeof(fInfo);
+        fInfo.dwFlags = FLASHW_TRAY | FLASHW_TIMERNOFG;
+        fInfo.hwnd = winId();
+        fInfo.uCount = 0;
+        fInfo.dwTimeout = 1000;
+        FlashWindowEx(&fInfo);
+    }
+#else
+#if defined(USE_KDE)
+#if KDE_IS_VERSION(3,2,0)
+    KWin::demandAttention(winId(), true);
+#endif	/* KDE_IS_VERSION(3,2,0) */
+#endif	/* USE_KDE */
+#endif	/* ndef WIN32 */
+}
+
 void *Container::processEvent(Event *e)
 {
     UserWnd *userWnd;
@@ -561,34 +588,19 @@ void *Container::processEvent(Event *e)
                 contactChanged(contact);
             return NULL;
         }
+        if (msg->getFlags() & MESSAGE_NOVIEW)
+            return NULL;
         if (CorePlugin::m_plugin->getContainerMode()){
-            if (isActiveWindow()){
+            if (isActiveWindow() && !isMinimized()){
                 userWnd = m_tabBar->currentWnd();
                 if (userWnd && (userWnd->id() == msg->contact()))
                     userWnd->markAsRead();
-            }
-#ifdef WIN32
-            if (!isActiveWindow()){
+            }else{
                 msg = (Message*)(e->param());
                 userWnd = wnd(msg->contact());
-                if (userWnd){
-                    if (!initFlash){
-                        HINSTANCE hLib = GetModuleHandleA("user32");
-                        if (hLib != NULL)
-                            (DWORD&)FlashWindowEx = (DWORD)GetProcAddress(hLib,"FlashWindowEx");
-                        initFlash = true;
-                    }
-                    if (FlashWindowEx){
-                        FLASHWINFO fInfo;
-                        fInfo.cbSize = sizeof(fInfo);
-                        fInfo.dwFlags = 0x0E;
-                        fInfo.hwnd = winId();
-                        fInfo.uCount = 0;
-                        FlashWindowEx(&fInfo);
-                    }
-                }
+                if (userWnd)
+                    QTimer::singleShot(0, this, SLOT(flash()));
             }
-#endif
         }
     case EventMessageRead:
         msg = (Message*)(e->param());
@@ -649,7 +661,7 @@ void *Container::processEvent(Event *e)
             if (userWnd->m_bTyping != bTyping){
                 userWnd->m_bTyping = bTyping;
                 if (bTyping){
-                    userWnd->setStatus(i18n("Contact typed message"));
+                    userWnd->setStatus(g_i18n("%1 is typing", contact) .arg(contact->getName()));
                 }else{
                     userWnd->setStatus("");
                 }
@@ -743,9 +755,18 @@ void Container::wndClosed()
 
 bool Container::event(QEvent *e)
 {
-    if (e->type() == QEvent::WindowActivate){
+#ifdef WIN32
+    if ((e->type() == QEvent::WindowActivate) && !m_bInitWnd){
+        m_bInitWnd = true;
+        m_bNoRead  = false;
+        show();
+    }
+#endif
+    if ((e->type() == QEvent::WindowActivate) ||
+            (((e->type() == QEvent::ShowNormal) || (e->type() == QEvent::ShowMaximized)) &&
+             isActiveWindow())){
         UserWnd *userWnd = m_tabBar->currentWnd();
-        if (userWnd)
+        if (userWnd && !m_bNoRead)
             userWnd->markAsRead();
         if (m_bNoSwitch){
             m_bNoSwitch = false;
@@ -829,10 +850,12 @@ list<UserWnd*> UserTabBar::windows()
 void UserTabBar::setCurrent(unsigned n)
 {
     n++;
-    for (unsigned i = 0; (i < (unsigned)count()) && (n > 0); i++){
+    unsigned m = 0;
+    for (unsigned i = 0; (m < (unsigned)count()) && (n > 0); i++){
         QTab *t = tab(i);
         if (t == NULL)
             continue;
+        m++;
         if (--n == 0){
             setCurrentTab(t);
         }
@@ -850,6 +873,11 @@ unsigned UserTabBar::current()
     return n;
 }
 
+void UserTabBar::slotRepaint()
+{
+    repaint();
+}
+
 void UserTabBar::removeTab(unsigned id)
 {
     layoutTabs();
@@ -860,6 +888,7 @@ void UserTabBar::removeTab(unsigned id)
             continue;
         if (tab->wnd()->id() == id){
             QTabBar::removeTab(tab);
+            QTimer::singleShot(0, this, SLOT(slotRepaint()));
             break;
         }
     }
@@ -873,6 +902,7 @@ void UserTabBar::changeTab(unsigned id)
         UserTab *tab = static_cast<UserTab*>(t);
         if (tab->wnd()->id() == id){
             tab->setText(tab->wnd()->getName());
+            QTimer::singleShot(0, this, SLOT(slotRepaint()));
             break;
         }
     }
@@ -913,6 +943,12 @@ bool UserTabBar::isBold(UserWnd *wnd)
     return false;
 }
 
+void UserTabBar::resizeEvent(QResizeEvent *e)
+{
+    QTabBar::resizeEvent(e);
+    QTimer::singleShot(0, this, SLOT(slotRepaint()));
+}
+
 void UserTabBar::mousePressEvent(QMouseEvent *e)
 {
     if (e->button() == RightButton){
@@ -943,7 +979,7 @@ UserWnd *UserTabBar::currentWnd()
 void UserTabBar::layoutTabs()
 {
     QTabBar::layoutTabs();
-#if QT_VERSION < 300
+#if COMPAT_QT_VERSION < 0x030000
     QList<QTab> *tList = tabList();
     for (QTab *t = tList->first(); t; t = tList->next()){
         t->r.setHeight(height());
