@@ -17,7 +17,7 @@
 
 #include "simapi.h"
 
-#if QT_VERSION < 300
+#if (COMPAT_QT_VERSION < 0x030000) || ((COMPAT_QT_VERSION >= 0x030000) && !(defined(HAVE_QSYNTAXHIGHLIGHTER_H)))
 #include "qt3/qsyntaxhighlighter.h"
 #else
 #include <qsyntaxhighlighter.h>
@@ -45,6 +45,12 @@
 #include <qtabwidget.h>
 #include <qspinbox.h>
 
+#ifdef USE_KDE
+#include <qapplication.h>
+#include <kglobal.h>
+#include <kstddirs.h>
+#endif
+
 #ifdef WIN32
 static char STYLES[] = "styles\\";
 #else
@@ -54,6 +60,7 @@ static char EXT[]    = ".xsl";
 
 #undef QTextEdit
 
+
 class XmlHighlighter : public QSyntaxHighlighter
 {
 public:
@@ -173,6 +180,7 @@ int XmlHighlighter::highlightParagraph(const QString &s, int state)
     return state;
 }
 
+
 HistoryConfig::HistoryConfig(QWidget *parent)
         : HistoryConfigBase(parent)
 {
@@ -200,7 +208,17 @@ HistoryConfig::HistoryConfig(QWidget *parent)
     new XmlHighlighter(edtStyle);
     QStringList styles;
     addStyles(user_file(STYLES).c_str(), true);
+#ifdef USE_KDE
+    QStringList lst = KGlobal::dirs()->findDirs("data", "sim");
+    for (QStringList::Iterator it = lst.begin(); it != lst.end(); ++it){
+        QFile fi(*it + STYLES);
+        if (!fi.exists())
+            continue;
+        addStyles(QFile::encodeName(fi.name()), false);
+    }
+#else
     addStyles(app_file(STYLES).c_str(), false);
+#endif
     fillCombo(CorePlugin::m_plugin->getHistoryStyle());
     connect(cmbStyle, SIGNAL(activated(int)), this, SLOT(styleSelected(int)));
     connect(btnCopy, SIGNAL(clicked()), this, SLOT(copy()));
@@ -213,10 +231,10 @@ HistoryConfig::HistoryConfig(QWidget *parent)
     connect(chkDays, SIGNAL(toggled(bool)), this, SLOT(toggledDays(bool)));
     connect(chkSize, SIGNAL(toggled(bool)), this, SLOT(toggledSize(bool)));
     HistoryUserData *data = (HistoryUserData*)(getContacts()->getUserData(CorePlugin::m_plugin->history_data_id));
-    chkDays->setChecked(data->CutDays != 0);
-    chkSize->setChecked(data->CutSize != 0);
-    edtDays->setValue(data->Days);
-    edtSize->setValue(data->MaxSize);
+    chkDays->setChecked(data->CutDays.bValue);
+    chkSize->setChecked(data->CutSize.bValue);
+    edtDays->setValue(data->Days.value);
+    edtSize->setValue(data->MaxSize.value);
     toggledDays(chkDays->isChecked());
     toggledSize(chkSize->isChecked());
 }
@@ -230,6 +248,11 @@ static char BACKUP_SUFFIX[] = "~";
 void HistoryConfig::apply()
 {
     bool bChanged = false;
+    if (tabStyle->currentPage() == source){
+        int cur = cmbStyle->currentItem();
+        if (m_bDirty && (cur >= 0))
+            m_styles[cur].text = unquoteText(edtStyle->text());
+    }
     for (unsigned i = 0; i < m_styles.size(); i++){
         if (m_styles[i].text.isEmpty() || !m_styles[i].bCustom)
             continue;
@@ -246,10 +269,10 @@ void HistoryConfig::apply()
             f.writeBlock(s.c_str(), s.length());
 
             const int status = f.status();
-#if QT_VERSION >= 0x030200
+#if COMPAT_QT_VERSION >= 0x030200
             const QString errorMessage = f.errorString();
 #else
-	    const QString errorMessage = "write file fail";
+            const QString errorMessage = "write file fail";
 #endif
             f.close();
             if (status != IO_Ok) {
@@ -257,7 +280,11 @@ void HistoryConfig::apply()
             } else {
                 // rename to normal file
                 QFileInfo fileInfo(f.name());
-                QString desiredFileName = QFile::decodeName(name.c_str());
+                QString desiredFileName = fileInfo.fileName();
+                desiredFileName = desiredFileName.left(desiredFileName.length() - strlen(BACKUP_SUFFIX));
+#ifdef WIN32
+                fileInfo.dir().remove(desiredFileName);
+#endif
                 if (!fileInfo.dir().rename(fileInfo.fileName(), desiredFileName)) {
                     log(L_ERROR, "Can't rename file %s to %s", (const char*)fileInfo.fileName().local8Bit(), (const char*)desiredFileName.local8Bit());
                 }
@@ -267,12 +294,14 @@ void HistoryConfig::apply()
         }
     }
     int cur = cmbStyle->currentItem();
-    if ((cur >= 0) && (m_styles[cur].name != QFile::decodeName(CorePlugin::m_plugin->getHistoryStyle()))){
+    if ((cur >= 0) && m_styles.size() &&
+            (m_styles[cur].bChanged ||
+             (m_styles[cur].name != QFile::decodeName(CorePlugin::m_plugin->getHistoryStyle())))){
         CorePlugin::m_plugin->setHistoryStyle(QFile::encodeName(m_styles[cur].name));
         bChanged = true;
+        delete CorePlugin::m_plugin->historyXSL;
+        CorePlugin::m_plugin->historyXSL = new XSL(m_styles[cur].name);
     }
-    delete CorePlugin::m_plugin->historyXSL;
-    CorePlugin::m_plugin->historyXSL = new XSL(m_styles[cur].name);
 
     if (chkOwn->isChecked() != CorePlugin::m_plugin->getOwnColors()){
         bChanged = true;
@@ -289,10 +318,10 @@ void HistoryConfig::apply()
         fillPreview();
     }
     HistoryUserData *data = (HistoryUserData*)(getContacts()->getUserData(CorePlugin::m_plugin->history_data_id));
-    data->CutDays = chkDays->isChecked();
-    data->CutSize = chkSize->isChecked();
-    data->Days    = atol(edtDays->text());
-    data->MaxSize = atol(edtSize->text());
+    data->CutDays.bValue = chkDays->isChecked();
+    data->CutSize.bValue = chkSize->isChecked();
+    data->Days.value	 = atol(edtDays->text());
+    data->MaxSize.value  = atol(edtSize->text());
 }
 
 void HistoryConfig::addStyles(const char *dir, bool bCustom)
@@ -310,8 +339,9 @@ void HistoryConfig::addStyles(const char *dir, bool bCustom)
         }
         if (its == m_styles.end()){
             StyleDef s;
-            s.name    = name;
-            s.bCustom = bCustom;
+            s.name     = name;
+            s.bCustom  = bCustom;
+            s.bChanged = false;
             m_styles.push_back(s);
         }
     }
@@ -336,12 +366,14 @@ void HistoryConfig::styleSelected(int n)
     btnDelete->setEnabled(bCustom);
     edtStyle->setReadOnly(!bCustom);
     fillPreview();
+    if (tabStyle->currentPage() == source)
+        viewChanged(source);
 }
 
 void HistoryConfig::copy()
 {
     int cur = cmbStyle->currentItem();
-    if (cur < 0)
+    if ((cur < 0) || (!m_styles.size()))
         return;
     QString name    = m_styles[cur].name;
     QString newName;
@@ -393,10 +425,10 @@ void HistoryConfig::copy()
     from.close();
 
     const int status = to.status();
-#if QT_VERSION >= 0x030200
+#if COMPAT_QT_VERSION >= 0x030200
     const QString errorMessage = to.errorString();
 #else
-    const QString errorMessage = "write file fail";
+const QString errorMessage = "write file fail";
 #endif
     to.close();
     if (status != IO_Ok) {
@@ -406,7 +438,11 @@ void HistoryConfig::copy()
 
     // rename to normal file
     QFileInfo fileInfo(to.name());
-    QString desiredFileName = QFile::decodeName(n.c_str());
+    QString desiredFileName = fileInfo.fileName();
+    desiredFileName = desiredFileName.left(desiredFileName.length() - strlen(BACKUP_SUFFIX));
+#ifdef WIN32
+    fileInfo.dir().remove(desiredFileName);
+#endif
     if (!fileInfo.dir().rename(fileInfo.fileName(), desiredFileName)) {
         log(L_ERROR, "Can't rename file %s to %s", (const char*)fileInfo.fileName().local8Bit(), (const char*)desiredFileName.local8Bit());
         return;
@@ -438,6 +474,8 @@ void HistoryConfig::fillCombo(const char *current)
 void HistoryConfig::del()
 {
     int cur = cmbStyle->currentItem();
+    if ((cur < 0) || (!m_styles.size()))
+        return;
     if (!m_styles[cur].bCustom)
         return;
     BalloonMsg::ask(NULL, i18n("Remove style '%1'?") .arg(m_styles[cur].name),
@@ -447,7 +485,7 @@ void HistoryConfig::del()
 void HistoryConfig::realDelete()
 {
     int cur = cmbStyle->currentItem();
-    if (cur < 0)
+    if ((cur < 0) || (!m_styles.size()))
         return;
     if (!m_styles[cur].bCustom)
         return;
@@ -469,8 +507,11 @@ void HistoryConfig::realDelete()
 void HistoryConfig::rename()
 {
     int cur = cmbStyle->currentItem();
+    if ((cur < 0) || (!m_styles.size()))
+        return;
     if (!m_styles[cur].bCustom)
         return;
+    m_edit = cur;
     cmbStyle->setEditable(true);
     cmbStyle->lineEdit()->setText(m_styles[cur].name);
     cmbStyle->lineEdit()->setFocus();
@@ -485,40 +526,36 @@ void HistoryConfig::cancelRename()
 
 void HistoryConfig::realRename()
 {
-    int cur = cmbStyle->currentItem();
-    if (cur < 0)
-        return;
     QString newName = cmbStyle->lineEdit()->text();
     cmbStyle->lineEdit()->removeEventFilter(this);
     cmbStyle->setEditable(false);
-    if (newName != m_styles[cur].name){
+    if (newName != m_styles[m_edit].name){
         int n = 0;
         vector<StyleDef>::iterator it;
         for (it = m_styles.begin(); it != m_styles.end(); ++it, n++){
             if ((*it).name == newName){
-                if (n < cur)
-                    cur--;
+                if (n < m_edit)
+                    m_edit--;
                 m_styles.erase(it);
                 break;
             }
         }
-
         string nn;
         nn = STYLES;
-        nn += QFile::encodeName(m_styles[cur].name);
+        nn += QFile::encodeName(m_styles[m_edit].name);
         nn += EXT;
         nn = user_file(nn.c_str());
-        if (m_styles[cur].text.isEmpty()){
+        if (m_styles[m_edit].text.isEmpty()){
             QFile f(QFile::decodeName(nn.c_str()));
             if (f.open(IO_ReadOnly)){
                 string s;
                 s.append(f.size(), '\x00');
                 f.readBlock((char*)(s.c_str()), f.size());
-                m_styles[cur].text = QString::fromUtf8(s.c_str());
+                m_styles[m_edit].text = QString::fromUtf8(s.c_str());
             }
         }
         QFile::remove(QFile::decodeName(nn.c_str()));
-        m_styles[cur].name = newName;
+        m_styles[m_edit].name = newName;
     }
     fillCombo(newName);
 }
@@ -534,10 +571,10 @@ bool HistoryConfig::eventFilter(QObject *o, QEvent *e)
         case Key_Enter:
         case Key_Return:
             QTimer::singleShot(0, this, SLOT(realRename()));
-            break;
+            return true;
         case Key_Escape:
             QTimer::singleShot(0, this, SLOT(cancelRename()));
-            break;
+            return true;
         }
     }
     return HistoryConfigBase::eventFilter(o, e);
@@ -546,7 +583,7 @@ bool HistoryConfig::eventFilter(QObject *o, QEvent *e)
 void HistoryConfig::viewChanged(QWidget *w)
 {
     int cur = cmbStyle->currentItem();
-    if (cur < 0)
+    if ((cur < 0) || (!m_styles.size()))
         return;
     if (w == preview){
         if (!m_styles[cur].bCustom)
@@ -587,13 +624,17 @@ void HistoryConfig::sync()
 void HistoryConfig::textChanged()
 {
     m_bDirty = true;
+    int cur = cmbStyle->currentItem();
+    if ((cur < 0) || (!m_styles.size()))
+        return;
+    m_styles[cur].bChanged = true;
 }
 
 void HistoryConfig::fillPreview()
 {
     m_bDirty = false;
     int cur = cmbStyle->currentItem();
-    if (cur < 0)
+    if ((cur < 0) || (cur >= (int)m_styles.size()))
         return;
     XSL *xsl = new XSL(m_styles[cur].name);
     if (!m_styles[cur].text.isEmpty())
