@@ -45,6 +45,10 @@
 
 #include <time.h>
 
+#ifdef USE_KDE
+#include "kdeisversion.h"
+#endif
+
 const unsigned NO_TYPE = (unsigned)(-1);
 
 MsgTextEdit::MsgTextEdit(MsgEdit *edit, QWidget *parent)
@@ -53,6 +57,11 @@ MsgTextEdit::MsgTextEdit(MsgEdit *edit, QWidget *parent)
     m_edit = edit;
     setBackground(CorePlugin::m_plugin->getEditBackground());
     setForeground(CorePlugin::m_plugin->getEditForeground(), true);
+#if defined(USE_KDE)
+#if KDE_IS_VERSION(3,2,0)
+    setCheckSpellingEnabled(CorePlugin::m_plugin->getEnableSpell());
+#endif
+#endif
 }
 
 QPopupMenu *MsgTextEdit::createPopupMenu(const QPoint &pos)
@@ -96,11 +105,10 @@ Message *MsgTextEdit::createMessage(QMimeSource *src)
 void MsgTextEdit::contentsDropEvent(QDropEvent *e)
 {
     Message *msg = createMessage(e);
-    log(L_DEBUG, "Drop %u", (unsigned)msg);
     if (msg){
         e->accept();
         msg->setContact(m_edit->m_userWnd->id());
-        Event eOpen(EventOpenMessage, msg);
+        Event eOpen(EventOpenMessage, &msg);
         eOpen.process();
         delete msg;
         return;
@@ -111,7 +119,6 @@ void MsgTextEdit::contentsDropEvent(QDropEvent *e)
 void MsgTextEdit::contentsDragEnterEvent(QDragEnterEvent *e)
 {
     Message *msg = createMessage(e);
-    log(L_DEBUG, "Drag enter %u", (unsigned)msg);
     if (msg){
         delete msg;
         e->acceptAction();
@@ -123,7 +130,6 @@ void MsgTextEdit::contentsDragEnterEvent(QDragEnterEvent *e)
 void MsgTextEdit::contentsDragMoveEvent(QDragMoveEvent *e)
 {
     Message *msg = createMessage(e);
-    log(L_DEBUG, "Drag enter %u", (unsigned)msg);
     if (msg){
         delete msg;
         e->acceptAction();
@@ -148,7 +154,7 @@ MsgEdit::MsgEdit(QWidget *parent, UserWnd *userWnd)
 
     connect(CorePlugin::m_plugin, SIGNAL(modeChanged()), this, SLOT(modeChanged()));
 
-    m_frame = new QFrame(this);
+    m_frame = new QFrame(this, "msgedit");
     setCentralWidget(m_frame);
     m_layout = new QVBoxLayout(m_frame);
 
@@ -157,6 +163,8 @@ MsgEdit::MsgEdit(QWidget *parent, UserWnd *userWnd)
     m_edit->setForeground(QColor(CorePlugin::m_plugin->getEditForeground() & 0xFFFFFF), true);
     m_edit->setFont(CorePlugin::m_plugin->editFont);
     m_edit->setCtrlMode(!CorePlugin::m_plugin->getSendOnEnter());
+    m_edit->setParam(this);
+    setFocusProxy(m_edit);
 
     QStyleSheet *style = new QStyleSheet(m_edit);
     QStyleSheetItem *style_p = style->item("p");
@@ -268,6 +276,8 @@ void MsgEdit::editFontChanged(const QFont &f)
 bool MsgEdit::setMessage(Message *msg, bool bSetFocus)
 {
     m_type = msg->type();
+    m_userWnd->setMessageType(msg->type());
+    m_resource  = msg->getResource();
     m_bReceived = msg->getFlags() & MESSAGE_RECEIVED;
     QObject *processor = NULL;
     MsgReceived *rcv = NULL;
@@ -420,6 +430,7 @@ i18n("Message", "%n messages", 1);
 
 static MessageDef defGeneric =
     {
+        NULL,
         NULL,
         MESSAGE_DEFAULT,
         "Message",
@@ -445,10 +456,11 @@ i18n("SMS", "%n SMSs", 1);
 
 static MessageDef defSMS =
     {
+        NULL,
         NULL,
         MESSAGE_DEFAULT,
         "SMS",
-        "SMSs",
+        "%n SMSs",
         createSMS,
         generateSMS,
         NULL
@@ -485,6 +497,7 @@ static Message *dropUrl(QMimeSource *src)
 
 static MessageDef defUrl =
     {
+        NULL,
         NULL,
         MESSAGE_DEFAULT,
         "URL",
@@ -522,6 +535,7 @@ i18n("Contact list", "%n contact lists", 1);
 
 static MessageDef defContacts =
     {
+        NULL,
         NULL,
         MESSAGE_DEFAULT,
         "Contact list",
@@ -618,6 +632,7 @@ static CommandDef fileCommands[] =
 static MessageDef defFile =
     {
         fileCommands,
+        NULL,
         MESSAGE_DEFAULT,
         "File",
         "%n files",
@@ -692,6 +707,7 @@ static CommandDef authRequestCommands[] =
 static MessageDef defAuthRequest =
     {
         authRequestCommands,
+        NULL,
         MESSAGE_DEFAULT | MESSAGE_SYSTEM,
         "Authorize request",
         "%n authorize requests",
@@ -711,6 +727,7 @@ i18n("Authorization granted", "%n authorization granted", 1);
 
 static MessageDef defAuthGranted =
     {
+        NULL,
         NULL,
         MESSAGE_SILENT,
         "Authorization granted",
@@ -731,6 +748,7 @@ i18n("Authorization refused", "%n authorization refused", 1);
 
 static MessageDef defAuthRefused =
     {
+        NULL,
         NULL,
         MESSAGE_SILENT | MESSAGE_ERROR,
         "Authorization refused",
@@ -751,6 +769,7 @@ i18n("Add to contacts", "%n add to contacts", 1);
 
 static MessageDef defAdded =
     {
+        NULL,
         NULL,
         MESSAGE_INFO | MESSAGE_SYSTEM,
         "Add to contacts",
@@ -771,6 +790,7 @@ i18n("Removed from contacts", "%n removed from contacts", 1);
 
 static MessageDef defRemoved =
     {
+        NULL,
         NULL,
         MESSAGE_INFO | MESSAGE_SYSTEM,
         "Removed from contacts",
@@ -791,6 +811,7 @@ i18n("Status changed", "%n times status changed", 1);
 
 static MessageDef defStatus =
     {
+        NULL,
         NULL,
         MESSAGE_HIDDEN,
         "Status changed",
@@ -806,9 +827,9 @@ static bool cmp_status(ClientStatus s1, ClientStatus s2)
         return true;
     if (s1.status < s2.status)
         return false;
-    if (s1.data->LastSend > s2.data->LastSend)
+    if (s1.data->LastSend.value > s2.data->LastSend.value)
         return true;
-    if (s1.data->LastSend < s2.data->LastSend)
+    if (s1.data->LastSend.value < s2.data->LastSend.value)
         return false;
     return s1.client < s2.client;
 }
@@ -864,7 +885,7 @@ bool MsgEdit::sendMessage(Message *msg)
     Contact *contact = getContacts()->contact(m_userWnd->id());
     if (contact){
         TranslitUserData *data = (TranslitUserData*)(contact->getUserData(CorePlugin::m_plugin->translit_data_id));
-        if (data && data->Translit)
+        if (data && data->Translit.bValue)
             msg->setFlags(msg->getFlags() | MESSAGE_TRANSLIT);
     }
 
@@ -881,6 +902,20 @@ bool MsgEdit::sendMessage(Message *msg)
         ++multiply_it;
         if (multiply_it != multiply.end())
             msg->setFlags(msg->getFlags() | MESSAGE_MULTIPLY);
+    }else if (!m_resource.isEmpty()){
+        void *data = NULL;
+        Client *c = client(data, true, false, msg->contact(), true);
+        if (c){
+            string resources = c->resources(data);
+            while (!resources.empty()){
+                string res = getToken(resources, ';');
+                getToken(res, ',');
+                if (m_resource == QString::fromUtf8(res.c_str())){
+                    msg->setResource(m_resource);
+                    break;
+                }
+            }
+        }
     }
 
     editLostFocus();
@@ -937,7 +972,7 @@ bool MsgEdit::send()
         if (data){
             time_t now;
             time(&now);
-            ((clientData*)data)->LastSend = now;
+            ((clientData*)data)->LastSend.value = now;
         }
     }else{
         if (m_msg){
@@ -997,10 +1032,78 @@ void MsgEdit::modeChanged()
     m_edit->setCtrlMode(CorePlugin::m_plugin->getSendOnEnter());
 }
 
+bool MsgEdit::setType(unsigned type)
+{
+    CommandDef *def;
+    def = CorePlugin::m_plugin->messageTypes.find(type);
+    if (def == NULL)
+        return false;
+    MessageDef *mdef = (MessageDef*)(def->param);
+    if (mdef->flags & MESSAGE_SILENT)
+        return false;
+    if (mdef->create == NULL)
+        return false;
+    Message *msg = mdef->create(NULL);
+    if (msg == NULL)
+        return false;
+    m_userWnd->setMessage(&msg);
+    delete msg;
+    return true;
+}
+
+bool MsgEdit::adjustType()
+{
+    if (m_bReceived)
+        return true;
+    Command cmd;
+    cmd->menu_id = MenuMessage;
+    cmd->param = (void*)(m_userWnd->m_id);
+    cmd->id = m_userWnd->getMessageType();
+    Event e1(EventCheckState, cmd);
+    if ((m_userWnd->getMessageType() != m_type) && e1.process()){
+        if (setType(m_userWnd->getMessageType()))
+            return true;
+    }
+    cmd->id = m_type;
+    Event e(EventCheckState, cmd);
+    if (e.process())
+        return true;
+    Event eMenu(EventGetMenuDef, (void*)MenuMessage);
+    CommandsDef *cmdsMsg = (CommandsDef*)(eMenu.process());
+    CommandsList itc(*cmdsMsg, true);
+    CommandDef *c;
+    unsigned desired = m_userWnd->getMessageType();
+    bool bSet = false;
+    while ((c = ++itc) != NULL){
+        if (c->id == CmdContactClients)
+            continue;
+        c->param = (void*)(m_userWnd->m_id);
+        Event eCheck(EventCheckState, c);
+        if (!eCheck.process())
+            continue;
+        if (setType(c->id)){
+            bSet = true;
+            break;
+        }
+    }
+    m_userWnd->setMessageType(desired);
+    return bSet;
+}
+
 void *MsgEdit::processEvent(Event *e)
 {
+    if ((e->type() == EventContactChanged) && (((Contact*)(e->param()))->id() == m_userWnd->m_id)){
+        adjustType();
+        return NULL;
+    }
+    if (e->type() == EventClientChanged){
+        adjustType();
+        return NULL;
+    }
     if (e->type() == EventMessageReceived){
         Message *msg = (Message*)(e->param());
+        if (msg->getFlags() & MESSAGE_NOVIEW)
+            return NULL;
         if ((msg->contact() == m_userWnd->id()) && (msg->type() != MessageStatus)){
             if (CorePlugin::m_plugin->getContainerMode()){
                 bool bSetFocus = false;
@@ -1031,7 +1134,7 @@ void *MsgEdit::processEvent(Event *e)
                 TranslitUserData *data = (TranslitUserData*)(contact->getUserData(CorePlugin::m_plugin->translit_data_id));
                 if (data){
                     cmd->flags &= ~COMMAND_CHECKED;
-                    if (data->Translit)
+                    if (data->Translit.bValue)
                         cmd->flags |= COMMAND_CHECKED;
                 }
             }
@@ -1078,6 +1181,18 @@ void *MsgEdit::processEvent(Event *e)
     }
     if (e->type() == EventCommandExec){
         CommandDef *cmd = (CommandDef*)(e->param());
+#if defined(USE_KDE)
+#if KDE_IS_VERSION(3,2,0)
+        if (cmd->id == CmdEnableSpell){
+            m_edit->setCheckSpellingEnabled(cmd->flags & COMMAND_CHECKED);
+            return NULL;
+        }
+        if ((cmd->id == CmdSpell) && (cmd->param == this)){
+            m_edit->checkSpelling();
+            return e->param();
+        }
+#endif
+#endif
         if ((cmd->id == CmdSmile) && (cmd->param == this)){
             Event eBtn(EventCommandWidget, cmd);
             QToolButton *btnSmile = (QToolButton*)(eBtn.process());
@@ -1096,7 +1211,7 @@ void *MsgEdit::processEvent(Event *e)
             Contact *contact = getContacts()->contact(m_userWnd->id());
             if (contact){
                 TranslitUserData *data = (TranslitUserData*)(contact->getUserData(CorePlugin::m_plugin->translit_data_id, true));
-                data->Translit = ((cmd->flags & COMMAND_CHECKED) != 0);
+                data->Translit.bValue = ((cmd->flags & COMMAND_CHECKED) != 0);
             }
             return e->param();;
         }
@@ -1107,11 +1222,12 @@ void *MsgEdit::processEvent(Event *e)
         if ((cmd->bar_id == ToolBarMsgEdit) && m_edit->isReadOnly() && (cmd->param == this)){
             switch (cmd->id){
             case CmdMsgAnswer:{
-                    Message msg(MessageGeneric);
-                    msg.setContact(m_userWnd->id());
-                    msg.setClient(m_client.c_str());
+                    Message *msg = new Message(MessageGeneric);
+                    msg->setContact(m_userWnd->id());
+                    msg->setClient(m_client.c_str());
                     Event e(EventOpenMessage, &msg);
                     e.process();
+                    delete msg;
                 }
             case CmdNextMessage:
                 QTimer::singleShot(0, this, SLOT(goNext()));
@@ -1191,6 +1307,7 @@ void *MsgEdit::processEvent(Event *e)
                         m_msg = (mdef->create)(cfg.c_str());
                         m_msg->setContact(*multiply_it);
                         m_msg->setClient(NULL);
+                        m_msg->setFlags(m_msg->getFlags() | MESSAGE_MULTIPLY);
                         ++multiply_it;
                         if (multiply_it == multiply.end())
                             m_msg->setFlags(m_msg->getFlags() | MESSAGE_LAST);
@@ -1214,11 +1331,10 @@ void *MsgEdit::processEvent(Event *e)
                 if (bClose){
                     QTimer::singleShot(0, m_userWnd, SLOT(close()));
                 }else{
-                    m_edit->setText("");
+                    setEmptyMessage();
                     m_edit->setFont(CorePlugin::m_plugin->editFont);
                     m_edit->setForeground(CorePlugin::m_plugin->getEditForeground(), true);
                     m_edit->setBackground(CorePlugin::m_plugin->getEditBackground());
-                    setEmptyMessage();
                 }
             }
             return NULL;
@@ -1229,40 +1345,33 @@ void *MsgEdit::processEvent(Event *e)
 
 void MsgEdit::setEmptyMessage()
 {
-    bool bGeneric = true;
-    Contact *contact = getContacts()->contact(m_userWnd->id());
-    if (contact){
-        Command cmd;
-        cmd->id      = MessageGeneric;
-        cmd->menu_id = MenuMessage;
-        cmd->param   = (void*)(m_userWnd->id());
-        Event e(EventCheckState, cmd);
-        if (e.process() == NULL){
-            QString phones = contact->getPhones();
-            while (phones.length()){
-                QString phoneItem = getToken(phones, ';', false);
-                phoneItem = getToken(phoneItem, '/', false);
-                QString phone = getToken(phoneItem, ',');
-                getToken(phoneItem, ',');
-                if (phoneItem.toUInt() == CELLULAR){
-                    cmd->id      = MessageSMS;
-                    Event e(EventCheckState, cmd);
-                    if (e.process())
-                        bGeneric = false;
-                    break;
-                }
-            }
+    m_edit->setText("");
+    Event eMenu(EventGetMenuDef, (void*)MenuMessage);
+    CommandsDef *cmdsMsg = (CommandsDef*)(eMenu.process());
+    CommandsList itc(*cmdsMsg, true);
+    CommandDef *c;
+    while ((c = ++itc) != NULL){
+        c->param = (void*)(m_userWnd->m_id);
+        Event eCheck(EventCheckState, c);
+        if (eCheck.process()){
+            Message *msg;
+            CommandDef *def = CorePlugin::m_plugin->messageTypes.find(c->id);
+            if (def == NULL)
+                continue;
+            MessageDef *mdef = (MessageDef*)(def->param);
+            if (mdef->create == NULL)
+                continue;
+            msg = mdef->create(NULL);
+            msg->setContact(m_userWnd->m_id);
+            if (mdef->flags & MESSAGE_SILENT)
+                continue;
+            msg->setFlags(MESSAGE_NORAISE);
+            Event eOpen(EventOpenMessage, &msg);
+            eOpen.process();
+            delete msg;
+            return;
         }
     }
-    Message *msg;
-    if (bGeneric){
-        msg = new Message(MessageGeneric);
-    }else{
-        msg = new SMSMessage;
-    }
-    msg->setContact(m_userWnd->id());
-    m_userWnd->setMessage(msg);
-    delete msg;
 }
 
 void MsgEdit::changeTyping(Client *client, void *data)
@@ -1378,7 +1487,7 @@ void MsgEdit::goNext()
         Message *msg = History::load((*it).id, (*it).client.c_str(), (*it).contact);
         if (msg == NULL)
             continue;
-        Event e(EventOpenMessage, msg);
+        Event e(EventOpenMessage, &msg);
         e.process();
         delete msg;
         return;
