@@ -29,6 +29,38 @@
 static char MSG_ANCHOR[] = "<a name=\"m:";
 static char MSG_BEGIN[]  = "<a name=\"b\">";
 
+typedef struct Smile
+{
+    unsigned	nSmile;
+    int			pos;
+    int			size;
+    QRegExp		re;
+} Smile;
+
+class ViewParser : public HTMLParser
+{
+public:
+    ViewParser(bool bIgnoreColors, bool bUseSmiles);
+    QString parse(const QString &str);
+    bool RTL;
+    bool m_bParaStart;
+protected:
+    QString res;
+    bool m_bIgnoreColors;
+    bool m_bUseSmiles;
+    bool m_bInLink;
+    bool m_bInHead;
+    bool m_bFirst;
+    bool m_bSpan;
+    bool m_bPara;
+    bool m_bRTL;
+    bool m_bParaEnd;
+    list<Smile> m_smiles;
+    virtual void text(const QString &text);
+    virtual void tag_start(const QString &tag, const list<QString> &options);
+    virtual void tag_end(const QString &tag);
+};
+
 MsgViewBase::MsgViewBase(QWidget *parent, const char *name, unsigned id)
         : TextShow(parent, name)
 {
@@ -69,6 +101,107 @@ void MsgViewBase::setSelect(const QString &str)
     m_selectStr = str;
 }
 
+void MsgViewBase::update()
+{
+    if (m_updated.empty())
+        return;
+    unsigned i;
+    for (i = 0; i < (unsigned)paragraphs(); i++){
+        QString s = text(i);
+        int n = s.find(MSG_ANCHOR);
+        if (n < 0)
+            continue;
+        s = s.mid(n + strlen(MSG_ANCHOR));
+        n = s.find("\"");
+        if (n < 0)
+            continue;
+        string client;
+        unsigned id = messageId(s.left(n), client);
+        list<Msg_Id>::iterator it;
+        for (it = m_updated.begin(); it != m_updated.end(); ++it){
+            if (((*it).id == id) && ((*it).client == client))
+                break;
+        }
+        if (it != m_updated.end())
+            break;
+    }
+    m_updated.clear();
+    if (i >= (unsigned)paragraphs())
+        return;
+    QPoint p = QPoint(0, 0);
+    p = mapToGlobal(p);
+    p = viewport()->mapFromGlobal(p);
+    int x, y;
+    viewportToContents(p.x(), p.y(), x, y);
+    int para;
+    int pos = charAt(QPoint(x, y), &para);
+    p = QPoint(0, viewport()->height());
+    p = viewport()->mapToGlobal(p);
+    p = mapFromGlobal(p);
+    if (p.y() + 2 == height())
+        pos = -1;
+    unsigned start = i;
+    list<Msg_Id> msgs;
+    for (; i < (unsigned)paragraphs(); i++){
+        QString s = text(i);
+        int n = s.find(MSG_ANCHOR);
+        if (n < 0)
+            continue;
+        s = s.mid(n + strlen(MSG_ANCHOR));
+        n = s.find("\"");
+        if (n < 0)
+            continue;
+        string client;
+        unsigned id = messageId(s.left(n), client);
+        list<Msg_Id>::iterator it;
+        for (it = msgs.begin(); it != msgs.end(); ++it){
+            if (((*it).id == id) && ((*it).client == client))
+                break;
+        }
+        if (it != msgs.end())
+            continue;
+        Msg_Id m_id;
+        m_id.id     = id;
+        m_id.client = client;
+        msgs.push_back(m_id);
+    }
+    int paraFrom, indexFrom;
+    int paraTo, indexTo;
+    getSelection(&paraFrom, &indexFrom, &paraTo, &indexTo);
+    setReadOnly(false);
+    setSelection(start, 0, paragraphs() - 1, 0xFFFF);
+    removeSelectedText();
+    setReadOnly(true);
+    QString text;
+    for (list<Msg_Id>::iterator it = msgs.begin(); it != msgs.end(); ++it){
+        Message *msg = History::load((*it).id, (*it).client.c_str(), m_id);
+        if (msg == NULL)
+            continue;
+        bool bUnread = false;
+        for (list<msg_id>::iterator itu = CorePlugin::m_plugin->unread.begin(); itu != CorePlugin::m_plugin->unread.end(); ++itu){
+            msg_id &m = (*itu);
+            if ((m.contact == msg->contact()) &&
+                    (m.id == msg->id()) &&
+                    (m.client == msg->client())){
+                bUnread = true;
+                break;
+            }
+        }
+        text += messageText(msg, bUnread);
+    }
+    append(text);
+    if (!CorePlugin::m_plugin->getOwnColors())
+        setBackground(0);
+    if ((paraFrom != paraTo) || (indexFrom != indexTo))
+        setSelection(paraFrom, indexFrom, paraTo, indexTo);
+    if (pos == -1){
+        scrollToBottom();
+    }else{
+        setCursorPosition(para, pos);
+        ensureCursorVisible();
+    }
+}
+
 QString MsgViewBase::messageText(Message *msg, bool bUnread)
 {
     QString options;
@@ -169,8 +302,14 @@ QString MsgViewBase::messageText(Message *msg, bool bUnread)
     info += QString("<from>%1</from>") .arg(quoteString(contactName));
     QString id = QString::number(msg->id());
     id += ",";
-    if (msg->getBackground() != msg->getForeground())
-        id += QString::number(msg->getBackground() & 0xFFFFFF);
+    // <hack>
+    // Terrible hack to set message bgcolor. We prefer to insert the entire history
+    // in one chunk (since it's more efficient and causes less redraws), and there's
+    // no way to set block's background color directly in Qt's HTML), so we make a note
+    // of it in the HTML and set it retroactively in setBackground.
+    if ((msg->getBackground() != 0xFFFFFFFF) && (msg->getForeground() != msg->getBackground()))
+        id += QString::number(msg->getBackground());
+    // </hack>
     string client_str;
     if (msg->client())
         client_str = msg->client();
@@ -178,10 +317,15 @@ QString MsgViewBase::messageText(Message *msg, bool bUnread)
         id += ",";
         id += quoteString(client_str.c_str());
     }
+    if (m_cut.size()){
+        id += ",";
+        id += QString::number(m_cut.size());
+    }
     info += "<id>";
     info += id;
     info += "</id>";
 
+
     QString icons;
     if (msg->getFlags() & MESSAGE_SECURE)
         options += " encrypted=\"1\"";
@@ -227,23 +371,56 @@ QString MsgViewBase::messageText(Message *msg, bool bUnread)
 
     Event e(EventEncodeText, &msgText);
     e.process();
-    msgText = parseText(msgText, CorePlugin::m_plugin->getOwnColors(), CorePlugin::m_plugin->getUseSmiles());
-    msgText = QString(MSG_BEGIN) + msgText;
-    s += "<body>";
-    s += quoteString(msgText);
+    ViewParser parser(CorePlugin::m_plugin->getOwnColors(), CorePlugin::m_plugin->getUseSmiles());
+    msgText = parser.parse(msgText);
+    s += "<body";
+
+    if ((msg->getForeground() != 0xFFFFFFFF) && (msg->getForeground() != msg->getBackground()))
+    {
+        s += " fgcolor=\"#";
+        s += QString::number(msg->getForeground(), 16).rightJustify(6, '0');
+        s += "\"";
+    }
+
+    // Some bright day might come when one could specify background color from inside Qt's richtext.
+    // Meanwhile, this is useless:
+    if ((msg->getBackground() != 0xFFFFFFFF) && (msg->getForeground() != msg->getBackground()))
+    {
+        s += " bgcolor=\"#";
+        s += QString::number(msg->getBackground(), 16).rightJustify(6, '0');
+        s += "\"";
+    }
+    s += ">";
+
+    // We pass the rich text quoted, since we're not sure of its' XML validity.
+    // The XSL engine should copy it as-is (using xsl:value-of with disable-output-escaping="yes").
+    s += quoteString(QString(MSG_BEGIN) + msgText);
+
     s += "</body>";
     s += "</message>";
     XSL *p = xsl;
     if (p == NULL)
         p = CorePlugin::m_plugin->historyXSL;
+    QString res = p->process(s);
     QString anchor = MSG_ANCHOR;
     anchor += id;
     anchor += "\">";
-    QString res = p->process(s);
-    if (res.left(3) == "<p>"){
-        res = QString("<p>") + anchor + res.mid(3);
+    res = anchor + res;
+    int startPos = res.find(MSG_BEGIN);
+    int paraPos;
+    if (parser.m_bParaStart){
+        paraPos = res.find("<p");
+        if (paraPos >= 0)
+            res = res.left(paraPos) + "</p>" + res.mid(paraPos);
+    }
+    paraPos = res.findRev("<p", startPos);
+    QString dir = " dir=\"";
+    dir += parser.RTL ? "rtl" : "ltr";
+    dir += "\"";
+    if (paraPos >= 0){
+        res = res.left(paraPos + 2) + dir + res.mid(paraPos + 2);
     }else{
-        res = anchor + res;
+        res = QString("<p") + dir + ">" + res;
     }
     return res;
 }
@@ -267,46 +444,66 @@ void MsgViewBase::setSource(const QString &url)
         client = QString::number(m_id);
     Message *msg = History::load(msg_id, client.utf8(), m_id);
     if (msg){
-        Event e(EventOpenMessage, msg);
+        Event e(EventOpenMessage, &msg);
         e.process();
         delete msg;
     }
 }
 
+// <hack>
+// We have to use this function since Qt has no tag to set background color per-paragraph
+// from within HTML. See matching hack in MsgViewBase::messageText.
 void MsgViewBase::setBackground(unsigned n)
 {
-    QColor c;
-    bool bSet = false;
-    bool bSetColor = false;
+    QColor bgcolor;
+    bool bInMsg = false;
+    bool bSet   = false;
+
+    QString sAnchor = QString::fromLatin1(MSG_ANCHOR),
+                      sBegin = QString::fromLatin1(MSG_BEGIN);
+
     for (unsigned i = n; i < (unsigned)paragraphs(); i++){
         QString s = text(i);
-        int n = s.find(MSG_ANCHOR);
-        if (n >= 0){
-            QString t = s.mid(n + strlen(MSG_BEGIN));
-            int p = t.find('\"');
-            if (p >= 0)
-                t = t.left(p);
-            getToken(t, ',');
-            t = getToken(t, ',');
-            if (t.isEmpty()){
-                bSet = false;
-            }else{
-                c = QColor(atol(t.latin1()));
-                bSet = true;
+        int anchorPos = s.find(sAnchor);
+        if (anchorPos >= 0)
+        {
+            bInMsg = false;
+            bSet   = false;
+
+            // This code could be a bit faster by making assumptions.
+            // However, I prefer to be correct HTML-parser-wise.
+
+            int idStart = anchorPos + sAnchor.length();
+            int idEnd = s.find('\"', idStart);
+            if ((idStart >= 0) && (idEnd >= 0))
+            {
+                QString id = s.mid(idStart, idEnd - idStart);
+
+                // Parse the message id (msgId,backgroundColor,...)
+                int bgcolorStart = id.find(',');
+                if (bgcolorStart >= 0)
+                {
+                    QString sBgcolor = id.mid(bgcolorStart + 1);
+                    int bgcolorEnd = sBgcolor.find(',');
+                    if (bgcolorEnd > 0)
+                        sBgcolor = sBgcolor.left(bgcolorEnd);
+                    if (!sBgcolor.isEmpty())
+                        bgcolor = QColor(sBgcolor.toULong(&bSet));
+                }
             }
-            bSetColor = false;
         }
-        n = s.find(MSG_BEGIN);
-        if (n >= 0)
-            bSetColor = true;
-        if (bSet && bSetColor){
-            setParagraphBackgroundColor(i, c);
+        if (s.find(sBegin) >= 0)
+            bInMsg = true;
+
+        if (bInMsg && bSet){
+            setParagraphBackgroundColor(i, bgcolor);
         }else{
             clearParagraphBackground(i);
         }
-        bSetColor = false;
     }
+
 }
+// </hack>
 
 void MsgViewBase::addMessage(Message *msg, bool bUnread)
 {
@@ -362,12 +559,8 @@ bool MsgViewBase::findMessage(Message *msg)
             ensureCursorVisible();
             return true;
         }
-        s = s.left(n);
-        unsigned id = atol(getToken(s, ',').latin1());
-        if (id != msg->id())
-            continue;
-        getToken(s, ',');
-        if (s != msg->client())
+        string client;
+        if ((messageId(s.left(n), client) != msg->id()) || (client != msg->client()))
             continue;
         setCursorPosition(i, 0);
         ensureCursorVisible();
@@ -386,8 +579,171 @@ void MsgViewBase::setColors()
     TextShow::setForeground(CorePlugin::m_plugin->getEditForeground());
 }
 
+unsigned MsgViewBase::messageId(const QString &_s, string &client)
+{
+    QString s(_s);
+    unsigned id = atol(getToken(s, ',').latin1());
+    getToken(s, ',');
+    client = getToken(s, ',').utf8();
+    if (id >= 0x80000000)
+        return id;
+    for (unsigned cut_id = atol(s.latin1()); cut_id < m_cut.size(); cut_id++){
+        if (m_cut[cut_id].client != client)
+            continue;
+        if (id < m_cut[cut_id].from)
+            continue;
+        id -= m_cut[cut_id].size;
+    }
+    return id;
+}
+
+void MsgViewBase::reload()
+{
+    QString t;
+    vector<Msg_Id> msgs;
+    unsigned i;
+    for (i = 0; i < (unsigned)paragraphs(); i++){
+        QString s = text(i);
+        int n = s.find(MSG_ANCHOR);
+        if (n < 0)
+            continue;
+        s = s.mid(n + strlen(MSG_ANCHOR));
+        n = s.find("\"");
+        if (n < 0)
+            continue;
+        string client;
+        Msg_Id id;
+        id.id = messageId(s.left(n), client);
+        id.client = client;
+        unsigned nn;
+        for (nn = 0; nn < msgs.size(); nn++){
+            if ((msgs[nn].id == id.id) && (msgs[nn].client == id.client))
+                break;
+        }
+        if (nn >= msgs.size())
+            msgs.push_back(id);
+    }
+    for (i = 0; i < msgs.size(); i++){
+        Message *msg = History::load(msgs[i].id, msgs[i].client.c_str(), m_id);
+        if (msg == NULL)
+            continue;
+        t += messageText(msg, false);
+        delete msg;
+    }
+    QPoint p = QPoint(0, height());
+    p = mapToGlobal(p);
+    p = viewport()->mapFromGlobal(p);
+    int x, y;
+    viewportToContents(p.x(), p.y(), x, y);
+    int para;
+    int pos = charAt(QPoint(x, y), &para);
+    setText(t);
+    if (!CorePlugin::m_plugin->getOwnColors())
+        setBackground(0);
+    if (pos == -1){
+        scrollToBottom();
+    }else{
+        setCursorPosition(para, pos);
+        ensureCursorVisible();
+    }
+}
+
 void *MsgViewBase::processEvent(Event *e)
 {
+    if ((e->type() == EventRewriteMessage) || (e->type() == EventMessageRead)){
+        Message *msg = (Message*)(e->param());
+        if (msg->contact() != m_id)
+            return NULL;
+        unsigned i;
+        for (i = 0; i < (unsigned)paragraphs(); i++){
+            QString s = text(i);
+            int n = s.find(MSG_ANCHOR);
+            if (n < 0)
+                continue;
+            s = s.mid(n + strlen(MSG_ANCHOR));
+            n = s.find("\"");
+            if (n < 0)
+                continue;
+            string client;
+            if ((messageId(s.left(n), client) == msg->id()) && (client == msg->client()))
+                break;
+        }
+        if (i >= (unsigned)paragraphs())
+            return NULL;
+        Msg_Id id;
+        id.id     = msg->id();
+        id.client = msg->client();
+        m_updated.push_back(id);
+        QTimer::singleShot(0, this, SLOT(update()));
+        return NULL;
+    }
+    if (e->type() == EventCutHistory){
+        CutHistory *ch = (CutHistory*)(e->param());
+        if (ch->contact != m_id)
+            return NULL;
+
+        bool bDelete = false;
+        vector<unsigned> start_pos;
+        vector<unsigned> end_pos;
+        for (unsigned i = 0; i < (unsigned)paragraphs(); i++){
+            QString s = text(i);
+            int n = s.find(MSG_ANCHOR);
+            if (n < 0)
+                continue;
+            s = s.mid(n + strlen(MSG_ANCHOR));
+            n = s.find("\"");
+            if (n < 0)
+                continue;
+            string client;
+            unsigned id = messageId(s.left(n), client);
+            if ((client == ch->client) && (id >= ch->from) && (id < ch->from + ch->size)){
+                if (!bDelete){
+                    bDelete = true;
+                    start_pos.push_back(i);
+                }
+            }else{
+                if (bDelete){
+                    bDelete = false;
+                    end_pos.push_back(i);
+                }
+            }
+        }
+        if (bDelete)
+            end_pos.push_back(paragraphs());
+        if (start_pos.size()){
+            int paraFrom, indexFrom;
+            int paraTo, indexTo;
+            getSelection(&paraFrom, &indexFrom, &paraTo, &indexTo);
+            QPoint p = QPoint(0, 0);
+            p = mapToGlobal(p);
+            p = viewport()->mapFromGlobal(p);
+            int x, y;
+            viewportToContents(p.x(), p.y(), x, y);
+            int para;
+            int pos = charAt(QPoint(x, y), &para);
+            setReadOnly(false);
+            for (unsigned i = 0; i < start_pos.size(); i++){
+                setSelection(start_pos[i], 0, end_pos[i], 0);
+                removeSelectedText();
+                if ((unsigned)pos >= start_pos[i])
+                    pos = end_pos[i] - start_pos[i];
+            }
+            if ((paraFrom == -1) && (paraTo == -1)){
+                if (pos == -1){
+                    scrollToBottom();
+                }else{
+                    setCursorPosition(para, pos);
+                    ensureCursorVisible();
+                }
+            }else{
+                setSelection(paraFrom, indexFrom, paraTo, indexTo);
+            }
+            setReadOnly(true);
+            repaint();
+        }
+        m_cut.push_back(*ch);
+        return NULL;
+    }
     if (e->type() == EventMessageDeleted){
         Message *msg = (Message*)(e->param());
         if (msg->contact() != m_id)
@@ -401,23 +757,38 @@ void *MsgViewBase::processEvent(Event *e)
             n = s.find("\"");
             if (n < 0)
                 continue;
-            s = s.left(n);
-            unsigned id = atol(getToken(s, ',').latin1());
-            if (id != msg->id())
-                continue;
-            getToken(s, ',');
-            if (s != msg->client())
+            string client;
+            if ((messageId(s.left(n), client) != msg->id()) || (client != msg->client()))
                 continue;
+            string ss;
+            ss = text(i).local8Bit();
+            log(L_DEBUG, "?: %s", ss.c_str());
+
             unsigned j;
             for (j = i + 1; j < (unsigned)paragraphs(); j++){
                 QString s = text(j);
-                if (s.find(MSG_ANCHOR) >= 0)
+                ss = text(j).local8Bit();
+                log(L_DEBUG, ">: %s", ss.c_str());
+                int n = s.find(MSG_ANCHOR);
+                if (n < 0)
+                    continue;
+                s = s.mid(n + strlen(MSG_ANCHOR));
+                n = s.find("\"");
+                if (n < 0)
+                    continue;
+                string client;
+                if ((messageId(s.left(n), client) != msg->id()) || (client != msg->client()))
                     break;
             }
             int paraFrom, indexFrom;
             int paraTo, indexTo;
             getSelection(&paraFrom, &indexFrom, &paraTo, &indexTo);
-            setSelection(i, 0, j - 1, 0xFFFF);
+            unsigned pos = 0xFFFF;
+            if (j == (unsigned)paragraphs()){
+                j++;
+                pos = 0;
+            }
+            setSelection(i, 0, j - 1, pos);
             setReadOnly(false);
             removeSelectedText();
             setReadOnly(true);
@@ -430,80 +801,11 @@ void *MsgViewBase::processEvent(Event *e)
         }
         return NULL;
     }
-    if (e->type() == EventMessageRead){
-        Message *msg = (Message*)(e->param());
-        if (msg->contact() != m_id)
-            return NULL;
-        for (unsigned i = 0; i < (unsigned)paragraphs(); i++){
-            QString s = text(i);
-            int n = s.find(MSG_ANCHOR);
-            if (n < 0)
-                continue;
-            s = s.mid(n + strlen(MSG_ANCHOR));
-            n = s.find("\"");
-            if (n < 0)
-                continue;
-            s = s.left(n);
-            unsigned id = atol(getToken(s, ',').latin1());
-            if (id != msg->id())
-                continue;
-            getToken(s, ',');
-            if (s != msg->client())
-                continue;
-            int paraFrom, indexFrom;
-            int paraTo, indexTo;
-            getSelection(&paraFrom, &indexFrom, &paraTo, &indexTo);
-            setSelection(i, 0, i, 0xFFFF);
-            setBold(false);
-            if ((paraFrom == -1) && (paraTo == -1)){
-                removeSelection();
-                scrollToBottom();
-            }else{
-                setSelection(paraFrom, indexFrom, paraTo, indexTo);
-            }
-            break;
-        }
-        return NULL;
-    }
     if (e->type() == EventHistoryConfig){
         unsigned id = (unsigned)(e->param());
         if (id && (id != m_id))
             return NULL;
-        QString t;
-        for (unsigned i = 0; i < (unsigned)paragraphs(); i++){
-            QString s = text(i);
-            int n = s.find(MSG_ANCHOR);
-            if (n < 0)
-                continue;
-            s = s.mid(n + strlen(MSG_ANCHOR));
-            n = s.find("\"");
-            if (n < 0)
-                continue;
-            s = s.left(n);
-            unsigned id = atol(getToken(s, ',').latin1());
-            getToken(s, ',');
-            Message *msg = History::load(id, s.utf8(), m_id);
-            if (msg == NULL)
-                continue;
-            t += messageText(msg, false);
-            delete msg;
-        }
-        QPoint p = QPoint(0, height());
-        p = mapToGlobal(p);
-        p = viewport()->mapFromGlobal(p);
-        int x, y;
-        viewportToContents(p.x(), p.y(), x, y);
-        int para;
-        int pos = charAt(QPoint(x, y), &para);
-        setText(t);
-        if (!CorePlugin::m_plugin->getOwnColors())
-            setBackground(0);
-        if (pos == -1){
-            scrollToBottom();
-        }else{
-            setCursorPosition(para, pos);
-            ensureCursorVisible();
-        }
+        reload();
     }
     if (e->type() == EventHistoryColors)
         setColors();
@@ -539,15 +841,21 @@ void *MsgViewBase::processEvent(Event *e)
 
                 unsigned n = 0;
                 MessageDef *mdef = NULL;
-                if (msg->getFlags() & MESSAGE_RECEIVED){
-                    unsigned type = msg->baseType();
-                    CommandDef *msgCmd = CorePlugin::m_plugin->messageTypes.find(type);
-                    if (msgCmd )
-                        mdef = (MessageDef*)(msgCmd->param);
-                }
-                if (mdef && mdef->cmd){
-                    for (const CommandDef *d = mdef->cmd; d->text; d++)
-                        n++;
+                unsigned type = msg->baseType();
+                const CommandDef *cmdsSpecial = NULL;
+                CommandDef *msgCmd = CorePlugin::m_plugin->messageTypes.find(type);
+                if (msgCmd)
+                    mdef = (MessageDef*)(msgCmd->param);
+
+                if (mdef){
+                    if (msg->getFlags() & MESSAGE_RECEIVED){
+                        cmdsSpecial = mdef->cmdReceived;
+                    }else{
+                        cmdsSpecial = mdef->cmdSent;
+                    }
+                    if (cmdsSpecial)
+                        for (const CommandDef *d = cmdsSpecial; d->text; d++)
+                            n++;
                 }
 
                 {
@@ -562,8 +870,8 @@ void *MsgViewBase::processEvent(Event *e)
                 CommandDef *cmds = new CommandDef[n];
                 memset(cmds, 0, sizeof(CommandDef) * n);
                 n = 0;
-                if (mdef && mdef->cmd){
-                    for (const CommandDef *d = mdef->cmd; d->text; d++){
+                if (cmdsSpecial){
+                    for (const CommandDef *d = cmdsSpecial; d->text; d++){
                         cmds[n] = *d;
                         cmds[n].id = CmdMsgSpecial + n;
                         n++;
@@ -595,6 +903,22 @@ void *MsgViewBase::processEvent(Event *e)
             return NULL;
         Message *msg;
         switch (cmd->id){
+        case CmdCutHistory:
+            msg = currentMessage();
+            if (msg){
+                History::cut(msg, 0, 0);
+                delete msg;
+                return e->param();
+            }
+            return NULL;
+        case CmdDeleteMessage:
+            msg = currentMessage();
+            if (msg){
+                History::del(msg);
+                delete msg;
+                return e->param();
+            }
+            return NULL;
         case CmdCopy:
             copy();
             return e->param();
@@ -602,7 +926,7 @@ void *MsgViewBase::processEvent(Event *e)
             msg = currentMessage();
             if (msg){
                 msg->setFlags(msg->getFlags() | MESSAGE_OPEN);
-                Event eOpen(EventOpenMessage, msg);
+                Event eOpen(EventOpenMessage, &msg);
                 eOpen.process();
                 delete msg;
                 return e->param();
@@ -617,9 +941,18 @@ void *MsgViewBase::processEvent(Event *e)
                     CommandDef *msgCmd = CorePlugin::m_plugin->messageTypes.find(type);
                     if (msgCmd)
                         mdef = (MessageDef*)(msgCmd->param);
-                    if (mdef && mdef->cmd){
+                    const CommandDef *cmds = NULL;
+                    if (mdef){
+                        if (msg->getFlags() & MESSAGE_RECEIVED){
+                            cmds = mdef->cmdReceived;
+                        }else{
+                            cmds = mdef->cmdSent;
+                        }
+                    }
+
+                    if (cmds){
                         unsigned n = cmd->id - CmdMsgSpecial;
-                        for (const CommandDef *d = mdef->cmd; d->text; d++){
+                        for (const CommandDef *d = cmds; d->text; d++){
                             if (n-- == 0){
                                 CommandDef cmd = *d;
                                 cmd.param = msg;
@@ -656,10 +989,12 @@ Message *MsgViewBase::currentMessage()
         if (n < 0)
             continue;
         s = s.mid(n + strlen(MSG_ANCHOR));
-        unsigned long id = atol(getToken(s, ',').latin1());
-        getToken(s, ',');
-        s = getToken(s, '\"');
-        Message *msg = History::load(id, s.utf8(), m_id);
+        n = s.find("\"");
+        if (n < 0)
+            continue;
+        string client;
+        unsigned id = messageId(s.left(n), client);
+        Message *msg = History::load(id, client.c_str(), m_id);
         if (msg)
             return msg;
     }
@@ -733,13 +1068,15 @@ void *MsgView::processEvent(Event *e)
         Message *msg = (Message*)(e->param());
         if (msg->contact() != m_id)
             return NULL;
+        if (msg->getFlags() & MESSAGE_NOVIEW)
+            return NULL;
         bool bAdd = true;
         if (msg->type() == MessageStatus){
             bAdd = false;
             Contact *contact = getContacts()->contact(msg->contact());
             if (contact){
                 CoreUserData *data = (CoreUserData*)(contact->getUserData(CorePlugin::m_plugin->user_data_id));
-                if (data && data->LogStatus)
+                if (data && data->LogStatus.bValue)
                     bAdd = true;
             }
         }
@@ -747,7 +1084,7 @@ void *MsgView::processEvent(Event *e)
             Contact *contact = getContacts()->contact(msg->contact());
             if (contact){
                 CoreUserData *data = (CoreUserData*)(contact->getUserData(CorePlugin::m_plugin->user_data_id));
-                if (data->OpenNewMessage)
+                if (data->OpenNewMessage.bValue)
                     bAdd = false;
             }
         }
@@ -760,49 +1097,25 @@ void *MsgView::processEvent(Event *e)
     return MsgViewBase::processEvent(e);
 }
 
-typedef struct Smile
-{
-    unsigned	nSmile;
-    int			pos;
-    int			size;
-    QRegExp		re;
-} Smile;
-
-class ViewParser : public HTMLParser
-{
-public:
-    ViewParser(bool bIgnoreColors, bool bUseSmiles);
-    QString parse(const QString &str);
-protected:
-    QString res;
-    bool m_bIgnoreColors;
-    bool m_bUseSmiles;
-    bool m_bInLink;
-    bool m_bInHead;
-    bool m_bFirst;
-    bool m_bSpan;
-    bool m_bPara;
-    list<Smile> m_smiles;
-    virtual void text(const QString &text);
-    virtual void tag_start(const QString &tag, const list<QString> &options);
-    virtual void tag_end(const QString &tag);
-};
-
 ViewParser::ViewParser(bool bIgnoreColors, bool bUseSmiles)
 {
     m_bIgnoreColors = bIgnoreColors;
     m_bUseSmiles    = bUseSmiles;
     m_bInLink       = false;
     m_bInHead       = false;
-    m_bFirst		= true;
+    m_bFirst     	= true;
     m_bSpan			= false;
     m_bPara			= false;
+    m_bParaStart	= false;
+    m_bRTL			= false;
+    m_bParaEnd		= false;
+    RTL				= false;
     if (m_bUseSmiles){
         for (unsigned i = 0; ;i++){
             const smile *s = smiles(i);
             if (s == NULL)
                 break;
-#if QT_VERSION < 300
+#if COMPAT_QT_VERSION < 0x030000
             string str;
             for (const char *p = s->exp;; p++){
                 if ((*p == 0) || (*p == '|')){
@@ -828,13 +1141,13 @@ ViewParser::ViewParser(bool bIgnoreColors, bool bUseSmiles)
                 str += *p;
             }
 #else
-if (*(s->exp)){
-            Smile ss;
-            ss.nSmile = i;
-            ss.re = QRegExp(s->exp);
-            if (ss.re.isValid())
-                m_smiles.push_back(ss);
-        }
+            if (*(s->exp)){
+                Smile ss;
+                ss.nSmile = i;
+                ss.re = QRegExp(s->exp);
+                if (ss.re.isValid())
+                    m_smiles.push_back(ss);
+            }
 #endif
         }
     }
@@ -844,21 +1157,23 @@ QString ViewParser::parse(const QString &str)
 {
     res = "";
     HTMLParser::parse(str);
+    if (m_bParaStart)
+        res += "</p>";
     return res;
 }
 
 void ViewParser::text(const QString &text)
 {
+    if (text.isEmpty())
+        return;
+    if (m_bParaEnd){
+        m_bParaEnd = false;
+        res += "<br/>";
+    }
     if (!m_bUseSmiles || m_bInLink){
         res += quoteString(text);
         return;
     }
-    if (text.isEmpty())
-        return;
-    if (m_bPara){
-        m_bPara = false;
-        res += "<br>";
-    }
     m_bFirst = false;
     QString str = text;
     for (list<Smile>::iterator it = m_smiles.begin(); it != m_smiles.end(); ++it){
@@ -952,26 +1267,37 @@ void ViewParser::tag_start(const QString &tag, const list<QString> &attrs)
     }else if (tag == "body"){ // we display as a part of a larger document
         oTag = "span";
     }else if (tag == "p"){
-        m_bPara = false;
-        if (m_bFirst){
-            m_bFirst = false;
-        }else{
-            res += "<br>";
-        }
+        bool bRTL = false;
+        m_bParaEnd = false;
         for (list<QString>::const_iterator it = attrs.begin(); it != attrs.end(); ++it){
             QString name = (*it).lower();
             ++it;
             QString value = *it;
-            if (name == "dir"){
-                if (value == "rtl"){
-                    res += "<span dir=\"rtl\">";
-                    m_bSpan = true;
-                }
-                break;
+            if ((name == "dir") && (value.lower() == "rtl"))
+                bRTL = true;
+        }
+        if (m_bPara){
+            if (bRTL == m_bRTL){
+                res += "<br/>";
+            }else{
+                if (m_bParaStart)
+                    res += "</p>";
+                res += "<p dir=\"";
+                res += bRTL ? "rtl" : "ltr";
+                res += "\">";
+                m_bParaStart = true;
             }
+        }else{
+            RTL     = bRTL;
+            m_bRTL  = bRTL;
+            m_bPara = true;
         }
         return;
     }
+    if (m_bParaEnd){
+        res += "<br/>";
+        m_bParaEnd = false;
+    }
     QString tagText;
     tagText += "<";
     tagText += oTag;
@@ -1051,10 +1377,8 @@ void ViewParser::tag_end(const QString &tag)
     }else if (tag == "body"){
         oTag = "span";
     }else if (tag == "p"){
-        m_bPara = true;
-        if (!m_bSpan)
-            return;
-        oTag = "span";
+        m_bParaEnd = true;
+        return;
     }
     if (m_bInHead)
         return;
