@@ -64,6 +64,7 @@ void UserViewItemBase::paintCell(QPainter *p, const QColorGroup &cg, int, int wi
         pv.isStatic = false;
         pv.height   = height();
         pv.margin   = 0;
+        pv.isGroup  = (type() == GRP_ITEM);
         if (CorePlugin::m_plugin->getUseSysColors()){
             pp.setPen(cg.text());
         }else{
@@ -107,11 +108,11 @@ int UserViewItemBase::drawText(QPainter *p, int x, int width, const QString &tex
 void UserViewItemBase::drawSeparator(QPainter *p, int x, int width, const QColorGroup &cg)
 {
     if (x < width - 6){
-#if QT_VERSION > 300
+#if COMPAT_QT_VERSION > 0x030000
         QRect rcSep(x, height()/2, width - 6 - x, 1);
         listView()->style().drawPrimitive(QStyle::PE_Separator, p, rcSep, cg);
 #else
-listView()->style().drawSeparator(p, x, height() / 2, width - 6, height() / 2, cg);
+        listView()->style().drawSeparator(p, x, height() / 2, width - 6, height() / 2, cg);
 #endif
     }
 }
@@ -153,9 +154,9 @@ void GroupItem::init(Group *grp)
         setOpen(true);
     }else{
         if (m_bOffline){
-            setOpen((data->OfflineOpen) != 0);
+            setOpen(data->OfflineOpen.bValue);
         }else{
-            setOpen((data->OnlineOpen) != 0);
+            setOpen(data->OnlineOpen.bValue);
         }
     }
     update(grp, true);
@@ -192,9 +193,9 @@ void GroupItem::setOpen(bool bOpen)
         ListUserData *data = (ListUserData*)(grp->getUserData(CorePlugin::m_plugin->list_data_id, !bOpen));
         if (data){
             if (m_bOffline){
-                data->OfflineOpen = bOpen;
+                data->OfflineOpen.bValue = bOpen;
             }else{
-                data->OnlineOpen = bOpen;
+                data->OnlineOpen.bValue  = bOpen;
             }
         }
     }
@@ -205,7 +206,7 @@ ContactItem::ContactItem(UserViewItemBase *view, Contact *contact, unsigned stat
 {
     m_id = contact->id();
     init(contact, status, style, icons, unread);
-#if QT_VERSION >= 300    
+#if COMPAT_QT_VERSION >= 0x030000    
     setDragEnabled(true);
 #endif
 }
@@ -271,6 +272,7 @@ UserListBase::UserListBase(QWidget *parent)
     m_bDirty = false;
     m_groupMode = 1;
     m_bShowOnline = false;
+    m_bShowEmpty  = false;
 
     header()->hide();
     addColumn("", -1);
@@ -302,10 +304,15 @@ void UserListBase::drawUpdates()
         case 1:
             item = findGroupItem(group->id());
             if (item){
-                static_cast<GroupItem*>(item)->update(group);
-                repaintItem(item);
+                if (!m_bShowEmpty && (item->firstChild() == NULL)){
+                    delete item;
+                }else{
+                    static_cast<GroupItem*>(item)->update(group);
+                    repaintItem(item);
+                }
             }else{
-                new GroupItem(this, group, true);
+                if (m_bShowEmpty)
+                    new GroupItem(this, group, true);
             }
             break;
         case 2:
@@ -315,10 +322,15 @@ void UserListBase::drawUpdates()
                 DivItem *divItem = static_cast<DivItem*>(i);
                 GroupItem *grpItem = findGroupItem(group->id(), divItem);
                 if (grpItem){
-                    grpItem->update(group);
-                    repaintItem(grpItem);
+                    if (!m_bShowEmpty && (item->firstChild() == NULL)){
+                        delete grpItem;
+                    }else{
+                        grpItem->update(group);
+                        repaintItem(grpItem);
+                    }
                 }else{
-                    new GroupItem(divItem, group, divItem->state() == DIV_OFFLINE);
+                    if (m_bShowEmpty)
+                        new GroupItem(divItem, group, divItem->state() == DIV_OFFLINE);
                 }
             }
             break;
@@ -352,11 +364,11 @@ void UserListBase::drawUpdates()
         unsigned unread = getUnread(contact->id());
         bool bShow = false;
         ListUserData *data = (ListUserData*)(contact->getUserData(CorePlugin::m_plugin->list_data_id));
-        if (data && data->ShowAlways)
+        if (data && data->ShowAlways.bValue)
             bShow = true;
         switch (m_groupMode){
         case 0:
-            if (status == STATUS_OFFLINE){
+            if (status <= STATUS_OFFLINE){
                 if (itemOnline){
                     contactItem = findContactItem(contact->id(), itemOnline);
                     if (contactItem){
@@ -422,28 +434,36 @@ void UserListBase::drawUpdates()
             grpItem = NULL;
             if (contactItem){
                 grpItem = static_cast<GroupItem*>(contactItem->parent());
-                if (((status == STATUS_OFFLINE) && (unread == 0) && !bShow && m_bShowOnline) ||
+                if (((status <= STATUS_OFFLINE) && (unread == 0) && !bShow && m_bShowOnline) ||
                         (contact->getGroup() != grpItem->id())){
                     grpItem->m_nContacts--;
                     if (contactItem->m_bOnline)
                         grpItem->m_nContactsOnline--;
                     addGroupForUpdate(grpItem->id());
                     deleteItem(contactItem);
+                    if (!m_bShowEmpty && (grpItem->firstChild() == NULL))
+                        delete grpItem;
                     contactItem = NULL;
                     grpItem = NULL;
                 }
             }
-            if ((status != STATUS_OFFLINE) || unread || bShow || !m_bShowOnline){
-                if (grpItem == NULL)
+            if ((status > STATUS_OFFLINE) || unread || bShow || !m_bShowOnline){
+                if (grpItem == NULL){
                     grpItem = findGroupItem(contact->getGroup());
+                    if (grpItem == NULL){
+                        Group *grp = getContacts()->group(contact->getGroup());
+                        if (grp)
+                            grpItem = new GroupItem(this, grp, true);
+                    }
+                }
                 if (grpItem){
                     if (contactItem){
                         if (contactItem->update(contact, status, style, icons.c_str(), unread))
                             addSortItem(grpItem);
                         repaintItem(contactItem);
                         if (!m_bShowOnline &&
-                                (contactItem->m_bOnline != (status != STATUS_OFFLINE))){
-                            if (status == STATUS_OFFLINE){
+                                (contactItem->m_bOnline != (status > STATUS_OFFLINE))){
+                            if (status <= STATUS_OFFLINE){
                                 grpItem->m_nContactsOnline--;
                                 contactItem->m_bOnline = false;
                             }else{
@@ -455,7 +475,7 @@ void UserListBase::drawUpdates()
                     }else{
                         contactItem = new ContactItem(grpItem, contact, status, style, icons.c_str(), unread);
                         grpItem->m_nContacts++;
-                        if (!m_bShowOnline && (status != STATUS_OFFLINE)){
+                        if (!m_bShowOnline && (status > STATUS_OFFLINE)){
                             grpItem->m_nContactsOnline++;
                             contactItem->m_bOnline = true;
                         }
@@ -469,10 +489,13 @@ void UserListBase::drawUpdates()
             grpItem = NULL;
             if (contactItem){
                 grpItem = static_cast<GroupItem*>(contactItem->parent());
-                if ((status == STATUS_OFFLINE) || (grpItem->id() != contact->getGroup())){
+                if ((status <= STATUS_OFFLINE) || (grpItem->id() != contact->getGroup())){
                     grpItem->m_nContacts--;
                     addGroupForUpdate(grpItem->id());
                     deleteItem(contactItem);
+                    if (!m_bShowEmpty && (grpItem->firstChild() == NULL))
+                        delete grpItem;
+                    grpItem = NULL;
                     contactItem = NULL;
                 }
             }
@@ -481,7 +504,7 @@ void UserListBase::drawUpdates()
                 grpItem = NULL;
                 if (contactItem){
                     grpItem = static_cast<GroupItem*>(contactItem->parent());
-                    if ((status != STATUS_OFFLINE) || (grpItem->id() != contact->getGroup())){
+                    if ((status > STATUS_OFFLINE) || (grpItem->id() != contact->getGroup())){
                         grpItem->m_nContacts--;
                         addGroupForUpdate(grpItem->id());
                         deleteItem(contactItem);
@@ -497,10 +520,10 @@ void UserListBase::drawUpdates()
                     }
                 }
             }
-            if ((unread == 0) && !bShow && (status == STATUS_OFFLINE) && m_bShowOnline)
+            if ((unread == 0) && !bShow && (status <= STATUS_OFFLINE) && m_bShowOnline)
                 break;
             DivItem *divItem;
-            if (status == STATUS_OFFLINE){
+            if (status <= STATUS_OFFLINE){
                 if (itemOffline == NULL){
                     itemOffline = new DivItem(this, DIV_OFFLINE);
                     setOpen(itemOffline, true);
@@ -515,6 +538,7 @@ void UserListBase::drawUpdates()
                 if (grp == NULL)
                     break;
                 grpItem = new GroupItem(divItem, grp, true);
+                addSortItem(divItem);
             }
             contactItem = findContactItem(contact->id(), grpItem);
             if (contactItem){
@@ -630,13 +654,13 @@ void UserListBase::fill()
             unsigned unread = getUnread(contact->id());
             bool bShow = false;
             ListUserData *data = (ListUserData*)contact->getUserData(CorePlugin::m_plugin->list_data_id);
-            if (data && data->ShowAlways)
+            if (data && data->ShowAlways.bValue)
                 bShow = true;
-            if ((unread == 0) && !bShow && (status == STATUS_OFFLINE) && m_bShowOnline)
+            if ((unread == 0) && !bShow && (status <= STATUS_OFFLINE) && m_bShowOnline)
                 continue;
-            divItem = (status == STATUS_OFFLINE) ? divItemOffline : divItemOnline;
+            divItem = (status <= STATUS_OFFLINE) ? divItemOffline : divItemOnline;
             if (divItem == NULL){
-                if (status == STATUS_OFFLINE){
+                if (status <= STATUS_OFFLINE){
                     divItemOffline = new DivItem(this, DIV_OFFLINE);
                     setOpen(divItemOffline, true);
                     divItem = divItemOffline;
@@ -650,14 +674,13 @@ void UserListBase::fill()
         }
         break;
     case 1:
-        while ((grp = ++grp_it) != NULL){
-            if (grp->id() == 0)
-                continue;
-            grpItem = new GroupItem(this, grp, true);
-        }
-        grp = list->group(0);
-        if (grp){
-            grpItem = new GroupItem(this, grp, true);
+        if (m_bShowEmpty){
+            while ((grp = ++grp_it) != NULL){
+                if (grp->id() == 0)
+                    continue;
+                grpItem = new GroupItem(this, grp, true);
+            }
+            grpItem = new GroupItem(this, list->group(0), true);
         }
         while ((contact = ++contact_it) != NULL){
             if (contact->getIgnore() || contact->getTemporary())
@@ -668,16 +691,21 @@ void UserListBase::fill()
             unsigned unread = getUnread(contact->id());
             bool bShow = false;
             ListUserData *data = (ListUserData*)contact->getUserData(CorePlugin::m_plugin->list_data_id);
-            if (data && data->ShowAlways)
+            if (data && data->ShowAlways.bValue)
                 bShow = true;
-            if ((status == STATUS_OFFLINE) && !bShow && (unread == 0) && m_bShowOnline)
+            if ((status <= STATUS_OFFLINE) && !bShow && (unread == 0) && m_bShowOnline)
                 continue;
             grpItem = findGroupItem(contact->getGroup());
-            if (grpItem == NULL)
-                continue;
+            if (grpItem == NULL){
+                grp = list->group(contact->getGroup());
+                if (grp)
+                    grpItem = new GroupItem(this, grp, true);
+                if (grpItem == NULL)
+                    continue;
+            }
             contactItem = new ContactItem(grpItem, contact, status, style, icons.c_str(), unread);
             grpItem->m_nContacts++;
-            if ((status != STATUS_OFFLINE) && !m_bShowOnline){
+            if ((status > STATUS_OFFLINE) && !m_bShowOnline){
                 grpItem->m_nContactsOnline++;
                 contactItem->m_bOnline = true;
             }
@@ -686,27 +714,25 @@ void UserListBase::fill()
     case 2:
         divItemOnline = new DivItem(this, DIV_ONLINE);
         setOpen(divItemOnline, true);
-        while ((grp = ++grp_it) != NULL){
-            if (grp->id() == 0)
-                continue;
-            grpItem = new GroupItem(divItemOnline, grp, false);
-        }
-        grp = list->group(0);
-        if (grp){
-            grpItem = new GroupItem(divItemOnline, grp, false);
+        if (m_bShowEmpty){
+            while ((grp = ++grp_it) != NULL){
+                if (grp->id() == 0)
+                    continue;
+                grpItem = new GroupItem(divItemOnline, grp, false);
+            }
+            grpItem = new GroupItem(divItemOnline, list->group(0), false);
         }
         if (!m_bShowOnline){
             divItemOffline = new DivItem(this, DIV_OFFLINE);
             setOpen(divItemOffline, true);
             grp_it.reset();
-            while ((grp = ++grp_it) != NULL){
-                if (grp->id() == 0)
-                    continue;
-                grpItem = new GroupItem(divItemOffline, grp, true);
-            }
-            grp = list->group(0);
-            if (grp){
-                grpItem = new GroupItem(divItemOffline, grp, true);
+            if (m_bShowEmpty){
+                while ((grp = ++grp_it) != NULL){
+                    if (grp->id() == 0)
+                        continue;
+                    grpItem = new GroupItem(divItemOffline, grp, true);
+                }
+                grpItem = new GroupItem(divItemOnline, list->group(0), true);
             }
         }
         while ((contact = ++contact_it) != NULL){
@@ -718,11 +744,11 @@ void UserListBase::fill()
             unsigned unread = getUnread(contact->id());
             bool bShow = false;
             ListUserData *data = (ListUserData*)contact->getUserData(CorePlugin::m_plugin->list_data_id);
-            if (data && data->ShowAlways)
+            if (data && data->ShowAlways.bValue)
                 bShow = true;
-            if ((unread == 0) && !bShow && (status == STATUS_OFFLINE) && m_bShowOnline)
+            if ((unread == 0) && !bShow && (status <= STATUS_OFFLINE) && m_bShowOnline)
                 continue;
-            if (status == STATUS_OFFLINE){
+            if (status <= STATUS_OFFLINE){
                 if (divItemOffline == NULL){
                     divItemOffline = new DivItem(this, DIV_OFFLINE);
                     setOpen(divItemOffline, true);
@@ -1030,7 +1056,7 @@ bool UserList::isGroupSelected(unsigned id)
     return bRes;
 }
 
-#if QT_VERSION < 300
+#if COMPAT_QT_VERSION < 0x030000
 #define CHECK_OFF	QButton::Off
 #define CHECK_ON	QButton::On
 #define CHECK_NOCHANGE	QButton::NoChange
@@ -1043,7 +1069,7 @@ bool UserList::isGroupSelected(unsigned id)
 int UserList::drawIndicator(QPainter *p, int x, QListViewItem *item, bool bState, const QColorGroup &cg)
 {
     int state = bState ? CHECK_ON : CHECK_OFF;
-#if QT_VERSION < 300
+#if COMPAT_QT_VERSION < 0x030000
     QSize s = style().indicatorSize();
     QPixmap pixInd(s.width(), s.height());
     QPainter pInd(&pixInd);
