@@ -56,6 +56,7 @@ UserView::UserView()
     m_bBlink = false;
     m_bUnreadBlink = false;
     m_bShowOnline = CorePlugin::m_plugin->getShowOnLine();
+    m_bShowEmpty = CorePlugin::m_plugin->getShowEmptyGroup();
 
     setBackgroundMode(NoBackground);
     viewport()->setBackgroundMode(NoBackground);
@@ -96,7 +97,7 @@ UserView::UserView()
     edtContact = new IntLineEdit(viewport());
     edtGroup->hide();
     edtContact->hide();
-    QFont font(font());
+    QFont font;
     int size = font.pixelSize();
     if (size <= 0){
         size = font.pointSize();
@@ -137,6 +138,7 @@ void UserView::paintEmptyArea(QPainter *p, const QRect &r)
     pv.isStatic = false;
     pv.height   = r.height();
     pv.margin   = 0;
+    pv.isGroup  = false;
     QListViewItem *item = firstChild();
     if (item)
         pv.height = item->height();
@@ -152,12 +154,14 @@ void UserView::drawItem(UserViewItemBase *base, QPainter *p, const QColorGroup &
     if (base->type() == GRP_ITEM){
         GroupItem *item = static_cast<GroupItem*>(base);
         QFont f(font());
-        int size = f.pixelSize();
-        if (size <= 0){
-            size = f.pointSize();
-            f.setPointSize(size * 3 / 4);
-        }else{
-            f.setPixelSize(size * 3 / 4);
+        if (CorePlugin::m_plugin->getSmallGroupFont()){
+            int size = f.pixelSize();
+            if (size <= 0){
+                size = f.pointSize();
+                f.setPointSize(size * 3 / 4);
+            }else{
+                f.setPixelSize(size * 3 / 4);
+            }
         }
         f.setBold(true);
         p->setFont(f);
@@ -384,8 +388,8 @@ void *UserView::processEvent(Event *e)
                             bool bShow = false;
                             if (cmd->flags & COMMAND_CHECKED)
                                 bShow = true;
-                            if ((data->ShowAlways != 0) != bShow){
-                                data->ShowAlways = bShow;
+                            if (data->ShowAlways.bValue != bShow){
+                                data->ShowAlways.bValue = bShow;
                                 Event e(EventContactChanged, contact);
                                 e.process();
                             }
@@ -466,12 +470,13 @@ void *UserView::processEvent(Event *e)
                         from->removeUserWnd(userWnd);
                     }
                     if (userWnd == NULL)
-                        userWnd = new UserWnd(contact->id(), NULL, true);
+                        userWnd = new UserWnd(contact->id(), NULL, true, true);
                     if (to == NULL)
                         to = new Container(max_id + 1);
                     to->addUserWnd(userWnd, true);
-                    to->setNoSwitch();
+                    to->setNoSwitch(true);
                     raiseWindow(to);
+                    to->setNoSwitch(false);
                 }
                 return e->param();
             }
@@ -487,6 +492,11 @@ void *UserView::processEvent(Event *e)
                 }
                 fill();
             }
+            if (cmd->id == CmdEmptyGroup){
+                CorePlugin::m_plugin->setShowEmptyGroup((cmd->flags & COMMAND_CHECKED) != 0);
+                m_bShowEmpty = (cmd->flags & COMMAND_CHECKED);
+                fill();
+            }
             if (cmd->id == CmdGrpOff)
                 setGroupMode(0);
             if (cmd->id == CmdGrpMode1)
@@ -558,6 +568,12 @@ void *UserView::processEvent(Event *e)
                         ((cmd->id == CmdGrpMode2) && (CorePlugin::m_plugin->getGroupMode() == 2)) ||
                         ((cmd->id == CmdOnline) && CorePlugin::m_plugin->getShowOnLine()))
                     cmd->flags |= COMMAND_CHECKED;
+                if (cmd->id == CmdEmptyGroup){
+                    if (CorePlugin::m_plugin->getGroupMode() == 0)
+                        return NULL;
+                    if (CorePlugin::m_plugin->getShowEmptyGroup())
+                        cmd->flags |= COMMAND_CHECKED;
+                }
                 return e->param();
             }
             if (cmd->menu_id == MenuContact){
@@ -573,7 +589,7 @@ void *UserView::processEvent(Event *e)
                     if (contact){
                         ListUserData *data = (ListUserData*)(contact->getUserData(CorePlugin::m_plugin->list_data_id, true));
                         cmd->flags &= ~COMMAND_CHECKED;
-                        if (data && data->ShowAlways)
+                        if (data && data->ShowAlways.bValue)
                             cmd->flags |= COMMAND_CHECKED;
                         return e->param();
                     }
@@ -1408,7 +1424,7 @@ void UserView::dragEvent(QDropEvent *e, bool isDrop)
             if (msg){
                 if (isDrop){
                     msg->setContact(static_cast<ContactItem*>(item)->id());
-                    Event e(EventOpenMessage, msg);
+                    Event e(EventOpenMessage, &msg);
                     e.process();
                 }
                 delete msg;
@@ -1422,7 +1438,7 @@ void UserView::dragEvent(QDropEvent *e, bool isDrop)
                         Message *msg = new Message(MessageGeneric);
                         msg->setText(str);
                         msg->setContact(static_cast<ContactItem*>(item)->id());
-                        Event e(EventOpenMessage, msg);
+                        Event e(EventOpenMessage, &msg);
                         e.process();
                         delete msg;
                     }
@@ -1484,6 +1500,20 @@ void UserView::joinContacts(void*)
     if ((contact1 == NULL) || (contact2 == NULL))
         return;
     contact1->clientData.join(contact2->clientData);
+    if (!contact2->getPhones().isEmpty()){
+        QString phones = contact1->getPhones();
+        if (!phones.isEmpty())
+            phones += ";";
+        phones += contact2->getPhones();
+        contact1->setPhones(phones);
+    }
+    if (!contact2->getEMails().isEmpty()){
+        QString mails = contact1->getEMails();
+        if (!mails.isEmpty())
+            mails += ";";
+        mails += contact2->getEMails();
+        contact1->setEMails(mails);
+    }
     delete contact2;
     contact1->setup();
     Event e(EventContactChanged, contact1);
