@@ -28,6 +28,7 @@
 #include <qtimer.h>
 #include <qapplication.h>
 #include <qvaluelist.h>
+#include <qregexp.h>
 
 #ifdef WIN32
 #include <windows.h>
@@ -43,7 +44,7 @@
 #endif
 
 #ifndef WIN32
-#ifndef QT_MACOSX_VERSION
+#if !defined(QT_MACOSX_VERSION) && !defined(QT_MAC)
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
@@ -59,7 +60,7 @@ static DockWnd *gDock;
 
 LRESULT CALLBACK DockWindowProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
 {
-    if (msg == WM_DOCK){
+    if ((msg == WM_DOCK) || (msg == 0)){
         gDock->callProc(lParam);
     }
     if (oldDockProc)
@@ -70,14 +71,12 @@ LRESULT CALLBACK DockWindowProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lPara
 void DockWnd::callProc(unsigned long param)
 {
     switch (param){
-    case WM_RBUTTONDOWN:
-        POINT pos;
-        GetCursorPos(&pos);
-        emit showPopup(QPoint(pos.x, pos.y));
+    case WM_RBUTTONUP:
+        QTimer::singleShot(0, this, SLOT(showPopup()));
         return;
     case WM_LBUTTONDBLCLK:
         bNoToggle = true;
-        emit doubleClicked();
+        QTimer::singleShot(0, this, SLOT(dbl_click()));
         return;
     case WM_LBUTTONDOWN:
         if (!bNoToggle)
@@ -86,9 +85,22 @@ void DockWnd::callProc(unsigned long param)
     }
 }
 
+void DockWnd::showPopup()
+{
+    POINT pos;
+    GetCursorPos(&pos);
+    if (qApp->activeWindow() == NULL)
+        setFocus();
+    emit showPopup(QPoint(pos.x, pos.y));
+}
+
 #else
 
-#ifndef QT_MACOSX_VERSION
+void DockWnd::showPopup()
+{
+}
+
+#if !defined(QT_MACOSX_VERSION) && !defined(QT_MAC)
 
 extern Time qt_x_time;
 
@@ -105,7 +117,9 @@ protected:
     virtual void mouseDoubleClickEvent(QMouseEvent *e);
     virtual void paintEvent(QPaintEvent *);
     virtual void enterEvent(QEvent*);
+#if !defined(QT_MACOSX_VERSION) && !defined(QT_MAC)
     virtual bool x11Event(XEvent*);
+#endif
     unsigned p_width;
     unsigned p_height;
     Window  parentWin;
@@ -119,7 +133,7 @@ WharfIcon::WharfIcon(DockWnd *parent)
     p_width  = 64;
     p_height = 64;
     setMouseTracking(true);
-    const QIconSet *icon = BigIcon("inactive");
+    const QIconSet *icon = Icon("inactive");
     if (icon){
         const QPixmap &pict = icon->pixmap(QIconSet::Large, QIconSet::Normal);
         setIcon(pict);
@@ -488,7 +502,7 @@ MyPixmap::MyPixmap(Pixmap pp, int w, int h)
         : QPixmap(w, h)
 {
     data->uninit = false;
-#if QT_VERSION >= 300
+#if COMPAT_QT_VERSION >= 0x030000
     Screen *screen =  XDefaultScreenOfDisplay(dd);
     int scr = XScreenNumberOfScreen(screen);
     x11SetScreen(scr);
@@ -563,7 +577,7 @@ DockWnd::DockWnd(DockPlugin *plugin, const char *icon, const char *text)
         EventReceiver(LowPriority)
 {
 #ifndef WIN32
-#ifndef QT_MACOSX_VERSION
+#if !defined(QT_MACOSX_VERSION) && !defined(QT_MAC)
     wharfIcon = NULL;
 #endif
 #endif
@@ -586,31 +600,34 @@ DockWnd::DockWnd(DockPlugin *plugin, const char *icon, const char *text)
     }
     if (IsWindowUnicode(winId()) && _Shell_NotifyIconW){
         oldDockProc = (WNDPROC)SetWindowLongW(winId(), GWL_WNDPROC, (LONG)DockWindowProc);
-        NOTIFYICONDATAW notifyIconData;
-        notifyIconData.cbSize = sizeof(notifyIconData);
-        notifyIconData.hIcon = topData()->winIcon;
-        notifyIconData.hWnd = winId();
-        notifyIconData.szTip[0] = 0;
-        notifyIconData.uCallbackMessage = WM_DOCK;
-        notifyIconData.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP;
-        notifyIconData.uID = 0;
-        _Shell_NotifyIconW(NIM_ADD, &notifyIconData);
+        /*
+                NOTIFYICONDATAW notifyIconData;
+                notifyIconData.cbSize = sizeof(notifyIconData);
+                notifyIconData.hIcon = topData()->winIcon;
+                notifyIconData.hWnd = winId();
+                notifyIconData.szTip[0] = 0;
+                notifyIconData.uCallbackMessage = WM_DOCK;
+                notifyIconData.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP;
+                notifyIconData.uID = 0;
+                _Shell_NotifyIconW(NIM_ADD, &notifyIconData);
+        */
     }else{
         oldDockProc = (WNDPROC)SetWindowLongA(winId(), GWL_WNDPROC, (LONG)DockWindowProc);
-        NOTIFYICONDATAA notifyIconData;
-        notifyIconData.cbSize = sizeof(notifyIconData);
-        notifyIconData.hIcon = topData()->winIcon;
-        notifyIconData.hWnd = winId();
-        notifyIconData.szTip[0] = 0;
-        notifyIconData.uCallbackMessage = WM_DOCK;
-        notifyIconData.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP;
-        notifyIconData.uID = 0;
-        Shell_NotifyIconA(NIM_ADD, &notifyIconData);
     }
+    NOTIFYICONDATAA notifyIconData;
+    notifyIconData.cbSize = sizeof(notifyIconData);
+    notifyIconData.hIcon = topData()->winIcon;
+    notifyIconData.hWnd = winId();
+    notifyIconData.szTip[0] = 0;
+    notifyIconData.uCallbackMessage = WM_DOCK;
+    notifyIconData.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP;
+    notifyIconData.uID = 0;
+    Shell_NotifyIconA(NIM_ADD, &notifyIconData);
+    //}
 #else
     setMinimumSize(22, 22);
     resize(22, 22);
-#ifndef QT_MACOSX_VERSION
+#if !defined(QT_MACOSX_VERSION) && !defined(QT_MAC)
     bInit = false;
     inTray = false;
     inNetTray = false;
@@ -711,7 +728,7 @@ DockWnd::DockWnd(DockPlugin *plugin, const char *icon, const char *text)
     setBackgroundMode(X11ParentRelative);
     setIcon(icon);
 
-#ifndef QT_MACOSX_VERSION
+#if !defined(QT_MACOSX_VERSION) && !defined(QT_MAC)
     XClassHint classhint;
     classhint.res_name  = (char*)"sim";
     classhint.res_class = (char*)"Wharf";
@@ -807,6 +824,11 @@ void DockWnd::quit()
 #endif
 }
 
+void DockWnd::dbl_click()
+{
+    emit doubleClicked();
+}
+
 void *DockWnd::processEvent(Event *e)
 {
     switch (e->type()){
@@ -838,7 +860,7 @@ void *DockWnd::processEvent(Event *e)
 }
 
 #ifndef WIN32
-#ifndef QT_MACOSX_VERSION
+#if !defined(QT_MACOSX_VERSION) && !defined(QT_MAC)
 
 bool DockWnd::x11Event(XEvent *e)
 {
@@ -894,7 +916,7 @@ bool DockWnd::x11Event(XEvent *e)
 void DockWnd::paintEvent( QPaintEvent* )
 {
 #ifndef WIN32
-#ifndef QT_MACOSX_VERSION
+#if !defined(QT_MACOSX_VERSION) && !defined(QT_MAC)
     if (!bInit) return;
 #endif
 #endif
@@ -905,15 +927,18 @@ void DockWnd::paintEvent( QPaintEvent* )
 void DockWnd::setIcon(const char *icon)
 {
 #ifndef WIN32
-#ifndef QT_MACOSX_VERSION
+#if !defined(QT_MACOSX_VERSION) && !defined(QT_MAC)
     if (wharfIcon)
         wharfIcon->set(icon, NULL);
 #endif
 #endif
     drawIcon = Pict(icon);
 #ifndef WIN32
-#ifndef QT_MACOSX_VERSION
-    if (!inTray) return;
+#if !defined(QT_MACOSX_VERSION) && !defined(QT_MAC)
+    if (!inTray){
+        repaint();
+        return;
+    }
 #endif
 #endif
 #ifdef WIN32
@@ -940,7 +965,7 @@ void DockWnd::setIcon(const char *icon)
         Shell_NotifyIconA(NIM_MODIFY, &notifyIconData);
     }
 #else
-#ifndef QT_MACOSX_VERSION
+#if !defined(QT_MACOSX_VERSION) && !defined(QT_MAC)
     if (wharfIcon)
         return;
 #endif
@@ -952,8 +977,10 @@ void DockWnd::setTip(const char *text)
 {
     m_tip = text;
     QString tip = m_unreadText;
-    if (tip.isEmpty())
+    if (tip.isEmpty()){
         tip = i18n(text);
+        tip = tip.replace(QRegExp("\\&"), "");
+    }
 #ifdef WIN32
     if (_Shell_NotifyIconW && IsWindowUnicode(winId())){
         NOTIFYICONDATAW notifyIconData;
@@ -981,13 +1008,13 @@ void DockWnd::setTip(const char *text)
         Shell_NotifyIconA(NIM_MODIFY, &notifyIconData);
     }
 #else
-#ifndef QT_MACOSX_VERSION
+#if !defined(QT_MACOSX_VERSION) && !defined(QT_MAC)
     if (wharfIcon == NULL){
         if (isVisible()){
 #endif
             QToolTip::remove(this);
             QToolTip::add(this, tip);
-#ifndef QT_MACOSX_VERSION
+#if !defined(QT_MACOSX_VERSION) && !defined(QT_MAC)
         }
     }else{
         if (wharfIcon->isVisible()){
@@ -1030,7 +1057,7 @@ void DockWnd::mousePressEvent( QMouseEvent *e)
 {
     QWidget::mousePressEvent(e);
 #ifndef WIN32
-#ifndef QT_MACOSX_VERSION
+#if !defined(QT_MACOSX_VERSION) && !defined(QT_MAC)
     if (inTray || wharfIcon) return;
 #endif
     grabMouse();
@@ -1042,7 +1069,7 @@ void DockWnd::mouseReleaseEvent( QMouseEvent *e)
 {
     QWidget::mouseReleaseEvent(e);
 #ifndef WIN32
-#ifndef QT_MACOSX_VERSION
+#if !defined(QT_MACOSX_VERSION) && !defined(QT_MAC)
     if (!inTray && (wharfIcon == NULL)){
 #endif
         releaseMouse();
@@ -1055,7 +1082,7 @@ void DockWnd::mouseReleaseEvent( QMouseEvent *e)
             if (p.manhattanLength() > 6)
                 return;
         }
-#ifndef QT_MACOSX_VERSION
+#if !defined(QT_MACOSX_VERSION) && !defined(QT_MAC)
     }
 #endif
 #endif
@@ -1066,7 +1093,7 @@ void DockWnd::mouseMoveEvent( QMouseEvent *e)
 {
     QWidget::mouseMoveEvent(e);
 #ifndef WIN32
-#ifndef QT_MACOSX_VERSION
+#if !defined(QT_MACOSX_VERSION) && !defined(QT_MAC)
     if (inTray || wharfIcon) return;
 #endif
     if (mousePos.isNull()) return;
@@ -1083,7 +1110,7 @@ void DockWnd::mouseDoubleClickEvent( QMouseEvent*)
 void DockWnd::enterEvent( QEvent* )
 {
 #ifndef WIN32
-#ifndef QT_MACOSX_VERSION
+#if !defined(QT_MACOSX_VERSION) && !defined(QT_MAC)
     if (wharfIcon != NULL) return;
     if ( !qApp->focusWidget() ) {
         XEvent ev;
