@@ -146,125 +146,129 @@ class DirectClient;
 
 typedef struct ICQUserData
 {
-    clientData		base;
-    char            *Alias;
-    char            *Cellular;
-    unsigned long    Status;
-    unsigned long	 Class;
-    unsigned long    StatusTime;
-    unsigned long    OnlineTime;
-    unsigned long	 WarningLevel;
-    void            *IP;
-    void            *RealIP;
-    unsigned long    Port;
-    unsigned long    DCcookie;
-    unsigned long    Caps;
-    char            *AutoReply;
-    unsigned long    Uin;
-    char            *Screen;
-    unsigned long    IcqID;
-    unsigned long    bChecked;
-    unsigned long    GrpId;
-    unsigned long    IgnoreId;
-    unsigned long    VisibleId;
-    unsigned long    ContactVisibleId;
-    unsigned long    InvisibleId;
-    unsigned long    ContactInvisibleId;
-    unsigned long    WaitAuth;
-    unsigned long    WantAuth;
-    unsigned long    WebAware;
-    unsigned long    InfoUpdateTime;
-    unsigned long    PluginInfoTime;
-    unsigned long    PluginStatusTime;
-    unsigned long    InfoFetchTime;
-    unsigned long    PluginInfoFetchTime;
-    unsigned long    PluginStatusFetchTime;
-    unsigned long    Mode;
-    unsigned long    Version;
-    unsigned long    Build;
-    char            *Nick;
-    char            *FirstName;
-    char            *LastName;
-    char			*MiddleName;
-    char			*Maiden;
-    char            *EMail;
-    unsigned long    HiddenEMail;
-    char            *City;
-    char            *State;
-    char            *HomePhone;
-    char            *HomeFax;
-    char            *Address;
-    char            *PrivateCellular;
-    char            *Zip;
-    unsigned long    Country;
-    unsigned long    TimeZone;
-    unsigned long    Age;
-    unsigned long    Gender;
-    char            *Homepage;
-    unsigned long    BirthYear;
-    unsigned long    BirthMonth;
-    unsigned long    BirthDay;
-    unsigned long    Language;
-    char            *EMails;
-    char            *WorkCity;
-    char            *WorkState;
-    char            *WorkPhone;
-    char            *WorkFax;
-    char            *WorkAddress;
-    char            *WorkZip;
-    unsigned long    WorkCountry;
-    char            *WorkName;
-    char            *WorkDepartment;
-    char            *WorkPosition;
-    unsigned long    Occupation;
-    char            *WorkHomepage;
-    char            *About;
-    char            *Interests;
-    char            *Backgrounds;
-    char            *Affilations;
-    unsigned long    FollowMe;
-    unsigned long    SharedFiles;
-    unsigned long    ICQPhone;
-    char            *Encoding;
-    char            *Picture;
-    unsigned long    PictureWidth;
-    unsigned long    PictureHeight;
-    char            *PhoneBook;
-    unsigned long	 ProfileFetch;
-    unsigned long    bTyping;
-    unsigned long    bBadClient;
-    DirectClient    *Direct;
-    DirectClient    *DirectPluginInfo;
-    DirectClient    *DirectPluginStatus;
-    unsigned long    bNoDirect;
-    unsigned long    bInvisible;
+    clientData	base;
+    Data        Alias;
+    Data        Cellular;
+    Data		Status;
+    Data		Class;
+    Data		StatusTime;
+    Data		OnlineTime;
+    Data		WarningLevel;
+    Data        IP;
+    Data        RealIP;
+    Data		Port;
+    Data		DCcookie;
+    Data		Caps;
+    Data        AutoReply;
+    Data		Uin;
+    Data        Screen;
+    Data		IcqID;
+    Data		bChecked;
+    Data		GrpId;
+    Data		IgnoreId;
+    Data		VisibleId;
+    Data		ContactVisibleId;
+    Data		InvisibleId;
+    Data		ContactInvisibleId;
+    Data		WaitAuth;
+    Data		WantAuth;
+    Data		WebAware;
+    Data		InfoUpdateTime;
+    Data		PluginInfoTime;
+    Data		PluginStatusTime;
+    Data		InfoFetchTime;
+    Data		PluginInfoFetchTime;
+    Data		PluginStatusFetchTime;
+    Data		Mode;
+    Data		Version;
+    Data		Build;
+    Data        Nick;
+    Data        FirstName;
+    Data        LastName;
+    Data		MiddleName;
+    Data		Maiden;
+    Data        EMail;
+    Data		HiddenEMail;
+    Data        City;
+    Data        State;
+    Data        HomePhone;
+    Data        HomeFax;
+    Data        Address;
+    Data        PrivateCellular;
+    Data        Zip;
+    Data		Country;
+    Data		TimeZone;
+    Data		Age;
+    Data		Gender;
+    Data        Homepage;
+    Data		BirthYear;
+    Data		BirthMonth;
+    Data		BirthDay;
+    Data		Language;
+    Data        EMails;
+    Data        WorkCity;
+    Data        WorkState;
+    Data        WorkPhone;
+    Data        WorkFax;
+    Data        WorkAddress;
+    Data        WorkZip;
+    Data		WorkCountry;
+    Data        WorkName;
+    Data        WorkDepartment;
+    Data        WorkPosition;
+    Data		Occupation;
+    Data        WorkHomepage;
+    Data        About;
+    Data        Interests;
+    Data        Backgrounds;
+    Data        Affilations;
+    Data		FollowMe;
+    Data		SharedFiles;
+    Data		ICQPhone;
+    Data        Encoding;
+    Data        Picture;
+    Data		PictureWidth;
+    Data		PictureHeight;
+    Data        PhoneBook;
+    Data		ProfileFetch;
+    Data		bTyping;
+    Data		bBadClient;
+    Data		Direct;
+    Data		DirectPluginInfo;
+    Data		DirectPluginStatus;
+    Data		bNoDirect;
+    Data		bInvisible;
 } ICQUserData;
 
 typedef struct ICQClientData
 {
-    char        *Server;
-    unsigned    Port;
-    unsigned    ContactsTime;
-    unsigned    ContactsLength;
-    unsigned    ContactsInvisible;
-    unsigned    HideIP;
-    unsigned    IgnoreAuth;
-    unsigned    DirectMode;
-    unsigned    IdleTime;
-    char        *ListRequests;
-    char        *Picture;
-    unsigned    RandomChatGroup;
-    unsigned    RandomChatGroupCurrent;
-    unsigned    SendFormat;
-    unsigned    AutoUpdate;
-    unsigned    AutoReplyUpdate;
-    unsigned    TypingNotification;
-    unsigned    AcceptInDND;
-    unsigned    AcceptInOccupied;
-    unsigned	MinPort;
-    unsigned	MaxPort;
-    unsigned	WarnAnonimously;
-    ICQUserData    owner;
+    Data        Server;
+    Data		Port;
+    Data		ContactsTime;
+    Data		ContactsLength;
+    Data		ContactsInvisible;
+    Data		HideIP;
+    Data		IgnoreAuth;
+    Data		DirectMode;
+    Data		IdleTime;
+    Data        ListRequests;
+    Data        Picture;
+    Data		RandomChatGroup;
+    Data		RandomChatGroupCurrent;
+    Data		SendFormat;
+    Data		AutoUpdate;
+    Data		AutoReplyUpdate;
+    Data		TypingNotification;
+    Data		AcceptInDND;
+    Data		AcceptInOccupied;
+    Data		MinPort;
+    Data		MaxPort;
+    Data		WarnAnonimously;
+    Data		AckMode;
+    Data		UseHTTP;
+    Data		AutoHTTP;
+    Data		KeepAlive;
+    ICQUserData	owner;
 } ICQClientData;
 
 class ICQClient;
@@ -324,16 +328,6 @@ const unsigned PLUGIN_AIM_FT_ACK		 = 17;
 
 class ICQClient;
 
-typedef struct ENCODING
-{
-    const char *language;
-    const char *codec;
-    int         mib;
-    int			rtf_code;
-    int			cp_code;
-    bool        bMain;
-} ENCODING;
-
 const unsigned LIST_USER_CHANGED    = 0;
 const unsigned LIST_USER_DELETED    = 1;
 const unsigned LIST_GROUP_CHANGED    = 2;
@@ -432,19 +426,16 @@ protected:
     void snac(unsigned short fam, unsigned short type, bool msgId=false, bool bType=true);
     void connect_ready();
     void packet_ready();
+    void write_ready();
     bool m_bHeader;
     char m_nChannel;
+    Buffer   delayed;
+    unsigned m_time;
+    unsigned m_packets;
     unsigned short m_nSequence;
     unsigned short m_nMsgSequence;
 };
 
-class my_string : public string
-{
-public:
-my_string(const char *str) : string(str) {}
-    bool operator < (const my_string &str) const;
-};
-
 typedef struct alias_group
 {
     string		alias;
@@ -453,7 +444,7 @@ typedef struct alias_group
 
 typedef map<my_string, alias_group> CONTACTS_MAP;
 
-class ICQClient : public TCPClient, public EventReceiver, public OscarSocket
+class ICQClient : public TCPClient, public OscarSocket
 {
     Q_OBJECT
 public:
@@ -466,7 +457,7 @@ public:
     void setUin(unsigned long);
     void setScreen(const char*);
     unsigned long getUin();
-    const char *getServer();
+    const char *getServer() const;
     void setServer(const char*);
     PROP_USHORT(Port);
     PROP_ULONG(ContactsTime);
@@ -489,6 +480,10 @@ public:
     PROP_USHORT(MinPort);
     PROP_USHORT(MaxPort);
     PROP_BOOL(WarnAnonimously);
+    PROP_USHORT(AckMode);
+    PROP_BOOL(UseHTTP);
+    PROP_BOOL(AutoHTTP);
+    PROP_BOOL(KeepAlive);
     ICQClientData    data;
     QString toUnicode(const char *str, ICQUserData *client_data);
     string fromUnicode(const QString &str, ICQUserData *client_data);
@@ -504,7 +499,7 @@ public:
                           unsigned short nAffiliation, const char *szAffiliation,
                           unsigned short nHomePoge, const char *szHomePage,
                           bool bOnlineOnly);
-    ICQUserData *findContact(const char *screen, const char *alias, bool bCreate, Contact *&contact, Group *grp=NULL);
+    ICQUserData *findContact(const char *screen, const char *alias, bool bCreate, Contact *&contact, Group *grp=NULL, bool bJoin=true);
     ICQUserData *findGroup(unsigned id, const char *name, Group *&group);
     void addFullInfoRequest(unsigned long uin, bool bInLast = true);
     ListRequest *findContactListRequest(const char *screen);
@@ -535,13 +530,15 @@ public:
     static QString pictureFile(ICQUserData *data);
     static const capability *capabilities;
     static const plugin *plugins;
-    static const ENCODING *encodings;
     static QString convert(Tlv *tlvInfo, TlvList &tlvs, unsigned n);
     static QString convert(const char *text, unsigned size, TlvList &tlvs, unsigned n);
     string screen(ICQUserData*);
     static unsigned warnLevel(unsigned short);
     static QString addCRLF(const QString &str);
+    static unsigned clearTags(QString &text);
     bool m_bAIM;
+    void fillEncoding(QComboBox *cmb, ICQUserData *data);
+    void getEncoding(QComboBox *cmb, ICQUserData *data, bool bDefault);
 protected slots:
     void ping();
     void infoRequest();
@@ -568,6 +565,7 @@ protected:
     virtual QWidget *searchWindow();
     virtual void updateInfo(Contact *contact, void *_data);
     virtual void setClientInfo(void *data);
+    virtual Socket  *createSocket();
     virtual QString ownerName();
     virtual QString contactName(void *clientData);
     string  dataName(const char *screen);
@@ -595,6 +593,7 @@ protected:
     void sendLogonStatus();
     void sendClientReady();
     void sendMessageRequest();
+    void setServiceSocket(Tlv *tlv_addr, Tlv *tlv_cookie, unsigned short service);
     void serverRequest(unsigned short cmd, unsigned short seq=0);
     void sendServerRequest();
     void sendVisibleList();
@@ -610,6 +609,7 @@ protected:
     string cryptPassword();
     virtual void connect_ready();
     virtual void packet_ready();
+    virtual void write_ready();
     const char* error_message(unsigned short error);
     ICQListener		*m_listener;
     list<ServiceSocket*> m_services;
@@ -622,7 +622,6 @@ protected:
     unsigned m_listRequestTime;
     bool m_bRosters;
     bool m_bBirthday;
-    bool m_bServerReady;
     list<ServerRequest*> varRequests;
     list<unsigned long>  infoRequests;
     list<string>		 buddies;
@@ -648,6 +647,7 @@ protected:
     void sendPluginStatusUpdate(unsigned plugin_id, unsigned long status);
     bool m_bIdleTime;
     bool hasCap(ICQUserData *data, int fcap);
+    bool isSupportPlugins(ICQUserData *data);
     string trimPhone(const char *phone);
     unsigned short getListId();
     TlvList *createListTlv(ICQUserData *data, Contact *contact);
@@ -698,11 +698,13 @@ protected:
     void encodeString(const char *_str, unsigned short nTlv, bool bWide);
     ICQUserData *findInfoRequest(unsigned short seq, Contact *&contact);
     INFO_REQ_MAP m_info_req;
-    unsigned clearTags(QString &text);
     unsigned short msgStatus();
     unsigned short m_advCounter;
     unsigned m_nUpdates;
     unsigned m_nSendTimeout;
+    bool     m_bJoin;
+    bool	 m_bFirstTry;
+    bool	 m_bHTTP;
     SendMsg  m_send;
     list<Message*>     	m_processMsg;
     list<DirectSocket*>	m_sockets;
@@ -725,9 +727,11 @@ public:
     void connect(const char *addr, unsigned short port, const char *cookie, unsigned cookie_size);
     virtual bool error_state(const char *err, unsigned code);
     bool connected() { return m_bConnected; }
+    void close();
 protected:
     virtual void connect_ready();
     virtual void packet_ready();
+    virtual void write_ready();
     virtual ClientSocket *socket();
     virtual void packet();
     virtual void data(unsigned short fam, unsigned short type, unsigned short seq) = 0;
@@ -770,6 +774,7 @@ public:
 protected slots:
     void timeout();
 protected:
+    virtual void login_timeout();
     virtual void processPacket() = 0;
     void init();
     void sendInit();
@@ -874,11 +879,13 @@ protected:
     virtual void bind_ready(unsigned short port);
     virtual bool accept(Socket *s, unsigned long ip);
     virtual bool error(const char *err);
+    virtual void login_timeout();
 
     void sendInit();
     void startPacket(char cmd);
     void sendPacket(bool dump=true);
     void sendFileInfo();
+    void initReceive(char cmd);
 
     friend class ICQClient;
 };
