@@ -19,6 +19,7 @@
 #include "icqmessage.h"
 #include "icq.h"
 #include "core.h"
+#include "html.h"
 
 #include <stdio.h>
 #ifdef WIN32
@@ -73,9 +74,9 @@ void ICQClient::snac_icmb(unsigned short type, unsigned short seq)
             ICQUserData *data = findContact(screen.c_str(), NULL, false, contact);
             if (data == NULL)
                 break;
-            if ((data->bTyping != 0) == bType)
+            if (data->bTyping.bValue == bType)
                 break;
-            data->bTyping = bType;
+            data->bTyping.bValue = bType;
             Event e(EventContactStatus, contact);
             e.process();
             break;
@@ -102,7 +103,7 @@ void ICQClient::snac_icmb(unsigned short type, unsigned short seq)
                         sendQueue.erase(it);
                         it = sendQueue.begin();
                     }
-                    data->bBadClient = true;
+                    data->bBadClient.bValue = true;
                     if (m_send.msg)
                         sendThruServer(m_send.msg, data);
                     m_send.msg    = NULL;
@@ -118,12 +119,12 @@ void ICQClient::snac_icmb(unsigned short type, unsigned short seq)
                 if ((m_send.msg->type() == MessageCheckInvisible) && (error == 14)) {
                     Contact *contact;
                     ICQUserData *data = findContact(m_send.screen.c_str(), NULL, false, contact);
-                    if (data && (data->bInvisible == 0)) {
-                        data->bInvisible = true;
+                    if (data && !data->bInvisible.bValue) {
+                        data->bInvisible.bValue = true;
                         Event e(EventContactStatus, contact);
                         e.process();
                     }
-                } else {
+                }else{
                     m_send.msg->setError(err_str);
                     Event e(EventMessageSent, m_send.msg);
                     e.process();
@@ -148,7 +149,7 @@ void ICQClient::snac_icmb(unsigned short type, unsigned short seq)
             screen = m_socket->readBuffer.unpackScreen();
             m_socket->readBuffer >> wrnLevel;
             m_socket->readBuffer >> nTlv;
-            for(unsigned short i = 0; i < nTlv; i++) {
+            for(unsigned i = 0; i < nTlv; i++) {
                 unsigned short num;
                 unsigned short size;
                 const char*    data;
@@ -209,12 +210,13 @@ void ICQClient::snac_icmb(unsigned short type, unsigned short seq)
                     bAck = true;
             }
             if (bAck){
+                log(L_DEBUG, "Ack: %u %u (%s)", m_send.id.id_h, m_send.id.id_l, m_send.screen.c_str());
                 if (m_send.msg){
                     if (m_send.msg->type() == MessageCheckInvisible){
                         Contact *contact;
                         ICQUserData *data = findContact(m_send.screen.c_str(), NULL, false, contact);
-                        if (data && data->bInvisible) {
-                            data->bInvisible = false;
+                        if (data && data->bInvisible.bValue) {
+                            data->bInvisible.bValue = false;
                             Event e(EventContactStatus, contact);
                             e.process();
                         }
@@ -222,7 +224,8 @@ void ICQClient::snac_icmb(unsigned short type, unsigned short seq)
                     }else{
                         Contact *contact;
                         ICQUserData *data = findContact(screen.c_str(), NULL, false, contact);
-                        if ((data == NULL) || (data->Status == ICQ_STATUS_OFFLINE)){
+                        if (((data == NULL) || (data->Status.value == ICQ_STATUS_OFFLINE) || (getAckMode() == 1)) &&
+                                (m_send.msg->type() != MessageFile)){
                             ackMessage(m_send);
                         }else{
                             replyQueue.push_back(m_send);
@@ -318,7 +321,7 @@ void ICQClient::snac_icmb(unsigned short type, unsigned short seq)
                 log(L_DEBUG, "Autoreply from %s %s", screen.c_str(), answer.c_str());
                 Contact *contact;
                 ICQUserData *data = findContact(screen.c_str(), NULL, false, contact);
-                if (data && set_str(&data->AutoReply, answer.c_str())){
+                if (data && set_str(&data->AutoReply.ptr, answer.c_str())){
                     Event e(EventContactChanged, contact);
                     e.process();
                 }
@@ -345,11 +348,13 @@ void ICQClient::snac_icmb(unsigned short type, unsigned short seq)
                     TlvList tlv_msg(m);
                     Tlv *m_tlv = tlv_msg(0x101);
                     if (m_tlv == NULL){
-                        log(L_WARN, "No found generic message tlv 101");
+                        log(L_WARN, "generic message tlv 0101 not found");
                         break;
                     }
-                    if (m_tlv->Size() <= 4)
+                    if (m_tlv->Size() <= 4){
+                        log(L_WARN, "Bad tlv 0101 size");
                         break;
+                    }
                     char *m_data = (*m_tlv);
                     unsigned short encoding = (unsigned short)((m_data[0] << 8) + m_data[1]);
                     m_data += 4;
@@ -370,29 +375,31 @@ void ICQClient::snac_icmb(unsigned short type, unsigned short seq)
                             msg->setBackground(bgColor);
                             msg->setFlags(MESSAGE_RICHTEXT);
                         }
+                        string s;
+                        if (!text.isEmpty())
+                            s = text.local8Bit();
+                        log(L_DEBUG, "Message %s", s.c_str());
                         messageReceived(msg, screen.c_str());
                         break;
                     }
-                    Message *mm = NULL;
-                    if (atol(screen.c_str())){
-                        ICQMessage *msg = new ICQMessage;
+                    log(L_DEBUG, "ServerText: %s", m_data);
+                    ICQMessage *msg = new ICQMessage;
+                    if (!m_bAIM && atol(screen.c_str())){
                         msg->setServerText(m_data);
-                        mm = msg;
                     }else{
-                        mm = new Message(MessageGeneric);
-                        QString text = QString::fromUtf8(m_data);
+                        QString text = m_data;
                         unsigned bgColor = clearTags(text);
-                        mm->setText(text);
-                        mm->setBackground(bgColor);
-                        mm->setFlags(MESSAGE_RICHTEXT);
+                        msg->setServerText(m_data);
+                        msg->setBackground(bgColor);
+                        msg->setFlags(MESSAGE_RICHTEXT);
                     }
-                    messageReceived(mm, screen.c_str());
+                    messageReceived(msg, screen.c_str());
                     break;
                 }
             case 0x0002:{
                     TlvList tlv(m_socket->readBuffer);
                     if (!tlv(5)){
-                        log(L_WARN, "No found ICMB message tlv");
+                        log(L_WARN, "ICMB message tlv5 not found");
                         break;
                     }
                     Buffer msg(*tlv(5));
@@ -444,7 +451,7 @@ void ICQClient::snac_icmb(unsigned short type, unsigned short seq)
             case 0x0004:{
                     TlvList tlv(m_socket->readBuffer);
                     if (!tlv(5)){
-                        log(L_WARN, "No found advanced message tlv");
+                        log(L_WARN, "Advanced message tlv5 not found");
                         break;
                     }
                     Buffer msg(*tlv(5));
@@ -454,7 +461,7 @@ void ICQClient::snac_icmb(unsigned short type, unsigned short seq)
                         parseAdvancedMessage(screen.c_str(), msg, tlv(6) != NULL, id);
                         return;
                     }
-                    char type, flags;
+                    unsigned char type, flags;
                     msg >> type;
                     msg >> flags;
                     string msg_str;
@@ -502,9 +509,9 @@ bool ICQClient::sendThruServer(Message *msg, void *_data)
     SendMsg s;
     switch (msg->type()){
     case MessageGeneric:
-        if ((data->Status != ICQ_STATUS_OFFLINE) && (getSendFormat() == 0) &&
+        if ((data->Status.value != ICQ_STATUS_OFFLINE) && (getSendFormat() == 0) &&
                 hasCap(data, CAP_RTF) && (msg->getFlags() & MESSAGE_RICHTEXT) &&
-                !data->bBadClient){
+                !data->bBadClient.bValue){
             s.flags  = SEND_RTF;
             s.msg    = msg;
             s.text   = msg->getRichText();
@@ -513,11 +520,11 @@ bool ICQClient::sendThruServer(Message *msg, void *_data)
             send(false);
             return true;
         }
-        if ((data->Status != ICQ_STATUS_OFFLINE) &&
+        if ((data->Status.value != ICQ_STATUS_OFFLINE) &&
                 (getSendFormat() <= 1) &&
                 hasCap(data, CAP_UTF) &&
                 ((msg->getFlags() & MESSAGE_SECURE) == 0) &&
-                (data->Version >= 8) && !data->bBadClient){
+                (data->Version.value >= 8) && !data->bBadClient.bValue){
             s.flags  = SEND_UTF;
             s.msg    = msg;
             s.text   = addCRLF(msg->getPlainText());
@@ -526,9 +533,9 @@ bool ICQClient::sendThruServer(Message *msg, void *_data)
             send(false);
             return true;
         }
-        if ((data->Status != ICQ_STATUS_OFFLINE) &&
-                (data->Version >= 8) &&
-                !data->bBadClient &&
+        if ((data->Status.value != ICQ_STATUS_OFFLINE) &&
+                (data->Version.value >= 8) &&
+                !data->bBadClient.bValue &&
                 (msg->getPlainText().length() >= MAX_PLAIN_MESSAGE_SIZE)){
             s.flags  = SEND_TYPE2;
             s.msg    = msg;
@@ -538,8 +545,9 @@ bool ICQClient::sendThruServer(Message *msg, void *_data)
             send(false);
             return true;
         }
-        if ((data->Uin == 0) || m_bAIM ||
-                (hasCap(data, CAP_AIM_BUDDYCON) && !hasCap(data, CAP_AIM_CHAT))){
+        if ((data->Uin.value == 0) || m_bAIM ||
+                (hasCap(data, CAP_AIM_BUDDYCON) && !hasCap(data, CAP_AIM_CHAT) &&
+                 !hasCap(data, CAP_LICQ) && ((data->InfoUpdateTime.value & 0xFF7F0000L) != 0x7D000000L))){
             s.msg	 = msg;
             if (msg->getFlags() & MESSAGE_RICHTEXT){
                 s.flags  = SEND_HTML;
@@ -561,8 +569,9 @@ bool ICQClient::sendThruServer(Message *msg, void *_data)
         send(false);
         return true;
     case MessageUrl:
-        if ((data->Uin == 0) || m_bAIM ||
-                (hasCap(data, CAP_AIM_BUDDYCON) && !hasCap(data, CAP_AIM_CHAT))){
+        if ((data->Uin.value == 0) || m_bAIM ||
+                (hasCap(data, CAP_AIM_BUDDYCON) && !hasCap(data, CAP_AIM_CHAT) &&
+                 !hasCap(data, CAP_LICQ) && ((data->InfoUpdateTime.value & 0xFF7F0000L) != 0x7D000000L))){
             UrlMessage *m = static_cast<UrlMessage*>(msg);
             QString text = "<a href=\"";
             text += m->getUrl();
@@ -602,7 +611,8 @@ void ICQClient::sendThroughServer(const char *screen, unsigned short channel, Bu
     m_socket->writeBuffer.packScreen(screen);
     if (channel == 1)
         tlv_type = 2;
-    m_socket->writeBuffer.tlv(tlv_type, b);
+    if (b.size())
+        m_socket->writeBuffer.tlv(tlv_type, b);
     if (bReqAck)
         m_socket->writeBuffer.tlv(3);		// req. ack from server
     if (bOffline)
@@ -740,7 +750,7 @@ void ICQClient::sendAdvMessage(const char *screen, Buffer &msgText, unsigned plu
     msgBuf.pack(0x00000000L);
     msgBuf.pack(0x00000000L);
     msgBuf.pack(msgText.data(0), msgText.size());
-    sendType2(screen, msgBuf, id, CAP_SRV_RELAY, bOffline, bDirect ? data.owner.Port : 0, NULL, type);
+    sendType2(screen, msgBuf, id, CAP_SRV_RELAY, bOffline, bDirect ? data.owner.Port.value : 0, NULL, type);
 }
 
 static void copyTlv(Buffer &b, TlvList *tlvs, unsigned nTlv)
@@ -875,8 +885,8 @@ void ICQClient::parseAdvancedMessage(const char *screen, Buffer &m, bool needAck
                 set_ip(&data->RealIP, real_ip);
             if (ip && (get_ip(data->IP) == 0))
                 set_ip(&data->IP, ip);
-            if (port && (data->Port == 0))
-                data->Port = port;
+            if (port && (data->Port.value == 0))
+                data->Port.value = port;
         }
     }
 
@@ -1106,8 +1116,8 @@ void ICQClient::parseAdvancedMessage(const char *screen, Buffer &m, bool needAck
             ICQUserData *data = findContact(screen, NULL, false, contact);
             if (data == NULL)
                 return;
-            if ((getInvisible() && (data->VisibleId == 0)) ||
-                    (!getInvisible() && data->InvisibleId))
+            if ((getInvisible() && (data->VisibleId.value == 0)) ||
+                    (!getInvisible() && data->InvisibleId.value))
                 return;
             ar_request req;
             req.screen  = screen;
@@ -1128,7 +1138,7 @@ void ICQClient::parseAdvancedMessage(const char *screen, Buffer &m, bool needAck
             e.process();
 
             if (!msg.empty()){
-                set_str(&data->AutoReply, msg.c_str());
+                set_str(&data->AutoReply.ptr, msg.c_str());
                 Event e(EventContactChanged, contact);
                 e.process();
             }
@@ -1143,9 +1153,17 @@ void ICQClient::parseAdvancedMessage(const char *screen, Buffer &m, bool needAck
         if (msg.size() || (msgType == ICQ_MSGxEXT)){
             Message *m = parseMessage(msgType, screen, msg, adv, id, cookie1 | (cookie2 << 16));
             if (m){
+                if ((m_send.id == id) && (m_send.screen == screen)){
+                    replyQueue.push_back(m_send);
+                    m_send.msg    = NULL;
+                    m_send.screen = "";
+                    m_sendTimer->stop();
+                    send(true);
+                }
                 list<SendMsg>::iterator it;
                 for (it = replyQueue.begin(); it != replyQueue.end(); ++it){
                     SendMsg &s = *it;
+                    log(L_DEBUG, "%u %u (%s) - %u %u (%s)", s.id.id_h, s.id.id_l, s.screen.c_str(), id.id_h, id.id_l, screen);
                     if ((s.id == id) && (s.screen == screen))
                         break;
                 }
@@ -1218,6 +1236,13 @@ void ICQClient::parseAdvancedMessage(const char *screen, Buffer &m, bool needAck
                             delete msg;
                             return;
                         }
+                        if (m_state == 1){
+                            msg->setError(I18N_NOOP("Message declined"));
+                            Event e(EventMessageSent, msg);
+                            e.process();
+                            delete msg;
+                            return;
+                        }
                         ICQFileTransfer *ft = new ICQFileTransfer(static_cast<FileMessage*>(msg), data, this);
                         Event e(EventMessageAcked, msg);
                         e.process();
@@ -1286,6 +1311,127 @@ void ICQClient::sendMTN(const char *screen, unsigned short type)
     sendPacket();
 }
 
+class AIMParser : public HTMLParser
+{
+public:
+    AIMParser() {}
+    QString parse(const QString &str);
+protected:
+    bool bPara;
+    QString res;
+    virtual void text(const QString &text);
+    virtual void tag_start(const QString &tag, const list<QString> &options);
+    virtual void tag_end(const QString &tag);
+};
+
+QString AIMParser::parse(const QString &str)
+{
+    bPara = false;
+    HTMLParser::parse(str);
+    return res;
+}
+
+void AIMParser::text(const QString &text)
+{
+    if (text.isEmpty())
+        return;
+    bPara = true;
+    res += text;
+}
+
+void AIMParser::tag_start(const QString &tag, const list<QString> &options)
+{
+    QString otag;
+    QString add;
+    if (tag == "br")
+        otag = "BR";
+    if (tag == "p"){
+        if (!bPara)
+            return;
+        otag = "BR";
+    }
+    if ((tag == "font") || (tag == "b") || (tag == "u") || (tag == "i"))
+        otag = tag.upper();
+    if (tag == "span")
+        otag = "FONT";
+    if (otag.isEmpty())
+        return;
+    res += "<";
+    res += otag;
+    for (list<QString>::const_iterator it = options.begin(); it != options.end(); ++it){
+        QString key = *it;
+        ++it;
+        QString value = *it;
+        if (key.lower() == "style"){
+            list<QString> styles = parseStyle(value);
+            for (list<QString>::const_iterator its = styles.begin(); its != styles.end(); ++its){
+                QString key = *its;
+                ++its;
+                QString value = *its;
+                if (key == "font-family")
+                {
+                    res += " FACE=\"";
+                    res += value;
+                    res += "\"";
+                }
+                else if (key == "font-size")
+                {
+                    if (value == "smaller")
+                    {
+                        res += " SIZE=2";
+                    }
+                    else if (value == "larger")
+                    {
+                        res += " SIZE=4";
+                    }
+                }
+                else if (key == "font-style")
+                {
+                    if (value.lower() == "italic")
+                        add = "<i>";
+                }
+                else if (key == "font-weight")
+                {
+                    if (value.toInt() >= 600)
+                        add = "<b>";
+                }
+                else if (key == "text-decoration")
+                {
+                    if (value.lower() == "underline")
+                        add = "u";
+                }
+                else if (key == "color")
+                {
+                    res += " COLOR=\"";
+                    res += value;
+                    res += "\"";
+                }
+            }
+            continue;
+        }
+        res += " ";
+        res += key.upper();
+        res += "=\"";
+        res += value;
+        res += "\"";
+    }
+    res += ">";
+}
+
+void AIMParser::tag_end(const QString &tag)
+{
+    QString otag;
+    if ((tag == "font") || (tag == "b") || (tag == "u") || (tag == "i"))
+        otag = tag.upper();
+    if (tag == "span")
+        otag = "FONT";
+    if (otag.isEmpty())
+        return;
+    res += "</";
+    res += otag;
+    res += ">";
+}
+
 void ICQClient::processSendQueue()
 {
     m_sendTimer->stop();
@@ -1295,8 +1441,8 @@ void ICQClient::processSendQueue()
             if (m_send.msg->type() == MessageCheckInvisible){
                 Contact *contact;
                 ICQUserData *data = findContact(m_send.screen.c_str(), NULL, false, contact);
-                if (data && (data->bInvisible == 0)) {
-                    data->bInvisible = true;
+                if (data && (data->bInvisible.bValue == 0)) {
+                    data->bInvisible.bValue = true;
                     Event e(EventContactStatus, contact);
                     e.process();
                 }
@@ -1311,7 +1457,7 @@ void ICQClient::processSendQueue()
         send(true);
         return;
     }
-    m_sendTimer->start(30000);
+    m_sendTimer->start(50000);
     for (;;){
         if ((getState() != Connected) || sendQueue.empty()){
             m_sendTimer->stop();
@@ -1334,7 +1480,10 @@ void ICQClient::processSendQueue()
             m_send.screen = "";
             continue;
         }
-
+        unsigned type = 0;
+        if (m_send.msg)
+            type = m_send.msg->type(),
+                   log(L_DEBUG, "Send: %s %u %X", m_send.screen.c_str(), type, m_send.flags);
         if (m_send.msg && (m_send.socket == NULL)){
             unsigned short type;
             Buffer b;
@@ -1342,7 +1491,7 @@ void ICQClient::processSendQueue()
             m_send.id.id_h = rand();
             switch (m_send.msg->type()){
             case MessageContacts:
-                if (data->Uin == 0){
+                if (data->Uin.value == 0){
                     CONTACTS_MAP c;
                     QString nc = packContacts(static_cast<ContactsMessage*>(m_send.msg), data, c);
                     if (c.empty()){
@@ -1359,7 +1508,7 @@ void ICQClient::processSendQueue()
                     vector<alias_group> cc;
                     for (CONTACTS_MAP::iterator it = c.begin(); it != c.end(); ++it){
                         alias_group c;
-                        c.alias = (*it).first;
+                        c.alias = (*it).first.c_str();
                         c.grp   = (*it).second.grp;
                         cc.push_back(c);
                     }
@@ -1405,7 +1554,7 @@ void ICQClient::processSendQueue()
                     return;
                 }
             case MessageUrl:{
-                    if (data->Uin == 0)
+                    if (data->Uin.value == 0)
                         break;
                     packMessage(b, m_send.msg, data, type);
                     const char *err = m_send.msg->getError();
@@ -1418,7 +1567,7 @@ void ICQClient::processSendQueue()
                         continue;
                     }
                     sendThroughServer(screen(data).c_str(), 4, b, m_send.id, true, true);
-                    if (data->Status != ICQ_STATUS_OFFLINE)
+                    if ((data->Status.value != ICQ_STATUS_OFFLINE) || (getAckMode() == 0))
                         ackMessage(m_send);
                     return;
                 }
@@ -1428,9 +1577,7 @@ void ICQClient::processSendQueue()
                 return;
             case MessageCheckInvisible:{
                     Buffer b;
-                    b.pack(this->data.owner.Uin);
-                    b << 0xE8000100L << (char)00;
-                    sendThroughServer(m_send.screen.c_str(), 4, b, m_send.id, true, false);
+                    sendThroughServer(m_send.screen.c_str(), 2, b, m_send.id, true, false);
                     return;
                 }
             case MessageWarning:{
@@ -1448,8 +1595,8 @@ void ICQClient::processSendQueue()
             }
             string text;
             string encoding;
-            if (data->Encoding)
-                encoding = data->Encoding;
+            if (data->Encoding.ptr)
+                encoding = data->Encoding.ptr;
             switch (m_send.flags & SEND_MASK){
             case SEND_RTF:
                 m_send.part = getRichTextPart(m_send.text, MAX_TYPE2_MESSAGE_SIZE);
@@ -1483,7 +1630,8 @@ void ICQClient::processSendQueue()
                     t += b;
                     t += "\">";
                     if ((m_send.flags & SEND_MASK) == SEND_HTML){
-                        t += m_send.part;
+                        AIMParser p;
+                        t += p.parse(m_send.part);
                     }else{
                         string s;
                         s = m_send.part.utf8();
@@ -1617,7 +1765,7 @@ void ICQClient::processSendQueue()
                 sendType2(m_send.screen.c_str(), msgBuf, m_send.id, CAP_AIM_SENDFILE, false, m_send.socket->localPort(), &tlvs);
                 return;
             }
-            msgBuf.pack(this->data.owner.Uin);
+            msgBuf.pack(this->data.owner.Uin.value);
             unsigned long ip = get_ip(this->data.owner.IP);
             if (ip == get_ip(m_send.socket->m_data->IP))
                 ip = get_ip(this->data.owner.RealIP);
@@ -1625,7 +1773,7 @@ void ICQClient::processSendQueue()
             msgBuf.pack((unsigned long)(m_send.socket->localPort()));
             msgBuf.pack((char)MODE_DIRECT);
             msgBuf.pack((unsigned long)(m_send.socket->remotePort()));
-            msgBuf.pack((unsigned long)(this->data.owner.Port));
+            msgBuf.pack(this->data.owner.Port.value);
             msgBuf.pack((unsigned short)8);
             msgBuf.pack((unsigned long)m_nMsgSequence);
             sendType2(m_send.screen.c_str(), msgBuf, m_send.id, CAP_DIRECT, false, 0);
@@ -1634,7 +1782,7 @@ void ICQClient::processSendQueue()
         if (m_send.flags == PLUGIN_AR){
             log(L_DEBUG, "Request auto response %s", m_send.screen.c_str());
 
-            unsigned long status = data->Status;
+            unsigned long status = data->Status.value;
             if ((status == ICQ_STATUS_ONLINE) || (status == ICQ_STATUS_OFFLINE))
                 continue;
 
@@ -1709,10 +1857,10 @@ void ICQClient::sendType1(const QString &text, bool bWide, ICQUserData *data)
         msgBuf.pack(msg_text.c_str(), msg_text.length());
     }else{
         string encoding;
-        if (data->Encoding){
-            encoding = data->Encoding;
-        }else if (this->data.owner.Encoding){
-            encoding = this->data.owner.Encoding;
+        if (data->Encoding.ptr){
+            encoding = data->Encoding.ptr;
+        }else if (this->data.owner.Encoding.ptr){
+            encoding = this->data.owner.Encoding.ptr;
         }
         QTextCodec *codec = getCodec(encoding.c_str());
         string msg_text;
@@ -1729,7 +1877,7 @@ void ICQClient::sendType1(const QString &text, bool bWide, ICQUserData *data)
     b.tlv(0x0501, "\x01", 1);
     b.tlv(0x0101, msgBuf);
     sendThroughServer(m_send.screen.c_str(), 1, b, m_send.id, true, true);
-    if (data->Status != ICQ_STATUS_OFFLINE)
+    if ((data->Status.value != ICQ_STATUS_OFFLINE) || (getAckMode() == 0))
         ackMessage(m_send);
 }
 
@@ -1751,11 +1899,11 @@ void ICQClient::accept(Message *msg, ICQUserData *data)
             log(L_WARN, "Data for request not found");
             return;
         }
-        if (data->Direct == NULL){
+        if (data->Direct.ptr == NULL){
             log(L_WARN, "No direct connection");
             return;
         }
-        data->Direct->acceptMessage(msg);
+        ((DirectClient*)(data->Direct.ptr))->acceptMessage(msg);
     }else{
         id.id_l = static_cast<ICQFileMessage*>(msg)->getID_L();
         id.id_h = static_cast<ICQFileMessage*>(msg)->getID_H();
@@ -1833,11 +1981,11 @@ void ICQClient::decline(Message *msg, const char *reason)
             log(L_WARN, "Data for request not found");
             return;
         }
-        if (data->Direct == NULL){
+        if (data->Direct.ptr == NULL){
             log(L_WARN, "No direct connection");
             return;
         }
-        data->Direct->declineMessage(msg, reason);
+        ((DirectClient*)(data->Direct.ptr))->declineMessage(msg, reason);
     }else{
         MessageId id;
         unsigned cookie = 0;
@@ -1911,13 +2059,17 @@ void ICQClient::requestReverseConnection(const char *screen, DirectSocket *socke
 
 void ICQClient::send(bool bTimer)
 {
-    if (sendQueue.size() == 0){
-        if (m_sendTimer->isActive() && m_send.screen.empty())
-            m_sendTimer->stop();
+    if (!m_send.screen.empty()){
+        m_sendTimer->start(m_nSendTimeout * 500);
         return;
     }
-    if (!m_send.screen.empty())
+    if (sendQueue.size() == 0){
+        if (m_sendTimer->isActive() && m_send.screen.empty()){
+            log(L_DEBUG, "Stop send timer");
+            m_sendTimer->stop();
+        }
         return;
+    }
     if (!bTimer){
         processSendQueue();
         return;
