@@ -28,6 +28,7 @@
 #include <qtextcodec.h>
 #include <qfile.h>
 #include <qimage.h>
+#include <qregexp.h>
 
 static DataDef icqMessageData[] =
     {
@@ -63,12 +64,15 @@ string ICQMessage::save()
     return s;
 }
 
-QString ICQMessage::getText()
+QString ICQMessage::getText() const
 {
     const char *serverText = getServerText();
     if ((serverText == NULL) || (*serverText == 0))
         return Message::getText();
-    return ICQClient::toUnicode(serverText, client(), contact());
+    QString res = ICQClient::toUnicode(serverText, client(), contact());
+    if ((getFlags() & MESSAGE_RICHTEXT) && (res.left(6) == "<HTML>"))
+        ICQClient::clearTags(res);
+    return res;
 }
 
 static DataDef aimFileMessageData[] =
@@ -122,7 +126,7 @@ QString ICQFileMessage::getDescription()
     return ICQClient::toUnicode(serverText, client(), contact());
 }
 
-QString ICQFileMessage::getText()
+QString ICQFileMessage::getText() const
 {
     const char *serverText = getServerText();
     if ((serverText == NULL) || (*serverText == 0))
@@ -172,7 +176,7 @@ string IcqUrlMessage::save()
     return s;
 }
 
-QString IcqUrlMessage::getText()
+QString IcqUrlMessage::getText() const
 {
     const char *serverText = getServerText();
     if ((serverText == NULL) || (*serverText == 0))
@@ -180,9 +184,9 @@ QString IcqUrlMessage::getText()
     return ICQClient::toUnicode(serverText, client(), contact());
 }
 
-QString IcqUrlMessage::getUrl()
+QString IcqUrlMessage::getUrl() const
 {
-    const char *serverText = data.ServerUrl;
+    const char *serverText = data.ServerUrl.ptr;
     if (serverText && *serverText)
         return ICQClient::toUnicode(serverText, client(), contact());
     return UrlMessage::getUrl();
@@ -205,7 +209,7 @@ IcqContactsMessage::~IcqContactsMessage()
     free_data(icqContactsMessageData, &data);
 }
 
-QString IcqContactsMessage::getContacts()
+QString IcqContactsMessage::getContacts() const
 {
     const char *serverText = getServerText();
     if ((serverText == NULL) || (*serverText == 0))
@@ -244,7 +248,7 @@ ICQAuthMessage::~ICQAuthMessage()
     free_data(icqAuthMessageData, &data);
 }
 
-QString ICQAuthMessage::getText()
+QString ICQAuthMessage::getText() const
 {
     const char *serverText = getServerText();
     if ((serverText == NULL) || (*serverText == 0))
@@ -480,7 +484,7 @@ Message *ICQClient::parseExtendedMessage(const char *screen, Buffer &packet, Mes
         ICQUserData *data = findContact(screen, NULL, true, contact);
         if (data == NULL)
             return NULL;
-        Message *msg = parseTextMessage(p.c_str(), cap_str.c_str(), data->Encoding ? data->Encoding : this->data.owner.Encoding);
+        Message *msg = parseTextMessage(p.c_str(), cap_str.c_str(), data->Encoding.ptr ? data->Encoding.ptr : this->data.owner.Encoding.ptr);
         if (msg){
             if (forecolor != backcolor){
                 msg->setForeground(forecolor >> 8);
@@ -593,7 +597,7 @@ Message *ICQClient::parseMessage(unsigned short type, const char *screen, string
             ICQUserData *data = findContact(screen, NULL, true, contact);
             if (data == NULL)
                 return NULL;
-            msg = parseTextMessage(p.c_str(), cap_str.c_str(), data->Encoding ? data->Encoding : this->data.owner.Encoding);
+            msg = parseTextMessage(p.c_str(), cap_str.c_str(), data->Encoding.ptr ? data->Encoding.ptr : this->data.owner.Encoding.ptr);
             if (msg == NULL)
                 break;
             if (forecolor != backcolor){
@@ -652,10 +656,11 @@ static Message *createIcq(const char *cfg)
 
 static MessageDef defIcq =
     {
-        NULL,
-        MESSAGE_DEFAULT,
         NULL,
         NULL,
+        MESSAGE_CHILD,
+        "Message",
+        "%n messages",
         createIcq,
         NULL,
         NULL
@@ -666,12 +671,17 @@ static Message *createIcqFile(const char *cfg)
     return new ICQFileMessage(cfg);
 }
 
+#if 0
+i18n("File", "%n files", 1);
+#endif
+
 static MessageDef defIcqFile =
     {
-        NULL,
-        MESSAGE_DEFAULT,
         NULL,
         NULL,
+        MESSAGE_CHILD,
+        "File",
+        "%n files",
         createIcqFile,
         NULL,
         NULL
@@ -688,6 +698,7 @@ static Message *createWebPanel(const char *cfg)
 
 static MessageDef defWebPanel =
     {
+        NULL,
         NULL,
         MESSAGE_DEFAULT,
         "WWW-panel message",
@@ -708,6 +719,7 @@ static Message *createEmailPager(const char *cfg)
 
 static MessageDef defEmailPager =
     {
+        NULL,
         NULL,
         MESSAGE_DEFAULT,
         "Email pager message",
@@ -728,6 +740,7 @@ static Message *createOpenSecure(const char *cfg)
 
 static MessageDef defOpenSecure =
     {
+        NULL,
         NULL,
         MESSAGE_SENDONLY,
         "Request secure channel",
@@ -748,6 +761,7 @@ static Message *createCloseSecure(const char *cfg)
 
 static MessageDef defCloseSecure =
     {
+        NULL,
         NULL,
         MESSAGE_SILENT | MESSAGE_SENDONLY,
         "Close secure channel",
@@ -768,6 +782,7 @@ static Message *createCheckInvisible(const char *cfg)
 
 static MessageDef defCheckInvisible =
     {
+        NULL,
         NULL,
         MESSAGE_SILENT | MESSAGE_SENDONLY,
         "Check invisible",
@@ -817,6 +832,7 @@ static Message *createWarning(const char *cfg)
 
 static MessageDef defWarning =
     {
+        NULL,
         NULL,
         MESSAGE_SENDONLY,
         "Warning",
@@ -834,7 +850,8 @@ static Message *createIcqAuthRequest(const char *cfg)
 static MessageDef defIcqAuthRequest =
     {
         NULL,
-        MESSAGE_DEFAULT,
+        NULL,
+        MESSAGE_CHILD,
         NULL,
         NULL,
         createIcqAuthRequest,
@@ -850,7 +867,8 @@ static Message *createIcqAuthGranted(const char *cfg)
 static MessageDef defIcqAuthGranted =
     {
         NULL,
-        MESSAGE_DEFAULT,
+        NULL,
+        MESSAGE_CHILD,
         NULL,
         NULL,
         createIcqAuthGranted,
@@ -866,7 +884,8 @@ static Message *createIcqAuthRefused(const char *cfg)
 static MessageDef defIcqAuthRefused =
     {
         NULL,
-        MESSAGE_DEFAULT,
+        NULL,
+        MESSAGE_CHILD,
         NULL,
         NULL,
         createIcqAuthRefused,
@@ -885,6 +904,7 @@ i18n("Contact request", "%n contact requests", 1);
 
 static MessageDef defContactRequest =
     {
+        NULL,
         NULL,
         MESSAGE_DEFAULT | MESSAGE_SYSTEM,
         "Contact request",
@@ -902,7 +922,8 @@ static Message *createIcqUrl(const char *cfg)
 static MessageDef defIcqUrl =
     {
         NULL,
-        MESSAGE_DEFAULT,
+        NULL,
+        MESSAGE_CHILD,
         NULL,
         NULL,
         createIcqUrl,
@@ -918,7 +939,8 @@ static Message *createIcqContacts(const char *cfg)
 static MessageDef defIcqContacts =
     {
         NULL,
-        MESSAGE_DEFAULT,
+        NULL,
+        MESSAGE_CHILD,
         NULL,
         NULL,
         createIcqContacts,
@@ -1102,11 +1124,6 @@ void ICQClient::packExtendedMessage(Message *msg, Buffer &buf, Buffer &msgBuf, I
     }
 }
 
-bool my_string::operator < (const my_string &a) const
-{
-    return strcmp(c_str(), a.c_str()) < 0;
-}
-
 QString ICQClient::packContacts(ContactsMessage *msg, ICQUserData *data, CONTACTS_MAP &c)
 {
     QString contacts = msg->getContacts();
@@ -1239,14 +1256,9 @@ void ICQClient::packMessage(Buffer &b, Message *msg, ICQUserData *data, unsigned
         if (msg->getFlags() & MESSAGE_LIST)
             flags = ICQ_TCPxMSG_LIST;
     }
-    if (type == ICQ_MSGxEXT){
-        b.pack(type);
-        b.pack(msgStatus());
-        b.pack(flags);
-    }else{
-        b.pack(this->data.owner.Uin);
-        b << (char)type << (char)0;
-    }
+    b.pack(type);
+    b.pack(msgStatus());
+    b.pack(flags);
     b << res;
     if (buf.size()){
         b.pack((unsigned short)buf.size());
@@ -1297,14 +1309,14 @@ void ICQClient::parsePluginPacket(Buffer &b, unsigned plugin_type, ICQUserData *
                 b.unpack(homepage);
                 ICQUserData data;
                 load_data(static_cast<ICQProtocol*>(protocol())->icqUserData, &data, NULL);
-                data.Uin = uin;
-                set_str(&data.Alias, name.c_str());
-                set_str(&data.About, topic.c_str());
-                data.Age = age;
-                data.Gender = gender;
-                data.Country = country;
-                data.Language = language;
-                set_str(&data.Homepage, homepage.c_str());
+                data.Uin.value = uin;
+                set_str(&data.Alias.ptr, name.c_str());
+                set_str(&data.About.ptr, topic.c_str());
+                data.Age.value = age;
+                data.Gender.value = gender;
+                data.Country.value = country;
+                data.Language.value = language;
+                set_str(&data.Homepage.ptr, homepage.c_str());
                 Event e(EventRandomChatInfo, &data);
                 e.process();
                 free_data(static_cast<ICQProtocol*>(protocol())->icqUserData, &data);
@@ -1384,8 +1396,8 @@ void ICQClient::parsePluginPacket(Buffer &b, unsigned plugin_type, ICQUserData *
                 }else{
                     log(L_ERROR, "Can't create %s", (const char*)fName.local8Bit());
                 }
-                data->PictureWidth  = img.width();
-                data->PictureHeight = img.height();
+                data->PictureWidth.value  = img.width();
+                data->PictureHeight.value = img.height();
             }
             break;
         case PLUGIN_PHONEBOOK:
@@ -1481,9 +1493,9 @@ void ICQClient::parsePluginPacket(Buffer &b, unsigned plugin_type, ICQUserData *
                         phones += ";";
                     phones += phone;
                 }
-                set_str(&data->PhoneBook, phones.c_str());
+                set_str(&data->PhoneBook.ptr, phones.c_str());
                 Contact *contact = NULL;
-                findContact(number(data->Uin).c_str(), NULL, false, contact);
+                findContact(number(data->Uin.value).c_str(), NULL, false, contact);
                 if (contact){
                     setupContact(contact, data);
                     Event e(EventContactChanged, contact);
@@ -1504,22 +1516,22 @@ void ICQClient::parsePluginPacket(Buffer &b, unsigned plugin_type, ICQUserData *
                 break;
             switch (plugin_type){
             case PLUGIN_FILESERVER:
-                if ((state != 0) != (data->SharedFiles != 0)){
-                    data->SharedFiles = state;
+                if ((state != 0) != (data->SharedFiles.bValue != 0)){
+                    data->SharedFiles.bValue = (state != 0);
                     Event e(EventContactChanged, contact);
                     e.process();
                 }
                 break;
             case PLUGIN_FOLLOWME:
-                if (state != data->FollowMe){
-                    data->FollowMe = state;
+                if (state != data->FollowMe.value){
+                    data->FollowMe.value = state;
                     Event e(EventContactChanged, contact);
                     e.process();
                 }
                 break;
             case PLUGIN_ICQPHONE:
-                if (state != data->ICQPhone){
-                    data->ICQPhone = state;
+                if ((state != 0) != data->ICQPhone.bValue){
+                    data->ICQPhone.bValue = (state != 0);
                     Event e(EventContactChanged, contact);
                     e.process();
                 }
@@ -1561,9 +1573,9 @@ void ICQClient::pluginAnswer(unsigned plugin_type, unsigned long uin, Buffer &in
     unsigned long time = 0;
     switch (plugin_type){
     case PLUGIN_PHONEBOOK:{
-            if (data && data->GrpId && !contact->getIgnore()){
+            if (data && data->GrpId.value && !contact->getIgnore()){
                 Buffer answer1;
-                time = this->data.owner.PluginInfoTime;
+                time = this->data.owner.PluginInfoTime.value;
                 QString phones = getContacts()->owner()->getPhones();
                 while (!phones.isEmpty()){
                     QString item = getToken(phones, ';', false);
@@ -1653,7 +1665,7 @@ void ICQClient::pluginAnswer(unsigned plugin_type, unsigned long uin, Buffer &in
             }
         }
     case PLUGIN_PICTURE:{
-            time = this->data.owner.PluginInfoTime;
+            time = this->data.owner.PluginInfoTime.value;
             typeAnswer = 0x00000001;
             QString pictFile = getPicture();
             if (!pictFile.isEmpty()){
@@ -1687,10 +1699,10 @@ int n = pictFile.findRev("/");
             break;
         }
     case PLUGIN_FOLLOWME:
-        time = this->data.owner.PluginStatusTime;
+        time = this->data.owner.PluginStatusTime.value;
         break;
     case PLUGIN_QUERYxINFO:
-        time = this->data.owner.PluginInfoTime;
+        time = this->data.owner.PluginInfoTime.value;
         typeAnswer = 0x00010002;
         if (!getPicture().isEmpty()){
             nEntries++;
@@ -1712,7 +1724,7 @@ int n = pictFile.findRev("/");
         }
         break;
     case PLUGIN_QUERYxSTATUS:
-        time = this->data.owner.PluginStatusTime;
+        time = this->data.owner.PluginStatusTime.value;
         typeAnswer = 0x00010000;
         nEntries++;
         answer.pack((char*)plugins[PLUGIN_FOLLOWME], sizeof(plugin));
@@ -1721,7 +1733,7 @@ int n = pictFile.findRev("/");
         answer.packStr32(plugin_name[PLUGIN_FOLLOWME]);
         answer.packStr32(plugin_descr[PLUGIN_FOLLOWME]);
         answer.pack((unsigned long)0);
-        if (this->data.owner.SharedFiles){
+        if (this->data.owner.SharedFiles.bValue){
             nEntries++;
             answer.pack((char*)plugins[PLUGIN_FILESERVER], sizeof(plugin));
             answer.pack((unsigned short)0);
@@ -1730,7 +1742,7 @@ int n = pictFile.findRev("/");
             answer.packStr32(plugin_descr[PLUGIN_FILESERVER]);
             answer.pack((unsigned long)0);
         }
-        if (this->data.owner.ICQPhone){
+        if (this->data.owner.ICQPhone.bValue){
             nEntries++;
             answer.pack((char*)plugins[PLUGIN_ICQPHONE], sizeof(plugin));
             answer.pack((unsigned short)0);
@@ -1748,7 +1760,7 @@ int n = pictFile.findRev("/");
     info.pack((unsigned short)1);
     switch (plugin_type){
     case PLUGIN_FOLLOWME:
-        info.pack(this->data.owner.FollowMe);
+        info.pack(this->data.owner.FollowMe.value);
         info.pack(time);
         info.pack((char)1);
         break;
