@@ -18,6 +18,7 @@
 #include "simapi.h"
 #include "icqclient.h"
 #include "html.h"
+#include "core.h"
 
 #include <qtextcodec.h>
 #include <qregexp.h>
@@ -316,7 +317,7 @@ string RTFGenParser::parse(const QString &text)
     m_codec = m_client->getCodec(m_encoding);
     m_encoding = m_codec->name();
     int charset = 0;
-    for (const ENCODING *c = ICQClient::encodings; c->language; c++){
+    for (const ENCODING *c = ICQPlugin::core->encodings; c->language; c++){
         if (!strcasecmp(c->codec, m_encoding)){
             charset = c->rtf_code;
             break;
@@ -341,7 +342,7 @@ string RTFGenParser::parse(const QString &text)
     const char *send_encoding = 0;
     m_codec = NULL;
     if (charset){
-        for (const ENCODING *c = ICQClient::encodings; c->language; c++){
+        for (const ENCODING *c = ICQPlugin::core->encodings; c->language; c++){
             if ((c->rtf_code == charset) && c->bMain){
                 send_encoding = c->codec;
                 m_codec = m_client->getCodec(send_encoding);
@@ -766,14 +767,12 @@ protected:
     void endBody();
     QString res;
     bool	 m_bBody;
-    bool     m_bPara;
     unsigned m_maxSmile;
 };
 
 ImageParser::ImageParser(unsigned maxSmile)
 {
     m_maxSmile = maxSmile;
-    m_bPara    = false;
 }
 
 QString ImageParser::parse(const QString &text)
@@ -806,15 +805,18 @@ void ImageParser::text(const QString &text)
 
 void ImageParser::tag_start(const QString &tag, const list<QString> &attrs)
 {
-    if (tag == "body"){
-        startBody();
-        return;
-    }
+    QString oTag = tag;
+
     if (tag == "html"){
         res = "";
         m_bBody = false;
         return;
     }
+    if (tag == "body"){
+        startBody();
+        // We still want BODY's styles
+        oTag = "span";
+    }
     if (!m_bBody)
         return;
     if (tag == "img"){
@@ -842,15 +844,8 @@ void ImageParser::tag_start(const QString &tag, const list<QString> &attrs)
             }
         }
     }
-    if (tag == "p"){
-        if (m_bPara){
-            res += "<br>";
-            m_bPara = false;
-        }
-        return;
-    }
     res += "<";
-    res += tag.upper();
+    res += oTag;
     for (list<QString>::const_iterator it = attrs.begin(); it != attrs.end(); ++it){
         QString name = *it;
         ++it;
@@ -868,18 +863,15 @@ void ImageParser::tag_start(const QString &tag, const list<QString> &attrs)
 
 void ImageParser::tag_end(const QString &tag)
 {
-    if (tag == "body"){
-        endBody();
-        return;
-    }
+    QString oTag = tag;
     if (!m_bBody)
         return;
-    if (tag == "p"){
-        m_bPara = true;
-        return;
+    if (tag == "body"){
+        endBody();
+        oTag = "span";
     }
     res += "</";
-    res += tag.upper();
+    res += oTag;
     res += ">";
 }
 
@@ -923,7 +915,8 @@ protected:
 
 BgParser::BgParser()
 {
-    bgColor = 0;
+    bgColor = 0xFFFFFF;
+    m_bBody = true;
 }
 
 QString BgParser::parse(const QString &text)
@@ -944,11 +937,12 @@ void BgParser::tag_start(const QString &tag, const list<QString> &attrs)
 {
     if (tag == "body"){
         m_bBody = true;
+        res = "";
         for (list<QString>::const_iterator it = attrs.begin(); it != attrs.end(); ++it){
             QString name = *it;
             ++it;
             QString value = *it;
-            if (name == "bgcolor"){
+            if (name.lower() == "bgcolor"){
                 QColor c(value);
                 bgColor = c.rgb();
             }
@@ -957,20 +951,6 @@ void BgParser::tag_start(const QString &tag, const list<QString> &attrs)
     }
     if (!m_bBody)
         return;
-    if (tag == "font"){
-        res += "<span";
-        QString style;
-        for (list<QString>::const_iterator it = attrs.begin(); it != attrs.end(); ++it){
-            QString name = *it;
-            ++it;
-            QString value = *it;
-        }
-        if (!style.isEmpty()){
-            res += " style=\"";
-            res += style;
-            res += "\">";
-        }
-    }
     res += "<";
     res += tag;
     for (list<QString>::const_iterator it = attrs.begin(); it != attrs.end(); ++it){
@@ -996,11 +976,8 @@ void BgParser::tag_end(const QString &tag)
     }
     if (!m_bBody)
         return;
-    QString outTag = tag;
-    if (tag == "font")
-        outTag = "span";
     res += "</";
-    res += outTag;
+    res += tag;
     res += ">";
 }
 
