@@ -125,6 +125,7 @@ JabberBrowser::JabberBrowser(JabberClient *client)
     m_list->addColumn(i18n("Name"));
     m_list->addColumn(i18n("Node"));
     m_list->setExpandingColumn(1);
+    m_list->setMenu(0);
     connect(m_list, SIGNAL(clickItem(QListViewItem*)), this, SLOT(clickItem(QListViewItem*)));
     connect(m_list, SIGNAL(dragStart()), this, SLOT(dragStart()));
 
@@ -255,14 +256,14 @@ void *JabberBrowser::processEvent(Event *e)
 {
     if (e->type() == static_cast<JabberPlugin*>(m_client->protocol()->plugin())->EventAgentInfo){
         JabberAgentInfo *data = (JabberAgentInfo*)(e->param());
-        if (m_search_id == data->ReqID){
-            if (data->Type == NULL){
-                if (data->nOptions){
+        if (m_search_id == data->ReqID.ptr){
+            if (data->Type.ptr == NULL){
+                if (data->nOptions.value){
                     QString err;
-                    if (data->Label && *data->Label)
-                        err = i18n(data->Label);
+                    if (data->Label.ptr && *data->Label.ptr)
+                        err = i18n(data->Label.ptr);
                     if (err.isEmpty())
-                        err = i18n("Error %1") .arg(data->nOptions);
+                        err = i18n("Error %1") .arg(data->nOptions.value);
                     m_search_id = "";
                     delete m_search;
                     m_search = NULL;
@@ -284,14 +285,14 @@ void *JabberBrowser::processEvent(Event *e)
             m_search->m_search->addWidget(data);
             return e->param();
         }
-        if (m_reg_id == data->ReqID){
-            if (data->Type == NULL){
-                if (data->nOptions){
+        if (m_reg_id == data->ReqID.ptr){
+            if (data->Type.ptr == NULL){
+                if (data->nOptions.value){
                     QString err;
-                    if (data->Label && *data->Label)
-                        err = i18n(data->Label);
+                    if (data->Label.ptr && *data->Label.ptr)
+                        err = i18n(data->Label.ptr);
                     if (err.isEmpty())
-                        err = i18n("Error %1") .arg(data->nOptions);
+                        err = i18n("Error %1") .arg(data->nOptions.value);
                     m_reg_id = "";
                     delete m_reg;
                     m_reg = NULL;
@@ -313,14 +314,14 @@ void *JabberBrowser::processEvent(Event *e)
             m_reg->m_search->addWidget(data);
             return e->param();
         }
-        if (m_config_id == data->ReqID){
-            if (data->Type == NULL){
-                if (data->nOptions){
+        if (m_config_id == data->ReqID.ptr){
+            if (data->Type.ptr == NULL){
+                if (data->nOptions.value){
                     QString err;
-                    if (data->Label && *data->Label)
-                        err = i18n(data->Label);
+                    if (data->Label.ptr && *data->Label.ptr)
+                        err = i18n(data->Label.ptr);
                     if (err.isEmpty())
-                        err = i18n("Error %1") .arg(data->nOptions);
+                        err = i18n("Error %1") .arg(data->nOptions.value);
                     m_config_id = "";
                     delete m_config;
                     m_config = NULL;
@@ -498,9 +499,11 @@ void JabberBrowser::dragStart()
     if (item == NULL)
         return;
     Contact *contact;
-    JabberUserData *data = m_client->findContact(item->text(COL_JID).utf8(), NULL, false, contact);
+    string resource;
+    JabberUserData *data = m_client->findContact(item->text(COL_JID).utf8(), NULL, false, contact, resource);
     if (data == NULL){
-        m_client->findContact(item->text(COL_JID).utf8(), item->text(COL_NAME).utf8(), true, contact);
+        string resource;
+        m_client->findContact(item->text(COL_JID).utf8(), item->text(COL_NAME).utf8(), true, contact, resource);
         contact->setTemporary(CONTACT_DRAG);
     }
     m_list->startDrag(new ContactDragObject(m_list, contact));
