@@ -301,7 +301,7 @@ bool Contact::setLastName(const QString &name, const char *client)
     return setLastName(addString(getLastName(), name, client));
 }
 
-static char tipDiv[] = "<br>___________<br>";
+static char tipDiv[] = "<br>__________<br>";
 
 QString Contact::tipText()
 {
@@ -397,9 +397,9 @@ typedef struct sortClientData
 
 static bool cmp_sd(sortClientData p1, sortClientData p2)
 {
-    if (((clientData*)(p1.data))->LastSend > ((clientData*)(p2.data))->LastSend)
+    if (((clientData*)(p1.data))->LastSend.value > ((clientData*)(p2.data))->LastSend.value)
         return true;
-    if (((clientData*)(p1.data))->LastSend < ((clientData*)(p2.data))->LastSend)
+    if (((clientData*)(p1.data))->LastSend.value < ((clientData*)(p2.data))->LastSend.value)
         return false;
     return p1.nClient < p2.nClient;
 }
@@ -477,13 +477,18 @@ unsigned long Contact::contactInfo(unsigned &style, const char *&statusIcon, str
         }
         if (statusIcon == NULL)
             statusIcon = "nonim";
-        return STATUS_NA;
+        return STATUS_UNKNOWN;
     }
     if (statusIcon == NULL)
         statusIcon = "empty";
     return status;
 }
 
+string Client::resources(void*)
+{
+    return "";
+}
+
 QString Client::ownerName()
 {
     return "";
@@ -1021,12 +1026,13 @@ typedef struct ClientData
 
 static DataDef _clientData[] =
     {
-        { "ManualStatus", DATA_LONG, 1, STATUS_OFFLINE },
-        { "CommonStatus", DATA_BOOL, 1, 1 },
+        { "ManualStatus", DATA_LONG, 1, DATA(1) },
+        { "CommonStatus", DATA_BOOL, 1, DATA(1) },
         { "Password", DATA_UTF, 1, 0 },
-        { "", DATA_BOOL, 1, 1 },		// SavePassword
-        { "", DATA_UTF, 1, 0 },			// PreviousPassword
+        { "", DATA_BOOL, 1, DATA(1) },		// SavePassword
+        { "", DATA_UTF, 1, 0 },				// PreviousPassword
         { "Invisible", DATA_BOOL, 1, 0 },
+        { "LastSend", DATA_STRLIST, 1, 0 },
         { NULL, 0, 0, 0 }
     };
 
@@ -1070,6 +1076,17 @@ Client::~Client()
 
 void Client::freeData()
 {
+    ContactListPrivate *p = getContacts()->p;
+    for (vector<Client*>::iterator it = p->clients.begin(); it != p->clients.end(); ++it){
+        if ((*it) != this)
+            continue;
+        p->clients.erase(it);
+        if (!getContacts()->p->bNoRemove){
+            Event e(EventClientsChanged);
+            e.process();
+        }
+        break;
+    }
     Group *grp;
     ContactList::GroupIterator itg;
     while ((grp = ++itg) != NULL){
@@ -1088,25 +1105,19 @@ void Client::freeData()
             continue;
         contact->clientData.freeClientData(this);
         if (contact->clientData.size()){
-            contact->setup();
-            Event e(EventContactChanged, contact);
-            e.process();
+            if (!getContacts()->p->bNoRemove){
+                contact->setup();
+                Event e(EventContactChanged, contact);
+                e.process();
+            }
             continue;
         }
         forRemove.push_back(contact);
     }
     list<Contact*>::iterator itr;
     for (itr = forRemove.begin(); itr != forRemove.end(); ++itr){
-        delete *itr;
-    }
-    ContactListPrivate *p = getContacts()->p;
-    for (vector<Client*>::iterator it = p->clients.begin(); it != p->clients.end(); ++it){
-        if ((*it) != this)
-            continue;
-        p->clients.erase(it);
-        Event e(EventClientsChanged);
-        e.process();
-        break;
+        Contact *contact = *itr;
+        delete contact;
     }
     free_data(_clientData, &data);
 }
@@ -1233,6 +1244,29 @@ unsigned ClientUserData::size()
     return p->size();
 }
 
+string ClientUserData::property(const char *name)
+{
+    for (ClientUserDataPrivate::iterator it = p->begin(); it != p->end(); ++it){
+        _ClientUserData &d = *it;
+        Data *user_data = (Data*)d.data;
+        for (const DataDef *def = d.client->protocol()->userDataDef(); def->name; def++){
+            if (!strcmp(def->name, name)){
+                switch (def->type){
+                case DATA_STRING:
+                case DATA_UTF:
+                    if (user_data->ptr)
+                        return user_data->ptr;
+                case DATA_ULONG:
+                    if (user_data->value != (unsigned long)(def->def_value))
+                        return number(user_data->value);
+                }
+            }
+            user_data += def->n_values;
+        }
+    }
+    return "";
+}
+
 bool ClientUserData::have(void *data)
 {
     for (ClientUserDataPrivate::iterator it = p->begin(); it != p->end(); ++it){
@@ -1248,7 +1282,7 @@ Client *ClientUserData::activeClient(void *&data, Client *client)
     for (it = p->begin(); it != p->end(); ++it){
         if (((*it).client == client) && ((*it).data == data))
             break;
-        if (((clientData*)((*it).data))->Sign != ((clientData*)data)->Sign)
+        if (((clientData*)((*it).data))->Sign.value != ((clientData*)data)->Sign.value)
             continue;
         if (client->compareData(data, (*it).data))
             return NULL;
@@ -1258,9 +1292,9 @@ Client *ClientUserData::activeClient(void *&data, Client *client)
     if (client->getState() == Client::Connected)
         return client;
     for (++it; it != p->end(); ++it){
-        if (client->getState() != Client::Connected)
+        if ((*it).client->getState() != Client::Connected)
             continue;
-        if (((clientData*)((*it).data))->Sign != ((clientData*)data)->Sign)
+        if (((clientData*)((*it).data))->Sign.value != ((clientData*)data)->Sign.value)
             continue;
         if (client->compareData(data, (*it).data)){
             data = (*it).data;
@@ -1275,6 +1309,8 @@ string ClientUserData::save()
     string res;
     for (ClientUserDataPrivate::iterator it = p->begin(); it != p->end(); ++it){
         _ClientUserData &d = *it;
+        if (d.client->protocol()->description()->flags & PROTOCOL_TEMP_DATA)
+            continue;
         string cfg = save_data(d.client->protocol()->userDataDef(), d.data);
         if (cfg.length()){
             if (res.length())
@@ -1661,7 +1697,7 @@ void ContactList::save()
     }
 
     const int status = f.status();
-#if QT_VERSION >= 0x030200
+#if COMPAT_QT_VERSION >= 0x030200
     const QString errorMessage = f.errorString();
 #else
     const QString errorMessage = "Write file fail";
@@ -1674,7 +1710,11 @@ void ContactList::save()
 
     // rename to normal file
     QFileInfo fileInfo(f.name());
-    QString desiredFileName = QFile::decodeName(cfgName.c_str());
+    QString desiredFileName = fileInfo.fileName();
+    desiredFileName = desiredFileName.left(desiredFileName.length() - strlen(BACKUP_SUFFIX));
+#ifdef WIN32
+    fileInfo.dir().remove(desiredFileName);
+#endif
     if (!fileInfo.dir().rename(fileInfo.fileName(), desiredFileName)) {
         log(L_ERROR, "Can't rename file %s to %s", (const char*)fileInfo.fileName().local8Bit(), (const char*)desiredFileName.local8Bit());
         return;
@@ -1833,10 +1873,12 @@ static string stripPhone(const char *phone)
         return res;
     for (; *phone; phone++){
         char c = *phone;
-        if ((c < '0') || (c >= '9'))
+        if ((c < '0') || (c > '9'))
             continue;
         res += c;
     }
+    if (res.length() > 7)
+        res = res.substr(res.length() - 7);
     return res;
 }
 
@@ -1856,7 +1898,7 @@ Contact *ContactList::contactByPhone(const char *_phone)
         QString phones = c->getPhones();
         while (!phones.isEmpty()){
             QString phoneItem = getToken(phones, ';', false);
-            if (cmpPhone(phoneItem.utf8(), _phone))
+            if (cmpPhone(getToken(phoneItem, ',').utf8(), _phone))
                 return c;
         }
     }
@@ -1912,3 +1954,17 @@ EXPORT ContactList *getContacts()
 
 };
 
+EXPORT QString g_i18n(const char *text, Contact *contact)
+{
+    QString male = i18n("male", text);
+    if (contact == NULL)
+        return male;
+    QString female = i18n("female", text);
+    if (male == female)
+        return male;
+    string gender = contact->clientData.property("Gender");
+    if (atol(gender.c_str()) == 1)
+        return female;
+    return male;
+}
+
