@@ -31,6 +31,14 @@
 #include <ctype.h>
 #endif
 
+#ifndef  LTDL_SHLIB_EXT
+#if !defined(QT_MACOSX_VERSION) && !defined(QT_MAC)
+#define  LTDL_SHLIB_EXT ".so"
+#else	/* MacOS needs .a */
+#define  LTDL_SHLIB_EXT ".a"
+#endif
+#endif
+
 #include <errno.h>
 #include <qdir.h>
 #include <qstringlist.h>
@@ -160,10 +168,24 @@ PluginManagerPrivate::PluginManagerPrivate(int argc, char **argv)
     lt_dlinit();
 
     QStringList pluginsList;
+#ifdef WIN32
     QDir pluginDir(app_file("plugins").c_str());
+#else
+    QDir pluginDir(PLUGIN_PATH);
+#endif
+    /* do some test so we can blame when sim can't access / find
+       the plugins */
     pluginsList = pluginDir.entryList("*" LTDL_SHLIB_EXT);
+    if (pluginsList.isEmpty()) {
+        log(L_ERROR,
+            "Can't access %s or directory contains no plugins!",
+            pluginDir.path().ascii());
+        m_bAbort = true;
+        return;
+    }
     m_bAbort = false;
 
+    log(L_DEBUG,"Loading plugins from %s",pluginDir.path().ascii());
     for (QStringList::Iterator it = pluginsList.begin(); it != pluginsList.end(); ++it){
         QString f = *it;
         int p = f.findRev('.');
@@ -183,6 +205,7 @@ PluginManagerPrivate::PluginManagerPrivate(int argc, char **argv)
         info.info		 = NULL;
         info.base		 = 0;
         plugins.push_back(info);
+        log(L_DEBUG,"Found plugin %s",info.name);
     }
     sort(plugins.begin(), plugins.end(), cmp_plugin);
     for (vector<pluginInfo>::iterator itp = plugins.begin(); itp != plugins.end(); ++itp){
@@ -190,15 +213,19 @@ PluginManagerPrivate::PluginManagerPrivate(int argc, char **argv)
         if (m_bAbort)
             return;
     }
-    m_bInInit = false;
     Event eStart(EventInit);
-    eStart.process();
+    if ((int)eStart.process() == -1) {
+        log(L_ERROR,"EventInit failed - aborting!");
+        m_bAbort = true;
+        return;
+    }
     for (list<string>::iterator it_args = args.begin(); it_args != args.end(); ++it_args){
         if ((*it_args).length()){
             usage((*it_args).c_str());
             break;
         }
     }
+    m_bInInit = false;
 }
 
 PluginManagerPrivate::~PluginManagerPrivate()
@@ -245,6 +272,16 @@ void *PluginManagerPrivate::processEvent(Event *e)
         return (void*)(m_argc);
     case EventArgv:
         return (void*)(m_argv);
+    case EventLog:
+        if (m_bInInit) {
+            LogInfo *li = (LogInfo*)e->param();
+            if (li->log_level == L_ERROR) {
+                fprintf(stderr,"%s\n",li->log_info);
+            } else {
+                fprintf(stdout,"%s\n",li->log_info);
+            }
+        }
+        break;
 #ifndef WIN32
     case EventExec:
         exec = (ExecParam*)(e->param());
@@ -294,15 +331,15 @@ void PluginManagerPrivate::load(pluginInfo &info)
 #ifdef WIN32
         string pluginName = "plugins\\";
 #else
-        string pluginName = "plugins/";
+        string pluginName = PLUGIN_PATH;
+        pluginName += "/";
 #endif
         pluginName += info.name;
         pluginName += LTDL_SHLIB_EXT;
         string fullName = app_file(pluginName.c_str());
         info.module = (void*)lt_dlopen(fullName.c_str());
         if (info.module == NULL){
-            log(L_WARN, "Can't load plugin %s: %s", fullName.c_str(), lt_dlerror());
-            return;
+            log(L_WARN, "Can't load plugin %s", lt_dlerror());
         }
     }
     if (info.module == NULL)
@@ -311,7 +348,7 @@ void PluginManagerPrivate::load(pluginInfo &info)
         PluginInfo* (*getInfo)() = NULL;
         (lt_ptr&)getInfo = lt_dlsym((lt_dlhandle)info.module, "GetPluginInfo");
         if (getInfo == NULL){
-            log(L_WARN, "Plugin %s haven't entry GetInfo", info.name);
+            log(L_WARN, "Plugin %s hasn't entry GetPluginInfo", info.name);
             release(info);
             return;
         }
@@ -319,16 +356,16 @@ void PluginManagerPrivate::load(pluginInfo &info)
 #ifndef WIN32
 #ifdef USE_KDE
         if (!(info.info->flags & PLUGIN_KDE_COMPILE)){
-            log(L_WARN, "Plugin %s compile without KDE support", info.name);
+            log(L_WARN, "Plugin %s is compiled without KDE support!", info.name);
             release(info);
             return;
         }
 #else
-        if (info.info->flags & PLUGIN_KDE_COMPILE){
-            log(L_WARN, "Plugin %s compile with KDE support", info.name);
-            release(info);
-            return;
-        }
+if (info.info->flags & PLUGIN_KDE_COMPILE){
+        log(L_WARN, "Plugin %s is compiled with KDE support!", info.name);
+        release(info);
+        return;
+    }
 #endif
 #endif
     }
@@ -520,20 +557,24 @@ void PluginManagerPrivate::saveState()
     }
 
     const int status = f.status();
-#if QT_VERSION >= 0x030200
+#if COMPAT_QT_VERSION >= 0x030200
     const QString errorMessage = f.errorString();
 #else
     const QString errorMessage = "write file fail";
 #endif
     f.close();
     if (status != IO_Ok) {
-        log(L_ERROR, "IO error during writting to file %s : %s", (const char*)f.name().local8Bit(), (const char*)errorMessage.local8Bit());
+        log(L_ERROR, "I/O error during writing to file %s : %s", (const char*)f.name().local8Bit(), (const char*)errorMessage.local8Bit());
         return;
     }
 
     // rename to normal file
     QFileInfo fileInfo(f.name());
-    QString desiredFileName = QFile::decodeName(cfgName.c_str());
+    QString desiredFileName = fileInfo.fileName();
+    desiredFileName = desiredFileName.left(desiredFileName.length() - strlen(BACKUP_SUFFIX));
+#ifdef WIN32
+    fileInfo.dir().remove(desiredFileName);
+#endif
     if (!fileInfo.dir().rename(fileInfo.fileName(), desiredFileName)) {
         log(L_ERROR, "Can't rename file %s to %s", (const char*)fileInfo.fileName().local8Bit(), (const char*)desiredFileName.local8Bit());
         return;
