@@ -26,7 +26,7 @@
 #include <netinet/in.h>
 #include <netdb.h>
 #include <arpa/inet.h>
-#ifndef QT_MACOSX_VERSION
+#if !defined(QT_MACOSX_VERSION) && !defined(QT_MAC)
 #include <X11/X.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -47,15 +47,17 @@
 #include <qdatetime.h>
 #include <qcombobox.h>
 #include <qlineedit.h>
+#include <qmultilineedit.h>
 #include <qregexp.h>
 
-#if QT_VERSION >= 300
+#if COMPAT_QT_VERSION >= 0x030000
 #include <qdesktopwidget.h>
 #endif
 
 #ifdef USE_KDE
 #include <kwin.h>
 #include <kglobal.h>
+#include "kdeisversion.h"
 #endif
 
 #include "stl.h"
@@ -66,22 +68,34 @@
 
 QString i18n(const char *text)
 {
-    return i18n(text, NULL);
-}
-
-QString i18n(const char *text, const char *comment)
-{
-    if (text == NULL)
+    if ((text == NULL) || (*text == 0))
         return QString::null;
-    QString res = QObject::tr(text, comment);
+    QString res = QObject::tr(text);
     if (res != QString::fromLatin1(text))
         return res;
     return QString::fromUtf8(text);
 }
 
+QString i18n(const char *comment, const char *text)
+{
+    if (text == NULL)
+        return QString::null;
+    if (comment == NULL)
+        return i18n(text);
+    string s;
+    s = "_: ";
+    s += comment;
+    s += "\n";
+    s += text;
+    QString res = QObject::tr(s.c_str());
+    if (res != s.c_str())
+        return res;
+    return i18n(text);
+}
+
 #endif
 
-#if !defined(USE_KDE) || (QT_VERSION < 300)
+#if !defined(USE_KDE) || (COMPAT_QT_VERSION < 0x030000)
 
 static bool bPluralInit = false;
 static int plural_form = -1;
@@ -150,6 +164,10 @@ QString i18n(const char *singular, const char *plural, unsigned long n)
     char *newstring = new char[strlen(singular) + strlen(plural) + 6];
     sprintf(newstring, "_n: %s\n%s", singular, plural);
     QString r = i18n(newstring);
+    if (r == newstring){
+        delete[] newstring;
+        goto NoTranslate;
+    }
     delete [] newstring;
     initPlural();
     if ( r.isEmpty() || plural_form == -1) {
@@ -313,16 +331,26 @@ void EventReceiver::destroyList()
 #ifdef WIN32
 
 static WNDPROC oldWndProc = 0;
-static bool inSetCaption = false;
+static bool bSetCaption = false;
+static bool bResize     = false;
 static MSG m;
 
+bool inResize()
+{
+    return bResize;
+}
+
 LRESULT CALLBACK WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
 {
     if (msg == WM_SETTEXT){
-        if (!inSetCaption)
+        if (!bSetCaption)
             return 0;
         return DefWindowProc(hWnd, msg, wParam, lParam);
     }
+    if (msg == WM_ENTERSIZEMOVE)
+        bResize = true;
+    if (msg == WM_EXITSIZEMOVE)
+        bResize = false;
     if ((msg >= WM_KEYFIRST) && (msg <= WM_KEYLAST)){
         m.hwnd = hWnd;
         m.message = msg;
@@ -355,7 +383,7 @@ void setWndProc(QWidget *w)
 
 void mySetCaption(QWidget *w, const QString &caption)
 {
-    inSetCaption = true;
+    bSetCaption = true;
     if (IsWindowUnicode(w->winId())){
         unsigned size = caption.length();
         wchar_t *text = new wchar_t[size + 1];
@@ -366,11 +394,11 @@ void mySetCaption(QWidget *w, const QString &caption)
     }else{
         SendMessageA(w->winId(), WM_SETTEXT, 0, (LPARAM)(const char*)caption.local8Bit());
     }
-    inSetCaption = false;
+    bSetCaption = false;
 }
 
 #else
-#ifndef QT_MACOSX_VERSION
+#if !defined(QT_MACOSX_VERSION) && !defined(QT_MAC)
 
 void setWndClass(QWidget *w, const char *name)
 {
@@ -383,6 +411,12 @@ void setWndClass(QWidget *w, const char *name)
     XSetClassHint(dsp, win, &classhint);
 }
 
+#else
+
+void setWndClass(QWidget*, const char*)
+{
+}
+
 #endif
 #endif
 
@@ -399,20 +433,23 @@ bool raiseWindow(QWidget *w, unsigned)
     /* info.currentDesktop is 0 when iconified :(
     also onAllDesktops is 0 when Objekt isn't
     shown already */
+#if KDE_IS_VERSION(3,2,0)
+    KWin::WindowInfo info = KWin::windowInfo(w->winId());
+    if ((!info.onAllDesktops()) || (desk == 0)) {
+        if (desk == 0) desk = KWin::currentDesktop();
+        KWin::setOnDesktop(w->winId(), desk);
+    }
+#else
     KWin::Info info = KWin::info(w->winId());
     if ((!info.onAllDesktops) || (desk == 0)) {
         if (desk == 0) desk = KWin::currentDesktop();
         KWin::setOnDesktop(w->winId(), desk);
     }
-    log(L_DEBUG,"desk: %u all %u",desk,info.onAllDesktops);
+#endif
 #endif
     w->show();
     w->showNormal();
-    w->setActiveWindow();
     w->raise();
-#ifdef USE_KDE
-    KWin::setActiveWindow(w->winId());
-#endif
 #ifdef WIN32
     AttachThreadInput(GetWindowThreadProcessId(GetForegroundWindow(),NULL), GetCurrentThreadId(), TRUE);
     SetForegroundWindow(w->winId());
@@ -459,12 +496,6 @@ const QIconSet *Icon(const char *name)
     return res;
 }
 
-const QIconSet *BigIcon(const char *name)
-{
-    Event e(EventGetBigIcon, (void*)name);
-    return (const QIconSet*)e.process();
-}
-
 QPixmap Pict(const char *name)
 {
     const QIconSet *icons = Icon(name);
@@ -497,10 +528,10 @@ EXPORT QString formatDate(unsigned long t)
 #endif
 }
 
-EXPORT QString formatAddr(void *ip, unsigned port)
+EXPORT QString formatAddr(Data &ip, unsigned port)
 {
     QString res;
-    if (ip == NULL)
+    if (ip.ptr == NULL)
         return res;
     struct in_addr inaddr;
     inaddr.s_addr = get_ip(ip);
@@ -598,13 +629,49 @@ EXPORT void disableWidget(QWidget *w)
     QPalette pal = w->palette();
     pal.setDisabled(pal.active());
     w->setPalette(pal);
-    w->setEnabled(false);
+    if (w->inherits("QLineEdit")){
+        static_cast<QLineEdit*>(w)->setReadOnly(true);
+    }else if (w->inherits("QMulitLineEdit")){
+        static_cast<QMultiLineEdit*>(w)->setReadOnly(true);
+    }else{
+        w->setEnabled(false);
+    }
 }
 
+const unsigned char NOP = 172;
+
+static unsigned char gsmToLatin1Table[] =
+    {
+        '@', 163, '$', 165, 232, 233, 249, 236,
+        242, 199,  10, 216, 248,  13, 197, 229,
+        NOP, '_', NOP, NOP, NOP, NOP, NOP, NOP,
+        NOP, NOP, NOP, NOP, 198, 230, 223, 201,
+        ' ', '!', '"', '#', 164, '%', '&', '\'',
+        '(', ')', '*', '+', ',', '-', '.', '/',
+        '0', '1', '2', '3', '4', '5', '6', '7',
+        '8', '9', ':', ';', '<', '=', '>', '?',
+        161, 'A', 'B', 'C', 'D', 'E', 'F', 'G',
+        'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O',
+        'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W',
+        'X', 'Y', 'Z', 196, 214, 209, 220, 167,
+        191, 'a', 'b', 'c', 'd', 'e', 'f', 'g',
+        'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o',
+        'p', 'q', 'r', 's', 't', 'u', 'v', 'w',
+        'x', 'y', 'z', 228, 246, 241, 252, 224
+    };
+
 EXPORT bool isLatin(const QString &str)
 {
     for (int i = 0; i < (int)str.length(); i++){
-        if (str[i].unicode() > 127)
+        unsigned short c = str[i].unicode();
+        if ((c > 255) || (c == NOP))
+            return false;
+        unsigned n;
+        for (n = 0; n < sizeof(gsmToLatin1Table); n++){
+            if (gsmToLatin1Table[n] == c)
+                break;
+        }
+        if (n >= sizeof(gsmToLatin1Table))
             return false;
     }
     return true;
@@ -621,7 +688,7 @@ EXPORT QString getPart(QString &str, unsigned size)
     int n = size;
     QChar c = str[(int)size];
     if (!c.isSpace()){
-        for (; n >= 0; n++){
+        for (; n >= 0; n--){
             c = str[n];
             if (c.isSpace())
                 break;
@@ -749,6 +816,8 @@ EXPORT void setSmiles(const char *pp)
 DECLARE_HANDLE(HMONITOR);
 typedef BOOL (CALLBACK* MONITORENUMPROC)(HMONITOR, HDC, LPRECT, LPARAM);
 
+#if COMPAT_QT_VERSION < 0x030000
+
 static BOOL CALLBACK enumScreens(HMONITOR, HDC, LPRECT rc, LPARAM data)
 {
     vector<QRect> *p = (vector<QRect>*)data;
@@ -756,11 +825,12 @@ static BOOL CALLBACK enumScreens(HMONITOR, HDC, LPRECT rc, LPARAM data)
     return TRUE;
 }
 
+#endif
 #endif
 
 EXPORT unsigned screens()
 {
-#if QT_VERSION >= 300
+#if COMPAT_QT_VERSION >= 0x030000
     QDesktopWidget *desktop = QApplication::desktop();
     return desktop->numScreens();
 #else 
@@ -779,14 +849,14 @@ EXPORT unsigned screens()
     }
     return rc.size();
 #else
-return 1;
+    return 1;
 #endif
 #endif
 }
 
 EXPORT QRect screenGeometry(unsigned nScreen)
 {
-#if QT_VERSION >= 300
+#if COMPAT_QT_VERSION >= 0x030000
     QDesktopWidget *desktop = QApplication::desktop();
     return desktop->screenGeometry(nScreen);
 #else 
@@ -812,7 +882,7 @@ EXPORT QRect screenGeometry(unsigned nScreen)
 
 EXPORT QRect screenGeometry()
 {
-#if QT_VERSION >= 300
+#if COMPAT_QT_VERSION >= 0x030000
     QDesktopWidget *desktop = QApplication::desktop();
     QRect rc;
     for (int i = 0; i < desktop->numScreens(); i++){
@@ -843,6 +913,18 @@ EXPORT QRect screenGeometry()
 #endif
 }
 
+static bool bLog = true;
+
+EXPORT bool logEnabled()
+{
+    return bLog;
+}
+
+EXPORT void setLogEnable(bool log)
+{
+    bLog = log;
+}
+
 static bool bRandInit = false;
 
 EXPORT unsigned get_random()
@@ -856,6 +938,31 @@ EXPORT unsigned get_random()
     return rand();
 }
 
+my_string::my_string(const char *str)
+{
+    m_str = new string(str);
+}
+
+my_string::my_string(const my_string &s)
+{
+    m_str = new string(*s.m_str);
+}
+
+void my_string::operator = (const my_string &s)
+{
+    *m_str = *s.m_str;
+}
+
+my_string::~my_string()
+{
+    delete m_str;
+}
+
+bool my_string::operator < (const my_string &a) const
+{
+    return strcmp(m_str->c_str(), a.m_str->c_str()) < 0;
+}
+
 };
 
 #ifndef HAVE_STRCASECMP
