@@ -22,6 +22,12 @@
 #include "config.h"
 #endif
 
+#include "compatqtversion.h"
+
+#if COMPAT_QT_VERSION >= 0x030000
+#include <qtextedit.h>
+#endif
+
 #ifdef STDC_HEADERS
 #include <stdlib.h>
 #include <stddef.h>
@@ -76,6 +82,7 @@ typedef unsigned char _Bool;
 #pragma warning(disable: 4530)  
 #pragma warning(disable: 4663)  
 #endif
+#define QT_THREAD_SUPPORT 1
 #endif
 
 #if defined(_MSC_VER) && defined(_DEBUG) && !defined(NO_CHECK_NEW)
@@ -89,10 +96,20 @@ typedef unsigned char _Bool;
 #define new new(_NORMAL_BLOCK, __FILE__, __LINE__)
 #endif
 
+#ifdef USE_OPENSSL
+#include <openssl/ssl.h>
+#include <openssl/err.h>
+#include <openssl/bio.h>
+#include <openssl/rand.h>
+#endif
+
 #include <string>
 using namespace std;
 
 #include <qwidget.h>
+#if COMPAT_QT_VERSION >= 0x030000
+#include <qdockwindow.h>
+#endif
 
 #ifdef WIN32
 #if _MSC_VER > 1020
@@ -128,10 +145,8 @@ EXPORT int strcasecmp(const char *a, const char *b);
 
 #ifdef USE_KDE
 
-class QKeySequence;
-
 #include <klocale.h>
-#if QT_VERSION < 300
+#if COMPAT_QT_VERSION < 0x030000
 EXPORT QString i18n(const char *singular, const char *plural, unsigned long n);
 #endif
 #else
@@ -141,11 +156,11 @@ EXPORT QString i18n(const char *text, const char *comment);
 EXPORT QString i18n(const char *singular, const char *plural, unsigned long n);
 #define I18N_NOOP(A)	A
 #endif
-#if !defined(USE_KDE) || (QT_VERSION < 300)
+#if !defined(USE_KDE) || (COMPAT_QT_VERSION < 0x030000)
 EXPORT void resetPlural();
 #endif
 
-#if QT_VERSION < 300
+#if COMPAT_QT_VERSION < 0x030000
 
 #include <qpoint.h>
 
@@ -393,13 +408,16 @@ typedef struct IconDef
 {
     const char *name;
     const char **xpm;
-    bool  isSystem;
+    const char *system;
+    unsigned   flags;
+    IconDef() : name(NULL), xpm(NULL), system(NULL), flags(0) {}
 } IconDef;
 
+const unsigned ICON_COLOR_MASK	= 0x000000FF;
+
 /* Get icon (param is const char *name)
    return const QIconSet* or NULL */
 const unsigned EventGetIcon = 0x0402;
-const unsigned EventGetBigIcon = 0x0403;
 
 /* Event icons changed */
 const unsigned EventIconChanged = 0x0404;
@@ -487,6 +505,7 @@ const unsigned BTN_EDIT				= 0x4000;
 
 const unsigned BTN_HIDE				= 0x10000;
 const unsigned BTN_NO_BUTTON		= 0x20000;
+const unsigned BTN_DIV				= 0x40000;
 
 /* Command - remove command id */
 const unsigned EventCommandRemove = 0x0507;
@@ -629,6 +648,7 @@ typedef struct PaintView
     int		 margin;	// icon margin
     QWidget	 *win;		// window
     bool	 isStatic;	// need static background
+    bool	 isGroup;	// draw group
 } PaintView;
 
 /* Event repaint list view */
@@ -740,7 +760,6 @@ typedef struct ListenParam
 {
     ServerSocketNotify	*notify;
     TCPClient			*client;
-    unsigned short		port;
 } ListenParam;
 
 /* Event send & receive message
@@ -788,16 +807,6 @@ const unsigned EventMessageDecline	= 0x1109;
 const unsigned EventMessageSend		= 0x110A;
 const unsigned EventSend			= 0x110B;
 
-const unsigned EventFetchDone		= 0x1300;
-
-typedef struct fetchData
-{
-    unsigned	req_id;
-    unsigned	result;
-    Buffer		*data;
-    const char	*headers;
-} fetchData;
-
 const unsigned EventClientError		= 0x1301;
 
 typedef struct clientErrorData
@@ -869,6 +878,108 @@ private:
     CommandsMapIteratorPrivate *p;
 };
 
+// ____________________________________________________________________________________
+// Configuration
+
+typedef struct DataDef
+{
+    const char	*name;
+    unsigned	type;
+    unsigned	n_values;
+    const char	*def_value;
+} DataDef;
+
+typedef union Data
+{
+    char			*ptr;
+    unsigned long	value;
+    bool			bValue;
+} Data;
+
+#define DATA(A)	((const char*)(A))
+
+const unsigned DATA_STRING		= 0;
+const unsigned DATA_LONG		= 1;
+const unsigned DATA_ULONG		= 2;
+const unsigned DATA_BOOL		= 3;
+const unsigned DATA_STRLIST		= 4;
+const unsigned DATA_UTF			= 5;
+const unsigned DATA_IP			= 6;
+const unsigned DATA_STRUCT		= 7;
+const unsigned DATA_UTFLIST		= 8;
+const unsigned DATA_OBJECT		= 9;
+
+EXPORT void free_data(const DataDef *def, void *data);
+EXPORT void load_data(const DataDef *def, void *data, const char *config);
+EXPORT string save_data(const DataDef *def, void *data);
+
+EXPORT bool set_str(char **str, const char *value);
+EXPORT const char *get_str(const Data &strlist, unsigned index);
+EXPORT void clear_list(Data *strlist);
+EXPORT void set_str(Data *strlist, unsigned index, const char *value);
+EXPORT unsigned long get_ip(Data &ip);
+EXPORT const char *get_host(Data &ip);
+EXPORT bool set_ip(Data *ip, unsigned long value, const char *host=NULL);
+
+#define PROP_STRLIST(A)	\
+	const char *get##A(unsigned index) const { return get_str(data.A, index); } \
+	void set##A(unsigned index, const char *value) { set_str(&data.A, index, value); } \
+	void clear##A()	{ clear_list(&data.A); }
+
+#define PROP_UTFLIST(A)	\
+	QString get##A(unsigned index) const { return QString::fromUtf8(get_str(data.A, index)); } \
+	void set##A(unsigned index, const QString &value) { set_str(&data.A, index, value.utf8()); } \
+	void clear##A()	{ clear_list(&data.A); }
+
+#define PROP_STR(A) \
+	const char *get##A() const { return data.A.ptr ? data.A.ptr : ""; } \
+	bool set##A(const char *r) { return set_str(&data.A.ptr, r); }
+
+#define PROP_UTF8(A) \
+	QString get##A() const { return data.A.ptr ? QString::fromUtf8(data.A.ptr) : QString(""); } \
+	bool set##A(const QString &r) { return set_str(&data.A.ptr, r.utf8()); }
+
+#define VPROP_UTF8(A) \
+	virtual QString get##A() const { return data.A.ptr ? QString::fromUtf8(data.A.ptr) : QString(""); } \
+	bool set##A(const QString &r) { return set_str(&data.A.ptr, r.utf8()); }
+
+#define PROP_LONG(A) \
+	unsigned long get##A() const { return data.A.value; } \
+	void set##A(unsigned long r) { data.A.value = r; }
+
+#define PROP_ULONG(A) \
+	unsigned long get##A() const { return data.A.value; } \
+	void set##A(unsigned long r) { data.A.value = r; }
+
+#define PROP_USHORT(A) \
+	unsigned short get##A() const { return (unsigned short)(data.A.value); } \
+	void set##A(unsigned short r) { data.A.value = r; }
+
+#define PROP_BOOL(A) \
+	bool get##A() const { return data.A.bValue; } \
+	void set##A(bool r) { data.A.bValue = r; }
+
+#define VPROP_BOOL(A) \
+	bool get##A() const { return data.A.bValue; } \
+	virtual void set##A(bool r) { data.A.bValue = r; }
+
+#define PROP_IP(A)	\
+	unsigned long get##A()	const { return get_ip(data.A); } \
+	const char *host##A() { return get_host(data.A); } \
+	void set##A(unsigned long r) { set_ip(&data.A, r); }
+
+const int LEFT		= 0;
+const int TOP		= 1;
+const int WIDTH		= 2;
+const int HEIGHT	= 3;
+const int DESKTOP	= 4;
+
+EXPORT void saveGeometry(QWidget*, Data[5]);
+EXPORT void restoreGeometry(QWidget*, Data[5], bool bPos, bool bSize);
+EXPORT void saveToolbar(QToolBar*, Data[7]);
+EXPORT void restoreToolbar(QToolBar*, Data[7]);
+EXPORT bool cmp(char *s1, char *s2);
+
 // _____________________________________________________________________________________
 // Utilities
 
@@ -889,7 +1000,7 @@ EXPORT string trim(const char *str);
 EXPORT QString trim(const QString &str);
 EXPORT QString formatDateTime(unsigned long t);
 EXPORT QString formatDate(unsigned long t);
-EXPORT QString formatAddr(void *addr, unsigned port);
+EXPORT QString formatAddr(Data &addr, unsigned port);
 EXPORT bool getLine(QFile &f, string &s);
 EXPORT string getToken(string &from, char c, bool bUnEscape=true);
 EXPORT QString getToken(QString &from, char c, bool bUnEsacpe=true);
@@ -898,88 +1009,6 @@ EXPORT QString quoteChars(const QString &from, const char *chars, bool bQuoteSla
 EXPORT char fromHex(char);
 EXPORT string unquoteString(const char *p);
 
-// ____________________________________________________________________________________
-// Configuration
-
-typedef struct DataDef
-{
-    const char	*name;
-    unsigned	type;
-    unsigned	n_values;
-    unsigned	def_value;
-} DataDef;
-
-const unsigned DATA_STRING		= 0;
-const unsigned DATA_LONG		= 1;
-const unsigned DATA_ULONG		= 2;
-const unsigned DATA_BOOL		= 3;
-const unsigned DATA_STRLIST		= 4;
-const unsigned DATA_UTF			= 5;
-const unsigned DATA_IP			= 6;
-const unsigned DATA_STRUCT		= 7;
-const unsigned DATA_UTFLIST		= 8;
-const unsigned DATA_OBJECT		= 9;
-
-EXPORT void free_data(const DataDef *def, void *data);
-EXPORT void load_data(const DataDef *def, void *data, const char *config);
-EXPORT string save_data(const DataDef *def, void *data);
-EXPORT bool set_str(char **str, const char *value);
-EXPORT const char *get_str(void *strlist, unsigned index);
-EXPORT void clear_list(void **strlist);
-EXPORT void set_str(void **strlist, unsigned index, const char *value);
-EXPORT unsigned long get_ip(void *ip);
-EXPORT const char *get_host(void *ip);
-EXPORT bool set_ip(void **ip, unsigned long value);
-
-#define PROP_STRLIST(A)	\
-	const char *get##A(unsigned index) { return get_str(data.A, index); } const	\
-	void set##A(unsigned index, const char *value) { set_str(&data.A, index, value); } \
-	void clear##A()	{ clear_list(&data.A); }
-
-#define PROP_STR(A) \
-	const char *get##A() { return data.A ? data.A : ""; } const \
-	bool set##A(const char *r) { return set_str(&data.A, r); }
-
-#define PROP_UTF8(A) \
-	QString get##A() { return data.A ? QString::fromUtf8(data.A) : QString(""); } const \
-	bool set##A(const QString &r) { return set_str(&data.A, r.utf8()); }
-
-#define VPROP_UTF8(A) \
-	virtual QString get##A() { return data.A ? QString::fromUtf8(data.A) : QString(""); } const \
-	bool set##A(const QString &r) { return set_str(&data.A, r.utf8()); }
-
-#define PROP_LONG(A) \
-	unsigned long get##A() { return data.A; } const \
-	void set##A(unsigned long r) { data.A = r; }
-
-#define PROP_ULONG(A) \
-	unsigned long get##A() { return data.A; } const \
-	void set##A(unsigned long r) { data.A = r; }
-
-#define PROP_USHORT(A) \
-	unsigned short get##A() { return (unsigned short)(data.A); } const \
-	void set##A(unsigned short r) { data.A = r; }
-
-#define PROP_BOOL(A) \
-	bool get##A() { return data.A != 0; } const \
-	void set##A(bool r) { data.A = r ? -1 : 0; }
-
-#define PROP_IP(A)	\
-	unsigned long get##A()	{ return get_ip(data.A); } const \
-	const char *host##A() { return get_host(data.A); } \
-	void set##A(unsigned long r) { set_ip(&data.A, r); }
-
-const int LEFT		= 0;
-const int TOP		= 1;
-const int WIDTH		= 2;
-const int HEIGHT	= 3;
-const int DESKTOP	= 4;
-
-EXPORT void saveGeometry(QWidget*, long[5]);
-EXPORT void restoreGeometry(QWidget*, long[5], bool bPos, bool bSize);
-EXPORT void saveToolbar(QToolBar*, long[7]);
-EXPORT void restoreToolbar(QToolBar*, long[7]);
-EXPORT bool cmp(char *s1, char *s2);
 
 // _____________________________________________________________________________________
 // Message
@@ -1005,6 +1034,7 @@ const unsigned	MESSAGE_RICHTEXT	= 0x00000002;
 const unsigned	MESSAGE_SECURE		= 0x00000004;
 const unsigned	MESSAGE_URGENT		= 0x00000008;
 const unsigned	MESSAGE_LIST		= 0x00000010;
+const unsigned	MESSAGE_NOVIEW		= 0x00000020;
 
 const unsigned	MESSAGE_SAVEMASK	= 0x0000FFFF;
 
@@ -1021,14 +1051,15 @@ const unsigned	MESSAGE_TEMP		= 0x10000000;
 
 typedef struct MessageData
 {
-    char		*Text;			// Message text (UTF-8)
-    unsigned	Flags;			//
-    unsigned	Background;
-    unsigned	Foreground;
-    unsigned	Time;
-    char		*Font;
-    char		*Error;
-    unsigned	RetryCode;
+    Data		Text;			// Message text (UTF-8)
+    Data		Flags;			//
+    Data		Background;
+    Data		Foreground;
+    Data		Time;
+    Data		Font;
+    Data		Error;
+    Data		RetryCode;
+    Data		Resource;
 } MessageData;
 
 class EXPORT Message
@@ -1036,10 +1067,10 @@ class EXPORT Message
 public:
     Message(unsigned type = MessageGeneric, const char *cfg = NULL);
     virtual ~Message();
-    unsigned type() { return m_type; }
-    unsigned id() { return m_id; }
+    unsigned type() const { return m_type; }
+    unsigned id() const { return m_id; }
     void setId(unsigned id) { m_id = id; }
-    unsigned contact() { return m_contact; }
+    unsigned contact() const { return m_contact; }
     void setContact(unsigned contact) { m_contact = contact; }
     virtual string save();
     virtual unsigned baseType() { return m_type; }
@@ -1054,7 +1085,8 @@ public:
     PROP_STR(Error);
     PROP_STR(Font);
     PROP_ULONG(RetryCode);
-    const char *client() { return m_client.c_str(); }
+    PROP_UTF8(Resource);
+    const char *client() const { return m_client.c_str(); }
     void setClient(const char *client);
     virtual QString presentation();
 protected:
@@ -1067,8 +1099,8 @@ protected:
 
 typedef struct MessageSMSData
 {
-    char	*Phone;
-    char	*Network;
+    Data	Phone;
+    Data	Network;
 } MessageSMSData;
 
 class EXPORT SMSMessage : public Message
@@ -1086,9 +1118,9 @@ protected:
 
 typedef struct MessageFileData
 {
-    char		*File;
-    char		*Description;
-    unsigned	Size;
+    Data		File;
+    Data		Description;
+    Data		Size;
 } MessageFileData;
 
 class FileMessageIteratorPrivate;
@@ -1135,7 +1167,6 @@ public:
         Read,
         Write,
         Done,
-
         Wait,
         Error
     };
@@ -1144,6 +1175,8 @@ public:
     virtual void	startReceive(unsigned pos) = 0;
     virtual void	setError();
     void	addFile(const QString &file, unsigned size);
+    QString filename()		{ return m_name; }
+    bool	isDirectory()	{ return m_bDir; }
 protected:
     bool	openFile();
     FileMessage			*m_msg;
@@ -1163,7 +1196,10 @@ protected:
 
     OverwriteMode m_overwrite;
     QString	 m_dir;
+    QString  m_base;
+    QString  m_name;
     State	 m_state;
+    bool	 m_bDir;
     friend class FileMessage;
 };
 
@@ -1190,6 +1226,7 @@ public:
         const QString *operator[](unsigned);
         void reset();
         unsigned count();
+        unsigned dirs();
         unsigned size();
     protected:
         FileMessageIteratorPrivate *p;
@@ -1212,7 +1249,7 @@ public:
 
 typedef struct MessageUrlData
 {
-    char	*Url;
+    Data	Url;
 } MessageUrlData;
 
 class EXPORT UrlMessage : public Message
@@ -1229,7 +1266,7 @@ protected:
 
 typedef struct MessageContactsData
 {
-    char	*Contacts;
+    Data	Contacts;
 } MessageContactsData;
 
 class EXPORT ContactsMessage : public Message
@@ -1246,7 +1283,7 @@ protected:
 
 typedef struct MessageStatusData
 {
-    unsigned long Status;
+    Data	Status;
 } MessageStatusData;
 
 class EXPORT StatusMessage : public Message
@@ -1260,7 +1297,6 @@ protected:
     MessageStatusData data;
 };
 
-
 // _____________________________________________________________________________________
 // Contact list
 
@@ -1285,8 +1321,8 @@ class ClientDataIteratorPrivate;
 
 typedef struct clientData		// Base struct for all clientData
 {
-    unsigned	Sign;
-    unsigned	LastSend;
+    Data	Sign;
+    Data	LastSend;
 } clientData;
 
 class EXPORT ClientUserData
@@ -1306,6 +1342,7 @@ public:
     void join(clientData *cData, ClientUserData &data);
     unsigned size();
     Client *activeClient(void *&data, Client *client);
+    string property(const char *name);
 protected:
     ClientUserDataPrivate *p;
     friend class ClientDataIterator;
@@ -1344,17 +1381,17 @@ const unsigned PAGER	= 3;
 
 typedef struct ContactData
 {
-    unsigned long	Group;		// Group ID
-    char			*Name;		// Contact Display Name (UTF-8)
-    unsigned long	Ignore;		// In ignore list
-    unsigned long	LastActive;
-    char			*EMails;
-    char			*Phones;
-    unsigned long	PhoneStatus;
-    char			*FirstName;
-    char			*LastName;
-    char			*Notes;
-    unsigned long	Temporary;
+    Data			Group;		// Group ID
+    Data			Name;		// Contact Display Name (UTF-8)
+    Data			Ignore;		// In ignore list
+    Data			LastActive;
+    Data			EMails;
+    Data			Phones;
+    Data			PhoneStatus;
+    Data			FirstName;
+    Data			LastName;
+    Data			Notes;
+    Data			Temporary;
 } ContactData;
 
 const unsigned CONTACT_TEMP	= 1;
@@ -1397,7 +1434,7 @@ protected:
 
 typedef struct GroupData
 {
-    char			*Name;		// Display name (UTF-8)
+    Data		Name;		// Display name (UTF-8)
 } GroupData;
 
 class EXPORT Group
@@ -1429,14 +1466,19 @@ const unsigned CONTACT_UNDERLINE	= 0x0001;
 const unsigned CONTACT_ITALIC		= 0x0002;
 const unsigned CONTACT_STRIKEOUT	= 0x0004;
 
-const unsigned PROTOCOL_INFO			= 0x010000;
-const unsigned PROTOCOL_SEARCH			= 0x020000;
-const unsigned PROTOCOL_SEARCH_ONLINE	= 0x040000;
-const unsigned PROTOCOL_INVISIBLE		= 0x080000;
-const unsigned PROTOCOL_AR				= 0x100000;
-const unsigned PROTOCOL_AR_USER			= 0x200000;
-const unsigned PROTOCOL_FOLLOWME		= 0x400000;
-const unsigned PROTOCOL_ANY_PORT		= 0x800000;
+const unsigned PROTOCOL_INFO			= 0x00010000;
+const unsigned PROTOCOL_SEARCH			= 0x00020000;
+const unsigned PROTOCOL_SEARCH_ONLINE	= 0x00040000;
+const unsigned PROTOCOL_INVISIBLE		= 0x00080000;
+const unsigned PROTOCOL_AR				= 0x00100000;
+const unsigned PROTOCOL_AR_USER			= 0x00200000;
+const unsigned PROTOCOL_FOLLOWME		= 0x00400000;
+const unsigned PROTOCOL_ANY_PORT		= 0x00800000;
+const unsigned PROTOCOL_NOSMS			= 0x01000000;
+const unsigned PROTOCOL_NOPROXY			= 0x02000000;
+const unsigned PROTOCOL_TEMP_DATA		= 0x04000000;
+const unsigned PROTOCOL_NODATA			= 0x08000000;
+const unsigned PROTOCOL_NO_AUTH			= 0x10000000;
 
 class ContactList;
 class Client;
@@ -1457,12 +1499,13 @@ protected:
 
 typedef struct ClientData
 {
-    unsigned	ManualStatus;
-    unsigned	CommonStatus;
-    char		*Password;
-    unsigned	SavePassword;
-    char		*PreviousPassword;
-    unsigned	Invisible;
+    Data	ManualStatus;
+    Data	CommonStatus;
+    Data	Password;
+    Data	SavePassword;
+    Data	PreviousPassword;
+    Data	Invisible;
+    Data	LastSend;
 } ClientData;
 
 const unsigned AuthError = 1;
@@ -1502,6 +1545,7 @@ public:
     virtual void updateInfo(Contact *contact, void *clientData);
     virtual void setClientInfo(void *data);
     virtual QWidget *searchWindow();
+    virtual string resources(void *data);
     void    setState(State, const char *text = NULL, unsigned code = 0);
     State   getState() { return m_state; }
     virtual unsigned getStatus();
@@ -1510,10 +1554,9 @@ public:
     PROP_UTF8(Password)
     PROP_BOOL(SavePassword)
     PROP_UTF8(PreviousPassword)
-    bool getInvisible() { return data.Invisible != 0; }
-    virtual void setInvisible(bool bInvisible) { data.Invisible = bInvisible; }
+    PROP_STRLIST(LastSend)
+    VPROP_BOOL(Invisible)
 protected:
-
     void  freeData();
     State m_state;
     unsigned m_status;
@@ -1688,11 +1731,6 @@ typedef struct pager_provider
 
 EXPORT const pager_provider *getProviders();
 
-// _____________________________________________________________________________________
-// Fetch Data
-
-EXPORT unsigned fetch(TCPClient *client, const char *url, Buffer *postData=NULL, const char *headers=NULL);
-
 // _____________________________________________________________________________________
 // User interface
 
@@ -1717,7 +1755,6 @@ EXPORT bool raiseWindow(QWidget *w, unsigned desk = 0);
 EXPORT void setButtonsPict(QWidget *w);
 
 EXPORT const QIconSet *Icon(const char *name);
-EXPORT const QIconSet *BigIcon(const char *name);
 EXPORT QPixmap Pict(const char *name);
 
 EXPORT void setAboutData(KAboutData*);
@@ -1749,8 +1786,27 @@ EXPORT QRect screenGeometry(unsigned nScreen);
 EXPORT QRect screenGeometry();
 
 EXPORT unsigned get_random();
+
+class EXPORT my_string
+{
+public:
+    my_string(const char *str);
+    my_string(const my_string &str);
+    ~my_string();
+    bool operator < (const my_string &str) const;
+    void operator = (const my_string &str);
+    const char *c_str() const { return m_str->c_str(); }
+    string *m_str;
+};
+
+EXPORT bool inResize();
+EXPORT bool logEnabled();
+EXPORT void setLogEnable(bool);
+
 };
 
 using namespace SIM;
 
+EXPORT QString g_i18n(const char *text, Contact *contact);
+
 #endif
