@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qcomplextext.cpp,v 1.5 2003/11/23 13:33:48 shutoff Exp $
+** $Id: qcomplextext.cpp,v 1.9 2004/02/23 16:00:29 thermoman Exp $
 **
 ** Implementation of some internal classes
 **
@@ -35,6 +35,12 @@
 **
 **********************************************************************/
 
+
+#include <qglobal.h>
+#include "compatqtversion.h"
+
+#if COMPAT_QT_VERSION < 0x030000
+
 #ifdef WIN32
 #if _MSC_VER > 1020
 #include <yvals.h>              
@@ -42,9 +48,8 @@
 #endif
 #endif
 
-#include "qcomplextext_p.h"
 
-#if QT_VERSION < 300
+#include "qcomplextext_p.h"
 #ifndef QT_NO_COMPLEXTEXT
 #include "qrichtext_p.h"
 #include "qfontmetrics.h"
