@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qrichtext_p.h,v 1.7 2003/11/23 13:33:48 shutoff Exp $
+** $Id: qrichtext_p.h,v 1.10 2004/03/16 16:15:30 shutoff Exp $
 **
 ** Definition of internal rich text classes
 **
@@ -75,7 +75,7 @@
 #include "qt3stuff.h"
 #include "qmap.h"
 
-#if QT_VERSION < 300
+#if COMPAT_QT_VERSION < 0x030000
 #ifndef QT_NO_RICHTEXT
 
 //#define DEBUG_COLLECTION
@@ -311,6 +311,7 @@ QTextParag *topParag() const { return parags.isEmpty() ? string : parags.first()
     int y() const;
 
     int nestedDepth() const { return (int)indices.count(); } //### size_t/int cast
+    void oneUp() { if ( !indices.isEmpty() ) pop(); }
 
 private:
     enum Operation { EnterBegin, EnterEnd, Next, Prev, Up, Down };
@@ -739,7 +740,7 @@ public:
     int numSelections() const { return nSelections; }
     void addSelection( int id );
 
-    QString selectedText( int id, bool withCustom = TRUE ) const;
+    QString selectedText( int id, bool asHTML = FALSE ) const;
     void copySelectedText( int id );
     void removeSelectedText( int id, QTextCursor *cursor );
     void indentSelection( int id );
@@ -856,7 +857,7 @@ private:
     QChar parseHTMLSpecialChar(const QString& doc, int& pos);
     QString parseWord(const QString& doc, int& pos, bool lower = TRUE);
     QChar parseChar(const QString& doc, int& pos, QStyleSheetItem::WhiteSpaceMode wsm );
-    void setRichTextInternal( const QString &text );
+    void setRichTextInternal( const QString &text, QTextCursor* cursor = NULL );
 
 private:
     struct Focus {
@@ -1819,28 +1820,6 @@ inline QTextFormatCollection *QTextFormat::parent() const
     return collection;
 }
 
-inline void QTextFormat::addRef()
-{
-    ref++;
-#ifdef DEBUG_COLLECTION
-    qDebug( "add ref of '%s' to %d (%p)", k.latin1(), ref, this );
-#endif
-}
-
-inline void QTextFormat::removeRef()
-{
-    ref--;
-    if ( !collection )
-        return;
-    if ( this == collection->defFormat )
-        return;
-#ifdef DEBUG_COLLECTION
-    qDebug( "remove ref of '%s' to %d (%p)", k.latin1(), ref, this );
-#endif
-    if ( ref == 0 )
-        collection->remove( this );
-}
-
 inline QString QTextFormat::key() const
 {
     return k;
