@@ -63,6 +63,74 @@
 
 #define MAX_HISTORY	100
 
+class RichTextDrag : public QTextDrag
+{
+public:
+    RichTextDrag(QWidget *dragSource = 0, const char *name = 0);
+
+    void setRichText(const QString &txt);
+
+    virtual QByteArray encodedData(const char *mime) const;
+    virtual const char* format(int i) const;
+
+    static bool decode(QMimeSource *e, QString &str, const QCString &mimetype, const QCString &subtype);
+    static bool canDecode( QMimeSource* e );
+private:
+    QString richTxt;
+};
+
+RichTextDrag::RichTextDrag( QWidget *dragSource, const char *name )
+        : QTextDrag( dragSource, name )
+{
+}
+
+QByteArray RichTextDrag::encodedData( const char *mime ) const
+{
+    if ( qstrcmp( "application/x-qrichtext", mime ) == 0 ) {
+        return richTxt.utf8(); // #### perhaps we should use USC2 instead?
+    } else
+        return QTextDrag::encodedData( mime );
+}
+
+bool RichTextDrag::decode(QMimeSource *e, QString &str, const QCString &mimetype, const QCString &subtype)
+{
+    if (mimetype == "application/x-qrichtext"){
+        const char *mime;
+        int i;
+        for ( i = 0; (mime = e->format(i)) != NULL; ++i ) {
+            if (qstrcmp("application/x-qrichtext", mime) != 0)
+                continue;
+            str = QString::fromUtf8(e->encodedData(mime));
+            return TRUE;
+        }
+        return FALSE;
+    }
+    QCString subt = subtype;
+    return QTextDrag::decode( e, str, subt );
+}
+
+bool RichTextDrag::canDecode(QMimeSource* e)
+{
+    if ( e->provides("application/x-qrichtext"))
+        return TRUE;
+    return QTextDrag::canDecode(e);
+}
+
+const char* RichTextDrag::format(int i) const
+{
+    if ( QTextDrag::format(i))
+        return QTextDrag::format(i);
+    if ( QTextDrag::format(i-1))
+        return "application/x-qrichtext";
+    return 0;
+}
+
+void RichTextDrag::setRichText(const QString &txt)
+{
+    richTxt = txt;
+    setText(unquoteText(txt));
+}
+
 TextEdit::TextEdit(QWidget *p, const char *name)
         : TextShow(p, name)
 {
@@ -74,17 +142,18 @@ TextEdit::TextEdit(QWidget *p, const char *name)
     m_bSelected  = false;
     m_bNoSelected = false;
     m_bInClick = false;
+    m_bChanged = false;
     setReadOnly(false);
     curFG = colorGroup().color(QColorGroup::Text);
     m_bCtrlMode = true;
     setWordWrap(WidgetWidth);
-#if QT_VERSION >= 0x030100
+#if COMPAT_QT_VERSION >= 0x030100
     setAutoFormatting(0);
 #endif
     connect(this, SIGNAL(currentFontChanged(const QFont&)), this, SLOT(fontChanged(const QFont&)));
     connect(this, SIGNAL(currentColorChanged(const QColor&)), this, SLOT(slotColorChanged(const QColor&)));
     connect(this, SIGNAL(textChanged()), this, SLOT(slotTextChanged()));
-#if QT_VERSION >= 300
+#if COMPAT_QT_VERSION >= 0x030000
     connect(this, SIGNAL(clicked(int,int)), this, SLOT(slotClicked(int,int)));
 #endif
     viewport()->installEventFilter(this);
@@ -116,7 +185,7 @@ void TextEdit::slotTextChanged()
 
 void TextEdit::slotClicked(int,int)
 {
-#if QT_VERSION >= 300
+#if COMPAT_QT_VERSION >= 0x030000
     int paraFrom, paraTo, indexFrom, indexTo;
     getSelection(&paraFrom, &indexFrom, &paraTo, &indexTo);
     if ((paraFrom != paraTo) || (indexFrom != indexTo))
@@ -142,7 +211,7 @@ bool TextEdit::isEmpty()
         QString t = text(0);
         if (textFormat() == QTextEdit::RichText)
             t = unquoteText(t);
-        return t.isEmpty();
+        return t.isEmpty() || (t == " ");
     }
     return false;
 }
@@ -154,6 +223,15 @@ void TextEdit::setParam(void *param)
 
 void TextEdit::slotColorChanged(const QColor &c)
 {
+    if (c == curFG)
+        return;
+    int parag;
+    int index;
+    getCursorPosition(&parag, &index);
+    if (QTextEdit::text(parag).isEmpty()){
+        setColor(curFG);
+        return;
+    }
     if (c != curFG)
         setForeground(c, false);
 }
@@ -193,6 +271,7 @@ void TextEdit::fontChanged(const QFont &f)
     }
     if (m_param == NULL)
         return;
+    m_bChanged = true;
     if (f.bold() != m_bBold){
         m_bBold = f.bold();
         Command cmd;
@@ -220,6 +299,7 @@ void TextEdit::fontChanged(const QFont &f)
         Event e(EventCommandChecked, cmd);
         e.process();
     }
+    m_bChanged = false;
 }
 
 void TextEdit::setCtrlMode(bool mode)
@@ -229,8 +309,14 @@ void TextEdit::setCtrlMode(bool mode)
 
 void TextEdit::keyPressEvent(QKeyEvent *e)
 {
-    if (((e->key() == Key_Enter) || (e->key() == Key_Return))){
-        if (!m_bCtrlMode || (e->state() & ControlButton)){
+    if (((e->key() == Key_Enter) || (e->key() == Key_Return)))
+    {
+        //   in m_bCtrlMode:    enter      --> newLine
+        //                      ctrl+enter --> sendMsg
+        //   in !m_bCtrlMode:   enter      --> sendMsg
+        //                      ctrl+enter --> newLine
+        // the (bool) is required due to the bitmap
+        if (m_bCtrlMode == (bool)(e->state() & ControlButton)){
             emit ctrlEnterPressed();
             return;
         }
@@ -245,7 +331,7 @@ void TextEdit::keyPressEvent(QKeyEvent *e)
             return;
         }
     }
-#if (QT_VERSION >= 300) && (QT_VERSION < 0x030100)
+#if (COMPAT_QT_VERSION >= 0x030000) && (COMPAT_QT_VERSION < 0x030100)
     // Workaround about autoformat feature in qt 3.0.x
     if ((e->text()[0] == '-') || (e->text()[0] == '*')){
         if (isOverwriteMode() && !hasSelectedText())
@@ -254,13 +340,8 @@ void TextEdit::keyPressEvent(QKeyEvent *e)
         return;
     }
 #endif
-#if QT_VERSION >= 300
-    if ((e->key() == Key_Return) || (e->key() == Key_Enter)){
-        QKeyEvent e1(QEvent::KeyPress, e->key(), e->ascii(), e->state() | ControlButton, e->text(), e->count());
-        QTextEdit::keyPressEvent(&e1);
-        return;
-    }
-#endif
+    // Note: We no longer translate Enter to Ctrl-Enter since we need
+    // to know about paragraph breaks now.
     TextShow::keyPressEvent(e);
 }
 
@@ -317,16 +398,22 @@ void *TextEdit::processEvent(Event *e)
                 return e->param();
             }
         case CmdBold:
-            m_bSelected = true;
-            setBold((cmd->flags & COMMAND_CHECKED) != 0);
+            if (!m_bChanged){
+                m_bSelected = true;
+                setBold((cmd->flags & COMMAND_CHECKED) != 0);
+            }
             return e->param();
         case CmdItalic:
-            m_bSelected = true;
-            setItalic((cmd->flags & COMMAND_CHECKED) != 0);
+            if (!m_bChanged){
+                m_bSelected = true;
+                setItalic((cmd->flags & COMMAND_CHECKED) != 0);
+            }
             return e->param();
         case CmdUnderline:
-            m_bSelected = true;
-            setUnderline((cmd->flags & COMMAND_CHECKED) != 0);
+            if (!m_bChanged){
+                m_bSelected = true;
+                setUnderline((cmd->flags & COMMAND_CHECKED) != 0);
+            }
             return e->param();
         case CmdFont:{
 #ifdef USE_KDE
@@ -371,9 +458,18 @@ void TextEdit::setForeground(const QColor& c, bool bDef)
     curFG = c;
     if (bDef)
         defFG = c;
-    setColor(c);
+    if (!hasSelectedText())
+        setColor(c);
+    int r = c.red();
+#if COMPAT_QT_VERSION >= 0x030000
+    if (r){
+        r--;
+    }else{
+        r++;
+    }
+#endif
     QPalette pal = palette();
-    pal.setColor(QPalette::Active, QColorGroup::Text, c);
+    pal.setColor(QPalette::Active, QColorGroup::Text, QColor(r, c.green(), c.blue()));
     setPalette(pal);
 }
 
@@ -405,7 +501,7 @@ void TextEdit::setTextFormat(QTextEdit::TextFormat format)
         QTextEdit::setTextFormat(format);
         return;
     }
-    QString t = plainText(0, paragraphs(), 0, 0);
+    QString t = unquoteText(text());
     QTextEdit::setTextFormat(format);
     setText(t);
 }
@@ -415,6 +511,12 @@ TextShow::TextShow(QWidget *p, const char *name)
 {
     setTextFormat(RichText);
     setReadOnly(true);
+#if COMPAT_QT_VERSION >= 0x030100
+    if (QApplication::clipboard()->supportsSelection())
+        connect(this, SIGNAL(selectionChanged()), this, SLOT(slotSelectionChanged()));
+#endif
+    m_timer = new QTimer(this);
+    connect(m_timer, SIGNAL(timeout()), this, SLOT(slotResizeTimer()));
 }
 
 TextShow::~TextShow()
@@ -461,8 +563,31 @@ void TextShow::setSource(const QString &name)
 #endif
 }
 
+void TextShow::slotResizeTimer()
+{
+#ifdef WIN32
+    if (inResize())
+        return;
+    m_timer->stop();
+    setVScrollBarMode(Auto);
+    setHScrollBarMode(Auto);
+    QResizeEvent re(QSize(0, 0), size());
+    resizeEvent(&re);
+#endif
+}
+
 void TextShow::resizeEvent(QResizeEvent *e)
 {
+#ifdef WIN32
+    if (inResize()){
+        if (!m_timer->isActive()){
+            setHScrollBarMode(AlwaysOff);
+            setVScrollBarMode(AlwaysOff);
+            m_timer->start(100);
+        }
+        return;
+    }
+#endif
     QPoint p = QPoint(0, height());
     p = mapToGlobal(p);
     p = viewport()->mapFromGlobal(p);
@@ -477,8 +602,11 @@ void TextShow::resizeEvent(QResizeEvent *e)
         setCursorPosition(para, pos);
         ensureCursorVisible();
     }
+    sync();
+    viewport()->repaint();
 }
 
+
 void TextShow::keyPressEvent(QKeyEvent *e)
 {
     if (((e->state() == Qt::ControlButton) && (e->key() == Qt::Key_C)) ||
@@ -491,100 +619,53 @@ void TextShow::keyPressEvent(QKeyEvent *e)
 
 void TextShow::copy()
 {
-#if QT_VERSION <= 0x030100
-    QApplication::clipboard()->setText(selectedText());
+    QTextDrag *drag = dragObject(NULL);
+    if ( !drag )
+        return;
+#if COMPAT_QT_VERSION <= 0x030100
+    QApplication::clipboard()->setData(drag);
 #else
-    QApplication::clipboard()->setText(selectedText(), QClipboard::Clipboard);
+    QApplication::clipboard()->setData(drag, QClipboard::Clipboard);
 #endif
 }
 
+void TextShow::cut()
+{
+    if (isReadOnly())
+        return;
+    if (hasSelectedText()) {
+        copy();
+        removeSelectedText();
+    }
+}
+
+QTextDrag *TextShow::dragObject(QWidget *parent) const
+{
+    if (!hasSelectedText())
+        return NULL;
+#if (COMPAT_QT_VERSION < 0x030000) || (COMPAT_QT_VERSION >= 0x030100)
+    if (textFormat() == RichText){
+        RichTextDrag *drag = new RichTextDrag(parent);
+        drag->setRichText(selectedText());
+        return drag;
+    }
+#endif
+    return new QTextDrag(selectedText(), parent );
+}
+
 void TextShow::startDrag()
 {
-    QDragObject *drag = new QTextDrag(selectedText(), viewport());
-    if ( isReadOnly() ) {
+    QDragObject *drag = dragObject(viewport());
+    if (drag == NULL)
+        return;
+    if (isReadOnly()) {
         drag->dragCopy();
     } else {
-        if ( drag->drag() && QDragObject::target() != this && QDragObject::target() != viewport() )
+        if (drag->drag() && QDragObject::target() != this && QDragObject::target() != viewport() )
             removeSelectedText();
     }
 }
 
-QString TextShow::selectedText()
-{
-    QString res;
-    int paraFrom, paraTo, indexFrom, indexTo;
-    getSelection(&paraFrom, &indexFrom, &paraTo, &indexTo);
-    return plainText(paraFrom, paraTo, indexFrom, indexTo);
-}
-
-QString TextShow::plainText(int paraFrom, int paraTo, int indexFrom, int indexTo)
-{
-    QString res;
-    if ((paraFrom > paraTo) || ((paraFrom == paraTo) && (indexFrom >= indexTo)))
-        return res;
-    for (int i = paraFrom; i <= paraTo; i++){
-        if (i >= paragraphs())
-            break;
-        res += unquoteString(text(i), (i == paraFrom) ? indexFrom : 0, (i == paraTo) ? indexTo : -1);
-        if ((i < paraTo) && (i < paragraphs()))
-            res += "\n";
-    }
-    return res;
-}
-
-QString TextShow::unquoteString(const QString &s, int from, int to)
-{
-    string ss;
-    if (!s.isEmpty())
-        ss = s.local8Bit();
-    unsigned startPos = textPosition(s, from);
-    unsigned endPos = textPosition(s, to);
-    return SIM::unquoteText(s.mid(startPos, endPos - startPos));
-}
-
-unsigned TextShow::textPosition(const QString &text, unsigned pos)
-{
-    if (pos >= text.length())
-        return text.length();
-    unsigned i;
-    for (i = 0; i < text.length(); i++){
-        QChar c = text[(int)i];
-        if (c == '<'){
-            QString tag = text.mid(i + 1, 3).lower();
-	    int n = tag.find(" ");
-	    if (n > 0)
-		tag = tag.left(n);
-	    n = tag.find("/");
-	    if (n > 0)
-		tag = tag.left(n);
-            if ((tag == "img") || (tag == "br")){
-                if (pos == 0)
-                    return i;
-                pos--;
-            }
-            for (; i < text.length(); i++){
-                c = text[(int)i];
-                if (c == '>')
-                    break;
-            }
-            continue;
-        }
-        if (c == '&'){
-            unsigned n = i;
-            for (; n < text.length(); n++){
-                c = text[(int)n];
-                if (c == ';')
-                    break;
-            }
-	    i = n;
-        }
-        if (pos == 0)
-            return i;
-        pos--;
-    }
-    return i;
-}
-
 QString TextShow::quoteText(const char *t, const char *charset)
 {
     if (t == NULL)
@@ -606,6 +687,20 @@ void TextShow::setText(const QString &text)
     QTextEdit::setText(text, "");
 }
 
+void TextShow::slotSelectionChanged()
+{
+#if COMPAT_QT_VERSION >= 0x030100
+    disconnect(QApplication::clipboard(), SIGNAL(selectionChanged()), this, 0);
+    if (QApplication::clipboard()->supportsSelection()){
+        QTextDrag *drag = dragObject(NULL);
+        if ( !drag )
+            return;
+        QApplication::clipboard()->setData(drag, QClipboard::Selection);
+        connect( QApplication::clipboard(), SIGNAL(selectionChanged()), this, SLOT(clipboardChanged()));
+    }
+#endif
+}
+
 class BgColorParser : public HTMLParser
 {
 public:
