@@ -144,6 +144,11 @@ CToolButton::~CToolButton()
     emit buttonDestroyed();
 }
 
+void CToolButton::setTextLabel(const QString &text)
+{
+    m_text = text;
+}
+
 void CToolButton::setTextLabel()
 {
     QString text = m_text;
@@ -174,7 +179,7 @@ void CToolButton::setState()
     if (m_def.icon_on && strcmp(m_def.icon, m_def.icon_on)){
         const QIconSet *offIcon = Icon(m_def.icon);
         if (offIcon){
-#if QT_VERSION < 300
+#if COMPAT_QT_VERSION < 0x030000
             setIconSet(*offIcon);
             const QIconSet *onIcon = Icon(m_def.icon_on);
             if (onIcon)
@@ -247,7 +252,9 @@ QPoint CToolButton::popupPos(QWidget *btn, QWidget *p)
             break;
         }
     }
+    QWidget *desktop = qApp->desktop();
     QSize s = p->sizeHint();
+    s = p->sizeHint();
     if (bar){
         if  (bar->orientation() == Vertical){
             pos = QPoint(btn->width(), 0);
@@ -258,17 +265,22 @@ QPoint CToolButton::popupPos(QWidget *btn, QWidget *p)
         pos = QPoint(btn->width() - s.width(), btn->height());
     }
     pos = btn->mapToGlobal(pos);
-    QWidget *desktop = qApp->desktop();
-    if (pos.x() + s.width() > desktop->width())
+    if (pos.x() + s.width() > desktop->width()){
         pos.setX(desktop->width() - s.width());
-    if (pos.y() + s.height() > desktop->height())
+        if (pos.x() < 0)
+            pos.setX(0);
+    }
+    if (pos.y() + s.height() > desktop->height()){
         pos.setY(pos.y() - btn->height() - s.height());
+        if (pos.y() < 0)
+            pos.setY(0);
+    }
     return pos;
 }
 
 void CToolButton::mousePressEvent(QMouseEvent *e)
 {
-#if QT_VERSION < 300
+#if COMPAT_QT_VERSION < 0x030000
     if (e->button() == RightButton){
         emit showPopup(e->globalPos());
         return;
@@ -399,25 +411,40 @@ void PictButton::paintEvent(QPaintEvent*)
             p.fillRect(0, 0, width(), height(), colorGroup().button());
         }
     }
-#if QT_VERSION < 300
+#if COMPAT_QT_VERSION < 0x030000
     style().drawToolButton(this, &p);
 #else
     drawButton(&p);
 #endif
+    int w = 4;
     QRect rc(4, 4, width() - 4, height() - 4);
     if (m_def.icon && strcmp(m_def.icon, "empty")){
         const QIconSet *icons = Icon(m_def.icon);
-        const QPixmap &pict = icons->pixmap(QIconSet::Small, isEnabled() ? QIconSet::Active : QIconSet::Disabled);
+        if (icons){
+            const QPixmap &pict = icons->pixmap(QIconSet::Small, isEnabled() ? QIconSet::Active : QIconSet::Disabled);
+            QToolBar *bar = static_cast<QToolBar*>(parent());
+            if (bar->orientation() == Vertical){
+                p.drawPixmap((width() - pict.width()) / 2, 4, pict);
+                QWMatrix m;
+                m.rotate(90);
+                p.setWorldMatrix(m);
+                rc = QRect(8 + pict.height(), -4, height() - 4, 4 - width());
+                w = pict.height() + 4;
+            }else{
+                p.drawPixmap(4, (height()  - pict.height()) / 2, pict);
+                rc = QRect(8 + pict.width(), 4, width() - 4, height() - 4);
+                w = pict.width() + 4;
+            }
+        }
+    }else{
         QToolBar *bar = static_cast<QToolBar*>(parent());
         if (bar->orientation() == Vertical){
-            p.drawPixmap((width() - pict.width()) / 2, 4, pict);
             QWMatrix m;
             m.rotate(90);
             p.setWorldMatrix(m);
-            rc = QRect(8 + pict.height(), -4, height() - 4, 4 - width());
+            rc = QRect(4, -4, height() - 4, 4 - width());
         }else{
-            p.drawPixmap(4, (height()  - pict.height()) / 2, pict);
-            rc = QRect(8 + pict.width(), 4, width() - 4, height() - 4);
+            rc = QRect(4, 4, width() - 4, height() - 4);
         }
     }
     const QColorGroup &cg = isEnabled() ? palette().active() : palette().disabled();
@@ -425,7 +452,23 @@ void PictButton::paintEvent(QPaintEvent*)
     QString text = m_text;
     if (text.isEmpty())
         text = i18n(m_def.text);
-    p.drawText(rc, AlignLeft | AlignVCenter | ShowPrefix, text);
+    if ((m_def.flags & BTN_DIV) && (text.find(" | ") >= 0)){
+        QStringList parts = QStringList::split(" | ", text);
+        unsigned n;
+        for (n = parts.count(); n > 0; n--){
+            text = "";
+            for (unsigned i = 0; i < n; i++){
+                if (!text.isEmpty())
+                    text += " ";
+                text += parts[i];
+            }
+            QRect rcb(0, 0, qApp->desktop()->width(), qApp->desktop()->height());
+            rcb = p.boundingRect(rcb, AlignLeft | ShowPrefix | SingleLine, text);
+            if (rcb.width() + w < rc.width())
+                break;
+        }
+    }
+    p.drawText(rc, AlignLeft | AlignVCenter | ShowPrefix | SingleLine, text);
     p.end();
     p.begin(this);
     p.drawPixmap(0, 0, pict);
@@ -547,7 +590,7 @@ void CToolBar::checkState()
 
 void CToolBar::mousePressEvent(QMouseEvent *e)
 {
-#if QT_VERSION < 300
+#if COMPAT_QT_VERSION < 0x030000
     if (e->button() == RightButton){
         showPopup(e->globalPos());
         return;
